@@ -3,12 +3,7 @@ from __future__ import annotations
 import functools
 import inspect
 import sys
-from typing import Any, Callable, ForwardRef, TypeVar, Union
-
-if sys.version_info < (3, 8):
-    from typing_extensions import get_args, get_origin
-else:
-    from typing import get_args, get_origin
+from typing import Any, Callable, ForwardRef, TypeVar, Union, get_args, get_origin
 
 if sys.version_info < (3, 10):
     from typing_extensions import ParamSpec
@@ -38,6 +33,7 @@ def PublicAPI(func: Callable[P, T]) -> Callable[P, T]:
 
     @functools.wraps(func)
     def _wrap(*args: P.args, **kwargs: P.kwargs) -> T:
+        __tracebackhide__ = True
         type_check_function(func, *args, **kwargs)
         timed_func = time_func(func)
         return timed_func(*args, **kwargs)
