@@ -2,40 +2,27 @@ from __future__ import annotations
 
 import itertools
 import pathlib
-import sys
 from dataclasses import dataclass, field
-from typing import Generic
+from typing import TYPE_CHECKING, Generic, Protocol
 
-if sys.version_info < (3, 8):
-    from typing_extensions import Protocol
-else:
-    from typing import Protocol
-
-from daft.daft import (
-    CsvSourceConfig,
-    FileFormat,
-    FileFormatConfig,
-    IOConfig,
-    JoinType,
-    JsonReadOptions,
-    JsonSourceConfig,
-    ParquetSourceConfig,
-    ResourceRequest,
-    StorageConfig,
-)
+from daft.daft import FileFormat, IOConfig, JoinType, ResourceRequest, ScanTask
 from daft.expressions import Expression, ExpressionsProjection, col
 from daft.logical.map_partition_ops import MapPartitionOp
 from daft.logical.schema import Schema
 from daft.runners.partitioning import (
+    Boundaries,
     MaterializedResult,
     PartialPartitionMetadata,
     PartitionMetadata,
     PartitionT,
-    TableParseCSVOptions,
-    TableReadOptions,
 )
 from daft.table import MicroPartition, table_io
 
+if TYPE_CHECKING:
+    from pyiceberg.schema import Schema as IcebergSchema
+    from pyiceberg.table import TableProperties as IcebergTableProperties
+
+
 ID_GEN = itertools.count()
 
 
@@ -54,6 +41,7 @@ class PartitionTask(Generic[PartitionT]):
     resource_request: ResourceRequest
     num_results: int
     stage_id: int
+    partial_metadatas: list[PartialPartitionMetadata]
     _id: int = field(default_factory=lambda: next(ID_GEN))
 
     def id(self) -> str:
@@ -71,6 +59,10 @@ class PartitionTask(Generic[PartitionT]):
         """Set the result of this Task. For use by the Task executor."""
         raise NotImplementedError
 
+    def is_empty(self) -> bool:
+        """Whether this partition task is guaranteed to result in an empty partition."""
+        return len(self.partial_metadatas) > 0 and all(meta.num_rows == 0 for meta in self.partial_metadatas)
+
     def __str__(self) -> str:
         return (
             f"{self.id()}\n"
@@ -116,6 +108,10 @@ class PartitionTaskBuilder(Generic[PartitionT]):
         self.num_results = instruction.num_outputs()
         return self
 
+    def is_empty(self) -> bool:
+        """Whether this partition task is guaranteed to result in an empty partition."""
+        return len(self.partial_metadatas) > 0 and all(meta.num_rows == 0 for meta in self.partial_metadatas)
+
     def finalize_partition_task_single_output(self, stage_id: int) -> SingleOutputPartitionTask[PartitionT]:
         """Create a SingleOutputPartitionTask from this PartitionTaskBuilder.
 
@@ -135,6 +131,7 @@ class PartitionTaskBuilder(Generic[PartitionT]):
             instructions=self.instructions,
             num_results=1,
             resource_request=resource_request_final_cpu,
+            partial_metadatas=self.partial_metadatas,
         )
 
     def finalize_partition_task_multi_output(self, stage_id: int) -> MultiOutputPartitionTask[PartitionT]:
@@ -154,6 +151,7 @@ class PartitionTaskBuilder(Generic[PartitionT]):
             instructions=self.instructions,
             num_results=self.num_results,
             resource_request=resource_request_final_cpu,
+            partial_metadatas=self.partial_metadatas,
         )
 
     def __str__(self) -> str:
@@ -198,7 +196,8 @@ class SingleOutputPartitionTask(PartitionTask[PartitionT]):
 
         (Avoids retrieving the actual partition itself if possible.)
         """
-        return self.result().metadata()
+        [partial_metadata] = self.partial_metadatas
+        return self.result().metadata().merge_with_partial(partial_metadata)
 
     def vpartition(self) -> MicroPartition:
         """Get the raw vPartition of the result."""
@@ -243,7 +242,10 @@ class MultiOutputPartitionTask(PartitionTask[PartitionT]):
         (Avoids retrieving the actual partition itself if possible.)
         """
         assert self._results is not None
-        return [result.metadata() for result in self._results]
+        return [
+            result.metadata().merge_with_partial(partial_metadata)
+            for result, partial_metadata in zip(self._results, self.partial_metadatas)
+        ]
 
     def vpartition(self, index: int) -> MicroPartition:
         """Get the raw vPartition of the result."""
@@ -263,7 +265,7 @@ class Instruction(Protocol):
     Most instructions take one partition and return another partition.
     However, some instructions take one partition and return many partitions (fanouts),
     and others take many partitions and return one partition (reduces).
-    To accomodate these, instructions are typed as list[Table] -> list[Table].
+    To accommodate these, instructions are typed as list[Table] -> list[Table].
     """
 
     def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
@@ -288,125 +290,44 @@ class SingleOutputInstruction(Instruction):
 
 
 @dataclass(frozen=True)
-class ReadFile(SingleOutputInstruction):
-    index: int | None
-    # Known number of rows.
-    file_rows: int | None
-    # Max number of rows to read.
-    limit_rows: int | None
-    schema: Schema
-    storage_config: StorageConfig
-    columns_to_read: list[str] | None
-    file_format_config: FileFormatConfig
+class ScanWithTask(SingleOutputInstruction):
+    scan_task: ScanTask
 
     def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
-        return self._read_file(inputs)
+        return self._scan(inputs)
 
-    def _read_file(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
-        assert len(inputs) == 1
-        [filepaths_partition] = inputs
-        partition = self._handle_tabular_files_scan(
-            filepaths_partition=filepaths_partition,
-        )
-        return [partition]
+    def _scan(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        assert len(inputs) == 0
+        table = MicroPartition._from_scan_task(self.scan_task)
+        return [table]
 
     def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
-        assert len(input_metadatas) == 1
-
-        num_rows = self.file_rows
-        # Only take the file read limit into account if we know how big the file is to begin with.
-        if num_rows is not None and self.limit_rows is not None:
-            num_rows = min(num_rows, self.limit_rows)
+        assert len(input_metadatas) == 0
 
         return [
             PartialPartitionMetadata(
-                num_rows=num_rows,
-                size_bytes=None,
+                num_rows=self.scan_task.num_rows(),
+                size_bytes=self.scan_task.size_bytes(),
             )
         ]
 
-    def _handle_tabular_files_scan(
-        self,
-        filepaths_partition: MicroPartition,
-    ) -> MicroPartition:
-        data = filepaths_partition.to_pydict()
-        filepaths = data["path"]
 
-        if self.index is not None:
-            filepaths = [filepaths[self.index]]
+@dataclass(frozen=True)
+class EmptyScan(SingleOutputInstruction):
+    schema: Schema
 
-        # Common options for reading vPartition
-        read_options = TableReadOptions(
-            num_rows=self.limit_rows,
-            column_names=self.columns_to_read,  # read only specified columns
-        )
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        return [MicroPartition.empty(self.schema)]
 
-        file_format = self.file_format_config.file_format()
-        format_config = self.file_format_config.config
-        if file_format == FileFormat.Csv:
-            assert isinstance(format_config, CsvSourceConfig)
-            table = MicroPartition.concat(
-                [
-                    table_io.read_csv(
-                        file=fp,
-                        schema=self.schema,
-                        storage_config=self.storage_config,
-                        csv_options=TableParseCSVOptions(
-                            delimiter=format_config.delimiter,
-                            header_index=0 if format_config.has_headers else None,
-                            double_quote=format_config.double_quote,
-                            quote=format_config.quote,
-                            escape_char=format_config.escape_char,
-                            comment=format_config.comment,
-                            buffer_size=format_config.buffer_size,
-                            chunk_size=format_config.chunk_size,
-                        ),
-                        read_options=read_options,
-                    )
-                    for fp in filepaths
-                ]
+    def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
+        assert len(input_metadatas) == 0
+
+        return [
+            PartialPartitionMetadata(
+                num_rows=0,
+                size_bytes=0,
             )
-        elif file_format == FileFormat.Json:
-            assert isinstance(format_config, JsonSourceConfig)
-            table = MicroPartition.concat(
-                [
-                    table_io.read_json(
-                        file=fp,
-                        schema=self.schema,
-                        storage_config=self.storage_config,
-                        json_read_options=JsonReadOptions(
-                            buffer_size=format_config.buffer_size, chunk_size=format_config.chunk_size
-                        ),
-                        read_options=read_options,
-                    )
-                    for fp in filepaths
-                ]
-            )
-        elif file_format == FileFormat.Parquet:
-            assert isinstance(format_config, ParquetSourceConfig)
-            table = MicroPartition.concat(
-                [
-                    table_io.read_parquet(
-                        file=fp,
-                        schema=self.schema,
-                        storage_config=self.storage_config,
-                        read_options=read_options,
-                    )
-                    for fp in filepaths
-                ]
-            )
-        else:
-            raise NotImplementedError(f"PyRunner has not implemented scan: {file_format}")
-
-        expected_schema = (
-            Schema._from_fields([self.schema[name] for name in read_options.column_names])
-            if read_options.column_names is not None
-            else self.schema
-        )
-        assert (
-            table.schema() == expected_schema
-        ), f"Expected table to have schema:\n{expected_schema}\n\nReceived instead:\n{table.schema()}"
-        return table
+        ]
 
 
 @dataclass(frozen=True)
@@ -432,38 +353,94 @@ class WriteFile(SingleOutputInstruction):
         assert len(input_metadatas) == 1
         return [
             PartialPartitionMetadata(
-                num_rows=1,  # We currently write one file per partition.
+                num_rows=None,  # we can write more than 1 file per partition
                 size_bytes=None,
             )
         ]
 
     def _handle_file_write(self, input: MicroPartition) -> MicroPartition:
-        if self.file_format == FileFormat.Parquet:
-            file_names = table_io.write_parquet(
-                input,
-                path=self.root_dir,
-                compression=self.compression,
-                partition_cols=self.partition_cols,
-                io_config=self.io_config,
-            )
-        elif self.file_format == FileFormat.Csv:
-            file_names = table_io.write_csv(
-                input,
-                path=self.root_dir,
-                compression=self.compression,
-                partition_cols=self.partition_cols,
-                io_config=self.io_config,
-            )
-        else:
-            raise ValueError(
-                f"Only Parquet and CSV file formats are supported for writing, but got: {self.file_format}"
-            )
+        return table_io.write_tabular(
+            input,
+            path=self.root_dir,
+            schema=self.schema,
+            file_format=self.file_format,
+            compression=self.compression,
+            partition_cols=self.partition_cols,
+            io_config=self.io_config,
+        )
 
-        assert len(self.schema) == 1
-        return MicroPartition.from_pydict(
-            {
-                self.schema.column_names()[0]: file_names,
-            }
+
+@dataclass(frozen=True)
+class WriteIceberg(SingleOutputInstruction):
+    base_path: str
+    iceberg_schema: IcebergSchema
+    iceberg_properties: IcebergTableProperties
+    spec_id: int
+    io_config: IOConfig | None
+
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        return self._write_iceberg(inputs)
+
+    def _write_iceberg(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        [input] = inputs
+        partition = self._handle_file_write(
+            input=input,
+        )
+        return [partition]
+
+    def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
+        assert len(input_metadatas) == 1
+        return [
+            PartialPartitionMetadata(
+                num_rows=None,  # we can write more than 1 file per partition
+                size_bytes=None,
+            )
+        ]
+
+    def _handle_file_write(self, input: MicroPartition) -> MicroPartition:
+        return table_io.write_iceberg(
+            input,
+            base_path=self.base_path,
+            schema=self.iceberg_schema,
+            properties=self.iceberg_properties,
+            spec_id=self.spec_id,
+            io_config=self.io_config,
+        )
+
+
+@dataclass(frozen=True)
+class WriteDeltaLake(SingleOutputInstruction):
+    base_path: str
+    large_dtypes: bool
+    current_version: int
+    io_config: IOConfig | None
+
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        return self._write_deltalake(inputs)
+
+    def _write_deltalake(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        [input] = inputs
+        partition = self._handle_file_write(
+            input=input,
+        )
+        return [partition]
+
+    def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
+        assert len(input_metadatas) == 1
+        return [
+            PartialPartitionMetadata(
+                num_rows=None,  # we can write more than 1 file per partition
+                size_bytes=None,
+            )
+        ]
+
+    def _handle_file_write(self, input: MicroPartition) -> MicroPartition:
+        return table_io.write_deltalake(
+            input,
+            large_dtypes=self.large_dtypes,
+            base_path=self.base_path,
+            current_version=self.current_version,
+            io_config=self.io_config,
         )
 
 
@@ -479,11 +456,12 @@ class Filter(SingleOutputInstruction):
         return [input.filter(self.predicate)]
 
     def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
-        # Can't derive anything.
+        [input_meta] = input_metadatas
         return [
             PartialPartitionMetadata(
                 num_rows=None,
                 size_bytes=None,
+                boundaries=input_meta.boundaries,
             )
         ]
 
@@ -501,14 +479,38 @@ class Project(SingleOutputInstruction):
 
     def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
         [input_meta] = input_metadatas
+        boundaries = input_meta.boundaries
+        if boundaries is not None:
+            boundaries = _prune_boundaries(boundaries, self.projection)
         return [
             PartialPartitionMetadata(
                 num_rows=input_meta.num_rows,
                 size_bytes=None,
+                boundaries=boundaries,
             )
         ]
 
 
+def _prune_boundaries(boundaries: Boundaries, projection: ExpressionsProjection) -> Boundaries | None:
+    """
+    If projection expression is a nontrivial computation (i.e. not a direct col() reference and not an alias) on top of a boundary
+    expression, then invalidate the boundary.
+    """
+    proj_all_names = projection.to_name_set()
+    proj_names_needing_compute = proj_all_names - projection.input_mapping().keys()
+    for i, e in enumerate(boundaries.sort_by):
+        if e.name() in proj_names_needing_compute:
+            # Found a sort expression that is no longer valid, so we invalidate that sort expression and all that follow it.
+            sort_by = boundaries.sort_by[:i]
+            if not sort_by:
+                return None
+            boundaries_ = boundaries.bounds.eval_expression_list(
+                ExpressionsProjection([col(e.name()) for e in sort_by])
+            )
+            return Boundaries(sort_by, boundaries_)
+    return boundaries
+
+
 @dataclass(frozen=True)
 class LocalCount(SingleOutputInstruction):
     schema: Schema
@@ -548,6 +550,7 @@ class LocalLimit(SingleOutputInstruction):
             PartialPartitionMetadata(
                 num_rows=(min(self.limit, input_meta.num_rows) if input_meta.num_rows is not None else None),
                 size_bytes=None,
+                boundaries=input_meta.boundaries,
             )
         ]
 
@@ -611,6 +614,30 @@ class Sample(SingleOutputInstruction):
         ]
 
 
+@dataclass(frozen=True)
+class MonotonicallyIncreasingId(SingleOutputInstruction):
+    partition_num: int
+    column_name: str
+
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        [input] = inputs
+        result = input.add_monotonically_increasing_id(self.partition_num, self.column_name)
+        return [result]
+
+    def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
+        [input_meta] = input_metadatas
+        return [
+            PartialPartitionMetadata(
+                num_rows=input_meta.num_rows,
+                size_bytes=(
+                    (input_meta.size_bytes + input_meta.num_rows * 8)  # 8 bytes per uint64
+                    if input_meta.size_bytes is not None and input_meta.num_rows is not None
+                    else None
+                ),
+            )
+        ]
+
+
 @dataclass(frozen=True)
 class Aggregate(SingleOutputInstruction):
     to_agg: list[Expression]
@@ -634,16 +661,63 @@ class Aggregate(SingleOutputInstruction):
 
 
 @dataclass(frozen=True)
-class Join(SingleOutputInstruction):
+class Pivot(SingleOutputInstruction):
+    group_by: ExpressionsProjection
+    pivot_col: Expression
+    value_col: Expression
+    names: list[str]
+
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        return self._pivot(inputs)
+
+    def _pivot(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        [input] = inputs
+        return [input.pivot(self.group_by, self.pivot_col, self.value_col, self.names)]
+
+    def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
+        return [
+            PartialPartitionMetadata(
+                num_rows=None,
+                size_bytes=None,
+            )
+        ]
+
+
+@dataclass(frozen=True)
+class Unpivot(SingleOutputInstruction):
+    ids: ExpressionsProjection
+    values: ExpressionsProjection
+    variable_name: str
+    value_name: str
+
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        return self._unpivot(inputs)
+
+    def _unpivot(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        [input] = inputs
+        return [input.unpivot(self.ids, self.values, self.variable_name, self.value_name)]
+
+    def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
+        [input_meta] = input_metadatas
+        return [
+            PartialPartitionMetadata(
+                num_rows=None if input_meta.num_rows is None else input_meta.num_rows * len(self.values),
+                size_bytes=None,
+            )
+        ]
+
+
+@dataclass(frozen=True)
+class HashJoin(SingleOutputInstruction):
     left_on: ExpressionsProjection
     right_on: ExpressionsProjection
     how: JoinType
     is_swapped: bool
 
     def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
-        return self._join(inputs)
+        return self._hash_join(inputs)
 
-    def _join(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+    def _hash_join(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         # All inputs except for the last are the left side of the join, in order to support left-broadcasted joins.
         *lefts, right = inputs
         if len(lefts) > 1:
@@ -655,7 +729,7 @@ class Join(SingleOutputInstruction):
             # Swap left/right back.
             # We don't need to swap left_on and right_on since those were never swapped in the first place.
             left, right = right, left
-        result = left.join(
+        result = left.hash_join(
             right,
             left_on=self.left_on,
             right_on=self.right_on,
@@ -673,8 +747,48 @@ class Join(SingleOutputInstruction):
         ]
 
 
-class ReduceInstruction(SingleOutputInstruction):
-    ...
+@dataclass(frozen=True)
+class BroadcastJoin(HashJoin): ...
+
+
+@dataclass(frozen=True)
+class MergeJoin(SingleOutputInstruction):
+    left_on: ExpressionsProjection
+    right_on: ExpressionsProjection
+    how: JoinType
+    preserve_left_bounds: bool
+
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        return self._join(inputs)
+
+    def _join(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        left, right = inputs
+        result = left.sort_merge_join(
+            right,
+            left_on=self.left_on,
+            right_on=self.right_on,
+            how=self.how,
+            is_sorted=True,
+        )
+        return [result]
+
+    def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
+        [left_meta, right_meta] = input_metadatas
+        # If the boundaries of the left and right partitions don't intersect, then the merge-join will result in an empty partition.
+        if left_meta.boundaries is None or right_meta.boundaries is None:
+            is_nonempty = True
+        else:
+            is_nonempty = left_meta.boundaries.intersects(right_meta.boundaries)
+        return [
+            PartialPartitionMetadata(
+                num_rows=None if is_nonempty else 0,
+                size_bytes=None,
+                boundaries=(left_meta.boundaries if self.preserve_left_bounds else right_meta.boundaries),
+            )
+        ]
+
+
+class ReduceInstruction(SingleOutputInstruction): ...
 
 
 @dataclass(frozen=True)
@@ -690,8 +804,8 @@ class ReduceMerge(ReduceInstruction):
         input_sizes = [_.size_bytes for _ in input_metadatas]
         return [
             PartialPartitionMetadata(
-                num_rows=sum(input_rows) if all(_ is not None for _ in input_rows) else None,
-                size_bytes=sum(input_sizes) if all(_ is not None for _ in input_sizes) else None,
+                num_rows=(sum(input_rows) if all(_ is not None for _ in input_rows) else None),
+                size_bytes=(sum(input_sizes) if all(_ is not None for _ in input_sizes) else None),
             )
         ]
 
@@ -700,6 +814,7 @@ class ReduceMerge(ReduceInstruction):
 class ReduceMergeAndSort(ReduceInstruction):
     sort_by: ExpressionsProjection
     descending: list[bool]
+    bounds: MicroPartition
 
     def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._reduce_merge_and_sort(inputs)
@@ -713,8 +828,9 @@ class ReduceMergeAndSort(ReduceInstruction):
         input_sizes = [_.size_bytes for _ in input_metadatas]
         return [
             PartialPartitionMetadata(
-                num_rows=sum(input_rows) if all(_ is not None for _ in input_rows) else None,
-                size_bytes=sum(input_sizes) if all(_ is not None for _ in input_sizes) else None,
+                num_rows=(sum(input_rows) if all(_ is not None for _ in input_rows) else None),
+                size_bytes=(sum(input_sizes) if all(_ is not None for _ in input_sizes) else None),
+                boundaries=Boundaries(list(self.sort_by), self.bounds),
             )
         ]
 
@@ -852,6 +968,7 @@ class FanoutSlices(FanoutInstruction):
                 PartialPartitionMetadata(
                     num_rows=num_rows,
                     size_bytes=None,
+                    boundaries=input_meta.boundaries,
                 )
             )
 
