@@ -1,27 +1,27 @@
 from __future__ import annotations
 
-from dataclasses import dataclass
-from typing import Iterator, cast
+from typing import TYPE_CHECKING
 
 from daft.daft import (
     FileFormat,
-    FileFormatConfig,
     IOConfig,
     JoinType,
     PyExpr,
     PySchema,
-    PyTable,
     ResourceRequest,
     ScanTask,
-    StorageConfig,
 )
 from daft.execution import execution_step, physical_plan
 from daft.expressions import Expression, ExpressionsProjection
 from daft.logical.map_partition_ops import MapPartitionOp
 from daft.logical.schema import Schema
-from daft.runners.partitioning import PartialPartitionMetadata, PartitionT
+from daft.runners.partitioning import PartitionT
 from daft.table import MicroPartition
 
+if TYPE_CHECKING:
+    from pyiceberg.schema import Schema as IcebergSchema
+    from pyiceberg.table import TableProperties as IcebergTableProperties
+
 
 def scan_with_tasks(
     scan_tasks: list[ScanTask],
@@ -33,8 +33,11 @@ def scan_with_tasks(
     # TODO(Clark): Currently hardcoded to have 1 file per instruction
     # We can instead right-size and bundle the ScanTask into single-instruction bulk reads.
     for scan_task in scan_tasks:
-        scan_step = execution_step.PartitionTaskBuilder[PartitionT](inputs=[], partial_metadatas=None,).add_instruction(
-            instruction=ScanWithTask(scan_task),
+        scan_step = execution_step.PartitionTaskBuilder[PartitionT](
+            inputs=[],
+            partial_metadatas=None,
+        ).add_instruction(
+            instruction=execution_step.ScanWithTask(scan_task),
             # Set the filesize as the memory request.
             # (Note: this is very conservative; file readers empirically use much more peak memory than 1x file size.)
             resource_request=ResourceRequest(memory_bytes=scan_task.size_bytes()),
@@ -46,84 +49,16 @@ def empty_scan(
     schema: Schema,
 ) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
     """yield a plan to create an empty Partition"""
-    scan_step = execution_step.PartitionTaskBuilder[PartitionT](inputs=[], partial_metadatas=None,).add_instruction(
-        instruction=EmptyScan(schema=schema),
+    scan_step = execution_step.PartitionTaskBuilder[PartitionT](
+        inputs=[],
+        partial_metadatas=None,
+    ).add_instruction(
+        instruction=execution_step.EmptyScan(schema=schema),
         resource_request=ResourceRequest(memory_bytes=0),
     )
     yield scan_step
 
 
-@dataclass(frozen=True)
-class ScanWithTask(execution_step.SingleOutputInstruction):
-    scan_task: ScanTask
-
-    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
-        return self._scan(inputs)
-
-    def _scan(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
-        assert len(inputs) == 0
-        table = MicroPartition._from_scan_task(self.scan_task)
-        return [table]
-
-    def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
-        assert len(input_metadatas) == 0
-
-        return [
-            PartialPartitionMetadata(
-                num_rows=self.scan_task.num_rows(),
-                size_bytes=self.scan_task.size_bytes(),
-            )
-        ]
-
-
-@dataclass(frozen=True)
-class EmptyScan(execution_step.SingleOutputInstruction):
-    schema: Schema
-
-    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
-        return [MicroPartition.empty(self.schema)]
-
-    def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
-        assert len(input_metadatas) == 0
-
-        return [
-            PartialPartitionMetadata(
-                num_rows=0,
-                size_bytes=0,
-            )
-        ]
-
-
-def tabular_scan(
-    schema: PySchema,
-    columns_to_read: list[str] | None,
-    file_info_table: PyTable,
-    file_format_config: FileFormatConfig,
-    storage_config: StorageConfig,
-    limit: int,
-    is_ray_runner: bool,
-) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
-    # TODO(Clark): Fix this Ray runner hack.
-    part = MicroPartition._from_pytable(file_info_table)
-    if is_ray_runner:
-        import ray
-
-        parts = [ray.put(part)]
-    else:
-        parts = [part]
-    parts_t = cast(Iterator[PartitionT], parts)
-
-    file_info_iter = physical_plan.partition_read(iter(parts_t))
-    return physical_plan.file_read(
-        child_plan=file_info_iter,
-        limit_rows=limit,
-        schema=Schema._from_pyschema(schema),
-        storage_config=storage_config,
-        columns_to_read=columns_to_read,
-        file_format_config=file_format_config,
-    )
-
-
 def project(
     input: physical_plan.InProgressPhysicalPlan[PartitionT], projection: list[PyExpr], resource_request: ResourceRequest
 ) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
@@ -162,6 +97,30 @@ def explode(
     )
 
 
+def unpivot(
+    input: physical_plan.InProgressPhysicalPlan[PartitionT],
+    ids: list[PyExpr],
+    values: list[PyExpr],
+    variable_name: str,
+    value_name: str,
+) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
+    ids_projection = ExpressionsProjection([Expression._from_pyexpr(expr) for expr in ids])
+    values_projection = ExpressionsProjection([Expression._from_pyexpr(expr) for expr in values])
+
+    unpivot_step = execution_step.Unpivot(
+        ids=ids_projection,
+        values=values_projection,
+        variable_name=variable_name,
+        value_name=value_name,
+    )
+
+    return physical_plan.pipeline_instruction(
+        child_plan=input,
+        pipeable_instruction=unpivot_step,
+        resource_request=ResourceRequest(),
+    )
+
+
 def local_aggregate(
     input: physical_plan.InProgressPhysicalPlan[PartitionT],
     agg_exprs: list[PyExpr],
@@ -179,6 +138,27 @@ def local_aggregate(
     )
 
 
+def pivot(
+    input: physical_plan.InProgressPhysicalPlan[PartitionT],
+    group_by: list[PyExpr],
+    pivot_col: PyExpr,
+    value_col: PyExpr,
+    names: list[str],
+) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
+    pivot_step = execution_step.Pivot(
+        group_by=ExpressionsProjection([Expression._from_pyexpr(pyexpr) for pyexpr in group_by]),
+        pivot_col=Expression._from_pyexpr(pivot_col),
+        value_col=Expression._from_pyexpr(value_col),
+        names=names,
+    )
+
+    return physical_plan.pipeline_instruction(
+        child_plan=input,
+        pipeable_instruction=pivot_step,
+        resource_request=ResourceRequest(),
+    )
+
+
 def sample(
     input: physical_plan.InProgressPhysicalPlan[PartitionT], fraction: float, with_replacement: bool, seed: int | None
 ) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
@@ -246,6 +226,48 @@ def hash_join(
     )
 
 
+def merge_join_sorted(
+    input: physical_plan.InProgressPhysicalPlan[PartitionT],
+    right: physical_plan.InProgressPhysicalPlan[PartitionT],
+    left_on: list[PyExpr],
+    right_on: list[PyExpr],
+    join_type: JoinType,
+    left_is_larger: bool,
+) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
+    left_on_expr_proj = ExpressionsProjection([Expression._from_pyexpr(expr) for expr in left_on])
+    right_on_expr_proj = ExpressionsProjection([Expression._from_pyexpr(expr) for expr in right_on])
+    return physical_plan.merge_join_sorted(
+        left_plan=input,
+        right_plan=right,
+        left_on=left_on_expr_proj,
+        right_on=right_on_expr_proj,
+        how=join_type,
+        left_is_larger=left_is_larger,
+    )
+
+
+def sort_merge_join_aligned_boundaries(
+    input: physical_plan.InProgressPhysicalPlan[PartitionT],
+    right: physical_plan.InProgressPhysicalPlan[PartitionT],
+    left_on: list[PyExpr],
+    right_on: list[PyExpr],
+    join_type: JoinType,
+    num_partitions: int,
+    left_is_larger: bool,
+) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
+    left_on_expr_proj = ExpressionsProjection([Expression._from_pyexpr(expr) for expr in left_on])
+    right_on_expr_proj = ExpressionsProjection([Expression._from_pyexpr(expr) for expr in right_on])
+    return physical_plan.sort_merge_join_aligned_boundaries(
+        left_plan=input,
+        right_plan=right,
+        left_on=left_on_expr_proj,
+        right_on=right_on_expr_proj,
+        how=join_type,
+        num_partitions=num_partitions,
+        left_is_larger=left_is_larger,
+    )
+
+
 def broadcast_join(
     broadcaster: physical_plan.InProgressPhysicalPlan[PartitionT],
     receiver: physical_plan.InProgressPhysicalPlan[PartitionT],
@@ -288,3 +310,37 @@ def write_file(
         expr_projection,
         io_config,
     )
+
+
+def write_iceberg(
+    input: physical_plan.InProgressPhysicalPlan[PartitionT],
+    base_path: str,
+    iceberg_schema: IcebergSchema,
+    iceberg_properties: IcebergTableProperties,
+    spec_id: int,
+    io_config: IOConfig | None,
+) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
+    return physical_plan.iceberg_write(
+        input,
+        base_path=base_path,
+        iceberg_schema=iceberg_schema,
+        iceberg_properties=iceberg_properties,
+        spec_id=spec_id,
+        io_config=io_config,
+    )
+
+
+def write_deltalake(
+    input: physical_plan.InProgressPhysicalPlan[PartitionT],
+    path: str,
+    large_dtypes: bool,
+    current_version: int,
+    io_config: IOConfig | None,
+) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
+    return physical_plan.deltalake_write(
+        input,
+        path,
+        large_dtypes,
+        current_version,
+        io_config,
+    )
