@@ -1,17 +1,11 @@
 from __future__ import annotations
 
 import dataclasses
+import logging
 import pathlib
 import sys
 import urllib.parse
-
-if sys.version_info < (3, 8):
-    from typing_extensions import Literal
-else:
-    from typing import Literal
-
-import logging
-from typing import Any
+from typing import Any, Literal
 
 import fsspec
 from fsspec.registry import get_filesystem_class
@@ -78,7 +72,8 @@ def get_filesystem(protocol: str, **kwargs) -> fsspec.AbstractFileSystem:
         klass = fsspec.get_filesystem_class(protocol)
     except ImportError:
         logger.error(
-            f"Error when importing dependencies for accessing data with: {protocol}. Please ensure that getdaft was installed with the appropriate extra dependencies (https://www.getdaft.io/projects/docs/en/latest/learn/install.html)"
+            "Error when importing dependencies for accessing data with: %s. Please ensure that getdaft was installed with the appropriate extra dependencies (https://www.getdaft.io/projects/docs/en/latest/learn/install.html)",
+            protocol,
         )
         raise
 
@@ -138,6 +133,10 @@ def _resolve_paths_and_filesystem(
     assert all(isinstance(p, str) for p in paths), paths
     assert len(paths) > 0, paths
 
+    # Sanitize s3a/s3n protocols, which are produced by Hadoop-based systems as a way of denoting which s3
+    # filesystem client to use. However this doesn't matter for Daft, and PyArrow cannot recognize these protocols.
+    paths = [f"s3://{p[6:]}" if p.startswith("s3a://") or p.startswith("s3n://") else p for p in paths]
+
     # Ensure that protocols for all paths are consistent, i.e. that they would map to the
     # same filesystem.
     protocols = {get_protocol_from_path(path) for path in paths}
@@ -269,7 +268,7 @@ def _infer_filesystem(
     ###
     # Azure: Use FSSpec as a fallback
     ###
-    elif protocol in {"az", "abfs"}:
+    elif protocol in {"az", "abfs", "abfss"}:
         fsspec_fs_cls = get_filesystem_class(protocol)
         fsspec_fs = fsspec_fs_cls()
         resolved_filesystem, resolved_path = pafs_resolve_filesystem_and_path(path, fsspec_fs)
