@@ -10,9 +10,14 @@ from daft.daft import (
     set_io_pool_num_threads,
 )
 from daft.io._csv import read_csv
+from daft.io._delta_lake import read_delta_lake
+from daft.io._hudi import read_hudi
 from daft.io._iceberg import read_iceberg
 from daft.io._json import read_json
+from daft.io._lance import read_lance
 from daft.io._parquet import read_parquet
+from daft.io._sql import read_sql
+from daft.io.catalog import DataCatalogTable, DataCatalogType
 from daft.io.file_path import from_glob_path
 
 
@@ -35,10 +40,16 @@ __all__ = [
     "read_json",
     "from_glob_path",
     "read_parquet",
+    "read_hudi",
     "read_iceberg",
+    "read_delta_lake",
+    "read_lance",
+    "read_sql",
     "IOConfig",
     "S3Config",
     "AzureConfig",
     "GCSConfig",
     "set_io_pool_num_threads",
+    "DataCatalogType",
+    "DataCatalogTable",
 ]
