@@ -14,7 +14,7 @@ from daft.daft import (
 )
 from daft.dataframe import DataFrame
 from daft.datatype import DataType
-from daft.io.common import _get_tabular_files_scan
+from daft.io.common import get_tabular_files_scan
 
 
 @PublicAPI
@@ -36,8 +36,8 @@ def read_json(
 
     Args:
         path (str): Path to JSON files (allows for wildcards)
-        schema_hints (dict[str, DataType]): A mapping between column names and datatypes - passing this option will
-            disable all schema inference on data being read, and throw an error if data being read is incompatible.
+        schema_hints (dict[str, DataType]): A mapping between column names and datatypes - passing this option
+            will override the specified columns on the inferred schema with the specified DataTypes
         io_config (IOConfig): Config to be used with the native downloader
         use_native_downloader: Whether to use the native downloader instead of PyArrow for reading Parquet. This
             is currently experimental.
@@ -46,7 +46,7 @@ def read_json(
         DataFrame: parsed DataFrame
     """
     if isinstance(path, list) and len(path) == 0:
-        raise ValueError(f"Cannot read DataFrame from from empty list of JSON filepaths")
+        raise ValueError("Cannot read DataFrame from from empty list of JSON filepaths")
 
     io_config = context.get_context().daft_planning_config.default_io_config if io_config is None else io_config
 
@@ -56,5 +56,5 @@ def read_json(
         storage_config = StorageConfig.native(NativeStorageConfig(True, io_config))
     else:
         storage_config = StorageConfig.python(PythonStorageConfig(io_config=io_config))
-    builder = _get_tabular_files_scan(path, schema_hints, file_format_config, storage_config=storage_config)
+    builder = get_tabular_files_scan(path, schema_hints, file_format_config, storage_config=storage_config)
     return DataFrame(builder)
