@@ -6,25 +6,22 @@ from typing import TYPE_CHECKING
 from daft.daft import (
     CountMode,
     FileFormat,
-    FileFormatConfig,
-    FileInfos,
     IOConfig,
+    JoinStrategy,
     JoinType,
-)
-from daft.daft import LogicalPlanBuilder as _LogicalPlanBuilder
-from daft.daft import (
-    PartitionScheme,
     PyDaftExecutionConfig,
     ResourceRequest,
     ScanOperatorHandle,
-    StorageConfig,
 )
+from daft.daft import LogicalPlanBuilder as _LogicalPlanBuilder
 from daft.expressions import Expression, col
 from daft.logical.schema import Schema
 from daft.runners.partitioning import PartitionCacheEntry
 
 if TYPE_CHECKING:
-    from daft.plan_scheduler.physical_plan_scheduler import PhysicalPlanScheduler
+    from pyiceberg.table import Table as IcebergTable
+
+    from daft.plan_scheduler.physical_plan_scheduler import AdaptivePhysicalPlanScheduler, PhysicalPlanScheduler
 
 
 class LogicalPlanBuilder:
@@ -46,6 +43,13 @@ class LogicalPlanBuilder:
 
         return PhysicalPlanScheduler(self._builder.to_physical_plan_scheduler(daft_execution_config))
 
+    def to_adaptive_physical_plan_scheduler(
+        self, daft_execution_config: PyDaftExecutionConfig
+    ) -> AdaptivePhysicalPlanScheduler:
+        from daft.plan_scheduler.physical_plan_scheduler import AdaptivePhysicalPlanScheduler
+
+        return AdaptivePhysicalPlanScheduler(self._builder.to_adaptive_physical_plan_scheduler(daft_execution_config))
+
     def schema(self) -> Schema:
         """
         The schema of the current logical plan.
@@ -74,41 +78,37 @@ class LogicalPlanBuilder:
 
     @classmethod
     def from_in_memory_scan(
-        cls, partition: PartitionCacheEntry, schema: Schema, num_partitions: int, size_bytes: int
+        cls, partition: PartitionCacheEntry, schema: Schema, num_partitions: int, size_bytes: int, num_rows: int
     ) -> LogicalPlanBuilder:
         builder = _LogicalPlanBuilder.in_memory_scan(
-            partition.key, partition, schema._schema, num_partitions, size_bytes
+            partition.key, partition, schema._schema, num_partitions, size_bytes, num_rows
         )
         return cls(builder)
 
-    @classmethod
-    def from_tabular_scan_with_scan_operator(
-        cls,
-        *,
-        scan_operator: ScanOperatorHandle,
-    ) -> LogicalPlanBuilder:
-        builder = _LogicalPlanBuilder.table_scan_with_scan_operator(scan_operator)
-        return cls(builder)
-
     @classmethod
     def from_tabular_scan(
         cls,
         *,
-        file_infos: FileInfos,
-        schema: Schema,
-        file_format_config: FileFormatConfig,
-        storage_config: StorageConfig,
+        scan_operator: ScanOperatorHandle,
     ) -> LogicalPlanBuilder:
-        builder = _LogicalPlanBuilder.table_scan(file_infos, schema._schema, file_format_config, storage_config)
+        builder = _LogicalPlanBuilder.table_scan(scan_operator)
         return cls(builder)
 
-    def project(
+    def select(
         self,
-        projection: list[Expression],
-        custom_resource_request: ResourceRequest = ResourceRequest(),
+        to_select: list[Expression],
     ) -> LogicalPlanBuilder:
-        projection_pyexprs = [expr._expr for expr in projection]
-        builder = self._builder.project(projection_pyexprs, custom_resource_request)
+        to_select_pyexprs = [expr._expr for expr in to_select]
+        builder = self._builder.select(to_select_pyexprs)
+        return LogicalPlanBuilder(builder)
+
+    def with_columns(self, columns: list[Expression], custom_resource_request: ResourceRequest) -> LogicalPlanBuilder:
+        column_pyexprs = [expr._expr for expr in columns]
+        builder = self._builder.with_columns(column_pyexprs, custom_resource_request)
+        return LogicalPlanBuilder(builder)
+
+    def exclude(self, to_exclude: list[str]) -> LogicalPlanBuilder:
+        builder = self._builder.exclude(to_exclude)
         return LogicalPlanBuilder(builder)
 
     def filter(self, predicate: Expression) -> LogicalPlanBuilder:
@@ -124,11 +124,19 @@ class LogicalPlanBuilder:
         builder = self._builder.explode(explode_pyexprs)
         return LogicalPlanBuilder(builder)
 
+    def unpivot(
+        self, ids: list[Expression], values: list[Expression], variable_name: str, value_name: str
+    ) -> LogicalPlanBuilder:
+        ids_pyexprs = [expr._expr for expr in ids]
+        values_pyexprs = [expr._expr for expr in values]
+        builder = self._builder.unpivot(ids_pyexprs, values_pyexprs, variable_name, value_name)
+        return LogicalPlanBuilder(builder)
+
     def count(self) -> LogicalPlanBuilder:
         # TODO(Clark): Add dedicated logical/physical ops when introducing metadata-based count optimizations.
         first_col = col(self.schema().column_names()[0])
-        builder = self._builder.aggregate([first_col._count(CountMode.All)._expr], [])
-        builder = builder.project([first_col.alias("count")._expr], ResourceRequest())
+        builder = self._builder.aggregate([first_col.count(CountMode.All)._expr], [])
+        builder = builder.select([first_col.alias("count")._expr])
         return LogicalPlanBuilder(builder)
 
     def distinct(self) -> LogicalPlanBuilder:
@@ -146,39 +154,43 @@ class LogicalPlanBuilder:
         builder = self._builder.sort(sort_by_pyexprs, descending)
         return LogicalPlanBuilder(builder)
 
-    def repartition(
-        self, num_partitions: int | None, partition_by: list[Expression], scheme: PartitionScheme
-    ) -> LogicalPlanBuilder:
+    def hash_repartition(self, num_partitions: int | None, partition_by: list[Expression]) -> LogicalPlanBuilder:
         partition_by_pyexprs = [expr._expr for expr in partition_by]
-        builder = self._builder.repartition(partition_by_pyexprs, scheme, num_partitions=num_partitions)
+        builder = self._builder.hash_repartition(partition_by_pyexprs, num_partitions=num_partitions)
+        return LogicalPlanBuilder(builder)
+
+    def random_shuffle(self, num_partitions: int | None) -> LogicalPlanBuilder:
+        builder = self._builder.random_shuffle(num_partitions)
+        return LogicalPlanBuilder(builder)
+
+    def into_partitions(self, num_partitions: int) -> LogicalPlanBuilder:
+        builder = self._builder.into_partitions(num_partitions)
         return LogicalPlanBuilder(builder)
 
     def agg(
         self,
-        to_agg: list[tuple[Expression, str]],
+        to_agg: list[Expression],
         group_by: list[Expression] | None,
     ) -> LogicalPlanBuilder:
-        exprs = []
-        for expr, op in to_agg:
-            if op == "sum":
-                exprs.append(expr._sum())
-            elif op == "count":
-                exprs.append(expr._count())
-            elif op == "min":
-                exprs.append(expr._min())
-            elif op == "max":
-                exprs.append(expr._max())
-            elif op == "mean":
-                exprs.append(expr._mean())
-            elif op == "list":
-                exprs.append(expr._agg_list())
-            elif op == "concat":
-                exprs.append(expr._agg_concat())
-            else:
-                raise NotImplementedError(f"Aggregation {op} is not implemented.")
-
         group_by_pyexprs = [expr._expr for expr in group_by] if group_by is not None else []
-        builder = self._builder.aggregate([expr._expr for expr in exprs], group_by_pyexprs)
+        builder = self._builder.aggregate([expr._expr for expr in to_agg], group_by_pyexprs)
+        return LogicalPlanBuilder(builder)
+
+    def map_groups(self, udf: Expression, group_by: list[Expression] | None) -> LogicalPlanBuilder:
+        group_by_pyexprs = [expr._expr for expr in group_by] if group_by is not None else []
+        builder = self._builder.aggregate([udf._expr], group_by_pyexprs)
+        return LogicalPlanBuilder(builder)
+
+    def pivot(
+        self,
+        group_by: list[Expression],
+        pivot_col: Expression,
+        value_col: Expression,
+        agg_fn: Expression,
+        names: list[str],
+    ) -> LogicalPlanBuilder:
+        group_by_pyexprs = [expr._expr for expr in group_by]
+        builder = self._builder.pivot(group_by_pyexprs, pivot_col._expr, value_col._expr, agg_fn._expr, names)
         return LogicalPlanBuilder(builder)
 
     def join(  # type: ignore[override]
@@ -187,26 +199,25 @@ class LogicalPlanBuilder:
         left_on: list[Expression],
         right_on: list[Expression],
         how: JoinType = JoinType.Inner,
+        strategy: JoinStrategy | None = None,
     ) -> LogicalPlanBuilder:
-        if how == JoinType.Left:
-            raise NotImplementedError("Left join not implemented.")
-        elif how == JoinType.Right:
-            raise NotImplementedError("Right join not implemented.")
-        elif how == JoinType.Inner:
-            builder = self._builder.join(
-                right._builder,
-                [expr._expr for expr in left_on],
-                [expr._expr for expr in right_on],
-                how,
-            )
-            return LogicalPlanBuilder(builder)
-        else:
-            raise NotImplementedError(f"{how} join not implemented.")
+        builder = self._builder.join(
+            right._builder,
+            [expr._expr for expr in left_on],
+            [expr._expr for expr in right_on],
+            how,
+            strategy,
+        )
+        return LogicalPlanBuilder(builder)
 
     def concat(self, other: LogicalPlanBuilder) -> LogicalPlanBuilder:  # type: ignore[override]
         builder = self._builder.concat(other._builder)
         return LogicalPlanBuilder(builder)
 
+    def add_monotonically_increasing_id(self, column_name: str | None) -> LogicalPlanBuilder:
+        builder = self._builder.add_monotonically_increasing_id(column_name)
+        return LogicalPlanBuilder(builder)
+
     def write_tabular(
         self,
         root_dir: str | pathlib.Path,
@@ -220,3 +231,35 @@ class LogicalPlanBuilder:
         part_cols_pyexprs = [expr._expr for expr in partition_cols] if partition_cols is not None else None
         builder = self._builder.table_write(str(root_dir), file_format, part_cols_pyexprs, compression, io_config)
         return LogicalPlanBuilder(builder)
+
+    def write_iceberg(self, table: IcebergTable) -> LogicalPlanBuilder:
+        from daft.io._iceberg import _convert_iceberg_file_io_properties_to_io_config
+
+        name = ".".join(table.name())
+        location = f"{table.location()}/data"
+        spec_id = table.spec().spec_id
+        schema = table.schema()
+        props = table.properties
+        columns = [col.name for col in schema.columns]
+        io_config = _convert_iceberg_file_io_properties_to_io_config(table.io.properties)
+        builder = self._builder.iceberg_write(name, location, spec_id, schema, props, columns, io_config)
+        return LogicalPlanBuilder(builder)
+
+    def write_delta(
+        self,
+        path: str | pathlib.Path,
+        mode: str,
+        current_version: int,
+        large_dtypes: bool,
+        io_config: IOConfig,
+    ) -> LogicalPlanBuilder:
+        columns_name = self.schema().column_names()
+        builder = self._builder.delta_write(
+            str(path),
+            columns_name,
+            mode,
+            current_version,
+            large_dtypes,
+            io_config,
+        )
+        return LogicalPlanBuilder(builder)
