@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-import sys
 from typing import TYPE_CHECKING, Iterator
 
 from daft.daft import CsvParseOptions, JsonParseOptions
@@ -14,12 +13,6 @@ from daft.datatype import DataType, TimeUnit
 if TYPE_CHECKING:
     import pyarrow as pa
 
-if sys.version_info < (3, 8):
-    pass
-else:
-    pass
-
-if TYPE_CHECKING:
     from daft.io import IOConfig
 
 
@@ -116,6 +109,9 @@ class Schema:
     def column_names(self) -> list[str]:
         return list(self._schema.names())
 
+    def estimate_row_size_bytes(self) -> float:
+        return self._schema.estimate_row_size_bytes()
+
     def __iter__(self) -> Iterator[Field]:
         col_names = self.column_names()
         yield from (self[name] for name in col_names)
@@ -132,6 +128,9 @@ class Schema:
     def _repr_html_(self) -> str:
         return self._schema._repr_html_()
 
+    def _truncated_table_string(self) -> str:
+        return self._schema._truncated_table_string()
+
     def union(self, other: Schema) -> Schema:
         if not isinstance(other, Schema):
             raise ValueError(f"Expected Schema, got other: {type(other)}")
@@ -142,12 +141,6 @@ class Schema:
 
         return Schema._from_pyschema(self._schema.union(other._schema))
 
-    def apply_hints(self, other: Schema) -> Schema:
-        if not isinstance(other, Schema):
-            raise ValueError(f"Expected Schema, got other: {type(other)}")
-
-        return Schema._from_pyschema(self._schema.apply_hints(other._schema))
-
     def __reduce__(self) -> tuple:
         return Schema._from_pyschema, (self._schema,)
 
