@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-import sys
+import threading
 import weakref
 from abc import abstractmethod
 from dataclasses import dataclass
@@ -10,14 +10,10 @@ from uuid import uuid4
 import pyarrow as pa
 
 from daft.datatype import TimeUnit
+from daft.expressions.expressions import Expression
 from daft.logical.schema import Schema
 from daft.table import MicroPartition
 
-if sys.version_info < (3, 8):
-    pass
-else:
-    pass
-
 if TYPE_CHECKING:
     import pandas as pd
 
@@ -74,20 +70,97 @@ class TableParseParquetOptions:
 class PartialPartitionMetadata:
     num_rows: None | int
     size_bytes: None | int
+    boundaries: None | Boundaries = None
 
 
 @dataclass(frozen=True)
 class PartitionMetadata(PartialPartitionMetadata):
     num_rows: int
     size_bytes: int | None
+    boundaries: Boundaries | None = None
 
     @classmethod
     def from_table(cls, table: MicroPartition) -> PartitionMetadata:
         return PartitionMetadata(
             num_rows=len(table),
             size_bytes=table.size_bytes(),
+            boundaries=None,
         )
 
+    def merge_with_partial(self, partial_metadata: PartialPartitionMetadata) -> PartitionMetadata:
+        num_rows = self.num_rows
+        size_bytes = self.size_bytes
+        boundaries = self.boundaries
+        if boundaries is None:
+            boundaries = partial_metadata.boundaries
+        return PartitionMetadata(num_rows, size_bytes, boundaries)
+
+    def downcast_to_partial(self) -> PartialPartitionMetadata:
+        return PartialPartitionMetadata(self.num_rows, self.size_bytes, self.boundaries)
+
+
+def _is_bound_null(bound_row: list[Any | None]) -> bool:
+    return all(bound is None for bound in bound_row)
+
+
+# TODO(Clark): Port this to the Rust side.
+@dataclass(frozen=True)
+class Boundaries:
+    sort_by: list[Expression]
+    bounds: MicroPartition
+
+    def __post_init__(self):
+        assert len(self.sort_by) > 0
+        assert len(self.bounds) == 2
+        assert self.bounds.column_names() == [e.name() for e in self.sort_by]
+
+    def intersects(self, other: Boundaries) -> bool:
+        if self.is_trivial_bounds() or other.is_trivial_bounds():
+            return True
+        self_bounds = self.bounds.to_pylist()
+        other_bounds = other.bounds.to_pylist()
+        self_lower = list(self_bounds[0].values())
+        self_upper = list(self_bounds[1].values())
+        other_lower = list(other_bounds[0].values())
+        other_upper = list(other_bounds[1].values())
+        if _is_bound_null(self_lower):
+            return _is_bound_null(other_lower) or other_lower <= self_upper
+        if _is_bound_null(other_lower):
+            return self_lower <= other_upper
+        if _is_bound_null(self_upper):
+            return _is_bound_null(other_upper) or other_upper >= self_lower
+        if _is_bound_null(other_upper):
+            return self_upper >= other_lower
+        return (self_lower <= other_lower and self_upper >= other_lower) or (
+            self_lower > other_lower and other_upper >= self_lower
+        )
+
+    def is_disjointly_bounded_above_by(self, other: Boundaries) -> bool:
+        # Check that upper of self is less than lower of other.
+        self_upper = list(self.bounds.to_pylist()[1].values())
+        if _is_bound_null(self_upper):
+            return False
+        other_lower = list(other.bounds.to_pylist()[0].values())
+        if _is_bound_null(other_lower):
+            return False
+        return self_upper < other_lower
+
+    def is_trivial_bounds(self) -> bool:
+        bounds = self.bounds.to_pylist()
+        lower = list(bounds[0].values())
+        upper = list(bounds[1].values())
+        return _is_bound_null(lower) and _is_bound_null(upper)
+
+    def is_strictly_bounded_above_by(self, other: Boundaries) -> bool:
+        # Check that upper of self is less than upper of other.
+        self_upper = list(self.bounds.to_pylist()[1].values())
+        if _is_bound_null(self_upper):
+            return False
+        other_upper = list(other.bounds.to_pylist()[1].values())
+        if _is_bound_null(other_upper):
+            return True
+        return self_upper < other_upper
+
 
 PartitionT = TypeVar("PartitionT")
 
@@ -130,33 +203,41 @@ class PartitionSet(Generic[PartitionT]):
     def _get_merged_vpartition(self) -> MicroPartition:
         raise NotImplementedError()
 
+    def _get_preview_vpartition(self, num_rows: int) -> list[MicroPartition]:
+        raise NotImplementedError()
+
     def to_pydict(self) -> dict[str, list[Any]]:
         """Retrieves all the data in a PartitionSet as a Python dictionary. Values are the raw data from each Block."""
         merged_partition = self._get_merged_vpartition()
         return merged_partition.to_pydict()
 
-    def to_pandas(self, schema: Schema | None = None, cast_tensors_to_ray_tensor_dtype: bool = False) -> pd.DataFrame:
+    def to_pandas(
+        self,
+        schema: Schema | None = None,
+        cast_tensors_to_ray_tensor_dtype: bool = False,
+    ) -> pd.DataFrame:
         merged_partition = self._get_merged_vpartition()
         return merged_partition.to_pandas(
-            schema=schema, cast_tensors_to_ray_tensor_dtype=cast_tensors_to_ray_tensor_dtype
+            schema=schema,
+            cast_tensors_to_ray_tensor_dtype=cast_tensors_to_ray_tensor_dtype,
         )
 
     def to_arrow(self, cast_tensors_to_ray_tensor_dtype: bool = False) -> pa.Table:
         merged_partition = self._get_merged_vpartition()
         return merged_partition.to_arrow(cast_tensors_to_ray_tensor_dtype)
 
-    def items(self) -> list[tuple[PartID, PartitionT]]:
+    def items(self) -> list[tuple[PartID, MaterializedResult[PartitionT]]]:
         """
         Returns all (partition id, partition) in this PartitionSet,
         ordered by partition ID.
         """
         raise NotImplementedError()
 
-    def values(self) -> list[PartitionT]:
+    def values(self) -> list[MaterializedResult[PartitionT]]:
         return [value for _, value in self.items()]
 
     @abstractmethod
-    def get_partition(self, idx: PartID) -> PartitionT:
+    def get_partition(self, idx: PartID) -> MaterializedResult[PartitionT]:
         raise NotImplementedError()
 
     @abstractmethod
@@ -215,27 +296,39 @@ class PartitionCacheEntry:
     def size_bytes(self) -> int | None:
         return self.value.size_bytes() if self.value is not None else None
 
+    def num_rows(self) -> int | None:
+        return len(self.value) if self.value is not None else None
+
 
 class PartitionSetCache:
     def __init__(self) -> None:
-        self._uuid_to_partition_set: weakref.WeakValueDictionary[
-            str, PartitionCacheEntry
-        ] = weakref.WeakValueDictionary()
+        self.__uuid_to_partition_set: weakref.WeakValueDictionary[str, PartitionCacheEntry] = (
+            weakref.WeakValueDictionary()
+        )
+        self._lock = threading.Lock()
 
     def get_partition_set(self, pset_id: str) -> PartitionCacheEntry:
-        assert pset_id in self._uuid_to_partition_set
-        return self._uuid_to_partition_set[pset_id]
+        with self._lock:
+            assert pset_id in self.__uuid_to_partition_set
+            return self.__uuid_to_partition_set[pset_id]
+
+    def get_all_partition_sets(self) -> dict[str, PartitionSet]:
+        with self._lock:
+            return {key: entry.value for key, entry in self.__uuid_to_partition_set.items() if entry.value is not None}
 
     def put_partition_set(self, pset: PartitionSet) -> PartitionCacheEntry:
         pset_id = uuid4().hex
         part_entry = PartitionCacheEntry(pset_id, pset)
-        self._uuid_to_partition_set[pset_id] = part_entry
-        return part_entry
+        with self._lock:
+            self.__uuid_to_partition_set[pset_id] = part_entry
+            return part_entry
 
     def rm(self, pset_id: str) -> None:
-        if pset_id in self._uuid_to_partition_set:
-            del self._uuid_to_partition_set[pset_id]
+        with self._lock:
+            if pset_id in self.__uuid_to_partition_set:
+                del self.__uuid_to_partition_set[pset_id]
 
     def clear(self) -> None:
-        del self._uuid_to_partition_set
-        self._uuid_to_partition_set = weakref.WeakValueDictionary()
+        with self._lock:
+            del self.__uuid_to_partition_set
+            self.__uuid_to_partition_set = weakref.WeakValueDictionary()
