@@ -1,30 +1,21 @@
 from __future__ import annotations
 
 import logging
-import multiprocessing
 from concurrent import futures
 from dataclasses import dataclass
 from typing import Iterable, Iterator
 
-import psutil
-
 from daft.context import get_context
-from daft.daft import (
-    FileFormatConfig,
-    FileInfos,
-    IOConfig,
-    ResourceRequest,
-    StorageConfig,
-)
+from daft.daft import FileFormatConfig, FileInfos, IOConfig, ResourceRequest, SystemInfo
 from daft.execution import physical_plan
 from daft.execution.execution_step import Instruction, PartitionTask
 from daft.filesystem import glob_path_with_stats
 from daft.internal.gpu import cuda_device_count
 from daft.logical.builder import LogicalPlanBuilder
-from daft.logical.schema import Schema
 from daft.runners import runner_io
 from daft.runners.partitioning import (
     MaterializedResult,
+    PartialPartitionMetadata,
     PartID,
     PartitionCacheEntry,
     PartitionMetadata,
@@ -38,24 +29,44 @@ from daft.table import MicroPartition
 logger = logging.getLogger(__name__)
 
 
-@dataclass
 class LocalPartitionSet(PartitionSet[MicroPartition]):
-    _partitions: dict[PartID, MicroPartition]
+    _partitions: dict[PartID, MaterializedResult[MicroPartition]]
 
-    def items(self) -> list[tuple[PartID, MicroPartition]]:
+    def __init__(self) -> None:
+        super().__init__()
+        self._partitions = {}
+
+    def items(self) -> list[tuple[PartID, MaterializedResult[MicroPartition]]]:
         return sorted(self._partitions.items())
 
     def _get_merged_vpartition(self) -> MicroPartition:
         ids_and_partitions = self.items()
         assert ids_and_partitions[0][0] == 0
         assert ids_and_partitions[-1][0] + 1 == len(ids_and_partitions)
-        return MicroPartition.concat([part for id, part in ids_and_partitions])
+        return MicroPartition.concat([part.partition() for id, part in ids_and_partitions])
 
-    def get_partition(self, idx: PartID) -> MicroPartition:
+    def _get_preview_vpartition(self, num_rows: int) -> list[MicroPartition]:
+        ids_and_partitions = self.items()
+        preview_parts = []
+        for _, mat_result in ids_and_partitions:
+            part: MicroPartition = mat_result.partition()
+            part_len = len(part)
+            if part_len >= num_rows:  # if this part has enough rows, take what we need and break
+                preview_parts.append(part.slice(0, num_rows))
+                break
+            else:  # otherwise, take the whole part and keep going
+                num_rows -= part_len
+                preview_parts.append(part)
+        return preview_parts
+
+    def get_partition(self, idx: PartID) -> MaterializedResult[MicroPartition]:
         return self._partitions[idx]
 
     def set_partition(self, idx: PartID, part: MaterializedResult[MicroPartition]) -> None:
-        self._partitions[idx] = part.partition()
+        self._partitions[idx] = part
+
+    def set_partition_from_table(self, idx: PartID, part: MicroPartition) -> None:
+        self._partitions[idx] = PyMaterializedResult(part, PartitionMetadata.from_table(part))
 
     def delete_partition(self, idx: PartID) -> None:
         del self._partitions[idx]
@@ -64,10 +75,10 @@ class LocalPartitionSet(PartitionSet[MicroPartition]):
         return idx in self._partitions
 
     def __len__(self) -> int:
-        return sum(len(partition) for partition in self._partitions.values())
+        return sum(len(partition.partition()) for partition in self._partitions.values())
 
     def size_bytes(self) -> int | None:
-        size_bytes_ = [partition.size_bytes() for partition in self._partitions.values()]
+        size_bytes_ = [partition.partition().size_bytes() for partition in self._partitions.values()]
         size_bytes: list[int] = [size for size in size_bytes_ if size is not None]
         if len(size_bytes) != len(size_bytes_):
             return None
@@ -100,26 +111,22 @@ class PyRunnerIO(runner_io.RunnerIO):
 
         return file_infos
 
-    def get_schema_from_first_filepath(
-        self,
-        file_infos: FileInfos,
-        file_format_config: FileFormatConfig,
-        storage_config: StorageConfig,
-    ) -> Schema:
-        if len(file_infos) == 0:
-            raise ValueError("No files to get schema from")
-        # Naively retrieve the first filepath in the PartitionSet
-        return runner_io.sample_schema(file_infos[0].file_path, file_format_config, storage_config)
-
 
 class PyRunner(Runner[MicroPartition]):
     def __init__(self, use_thread_pool: bool | None) -> None:
         super().__init__()
         self._use_thread_pool: bool = use_thread_pool if use_thread_pool is not None else True
+        system_info = SystemInfo()
+        num_cpus = system_info.cpu_count()
+        if num_cpus is None:
+            import multiprocessing
+
+            self.num_cpus = multiprocessing.cpu_count()
+        else:
+            self.num_cpus = num_cpus
 
-        self.num_cpus = multiprocessing.cpu_count()
         self.num_gpus = cuda_device_count()
-        self.bytes_memory = psutil.virtual_memory().total
+        self.bytes_memory = system_info.total_memory()
 
     def runner_io(self) -> PyRunnerIO:
         return PyRunnerIO()
@@ -127,7 +134,7 @@ class PyRunner(Runner[MicroPartition]):
     def run(self, builder: LogicalPlanBuilder) -> PartitionCacheEntry:
         results = list(self.run_iter(builder))
 
-        result_pset = LocalPartitionSet({})
+        result_pset = LocalPartitionSet()
         for i, result in enumerate(results):
             result_pset.set_partition(i, result)
 
@@ -145,19 +152,39 @@ class PyRunner(Runner[MicroPartition]):
 
         # Optimize the logical plan.
         builder = builder.optimize()
-        # Finalize the logical plan and get a physical plan scheduler for translating the
-        # physical plan to executable tasks.
-        plan_scheduler = builder.to_physical_plan_scheduler(daft_execution_config)
-        psets = {
-            key: entry.value.values()
-            for key, entry in self._part_set_cache._uuid_to_partition_set.items()
-            if entry.value is not None
-        }
-        # Get executable tasks from planner.
-        tasks = plan_scheduler.to_partition_tasks(psets, is_ray_runner=False)
-        with profiler("profile_PyRunner.run_{datetime.now().isoformat()}.json"):
-            results_gen = self._physical_plan_to_partitions(tasks)
-            yield from results_gen
+
+        if daft_execution_config.enable_aqe:
+            adaptive_planner = builder.to_adaptive_physical_plan_scheduler(daft_execution_config)
+            while not adaptive_planner.is_done():
+                source_id, plan_scheduler = adaptive_planner.next()
+                # don't store partition sets in variable to avoid reference
+                tasks = plan_scheduler.to_partition_tasks(
+                    {k: v.values() for k, v in self._part_set_cache.get_all_partition_sets().items()}
+                )
+                del plan_scheduler
+                results_gen = self._physical_plan_to_partitions(tasks)
+                # if source_id is none that means this is the final stage
+                if source_id is None:
+                    yield from results_gen
+                else:
+                    intermediate = LocalPartitionSet()
+                    for i, rg in enumerate(results_gen):
+                        intermediate.set_partition(i, rg)
+                    cache_entry = self._part_set_cache.put_partition_set(intermediate)
+                    del intermediate
+                    adaptive_planner.update(source_id, cache_entry)
+                    del cache_entry
+        else:
+            # Finalize the logical plan and get a physical plan scheduler for translating the
+            # physical plan to executable tasks.
+            plan_scheduler = builder.to_physical_plan_scheduler(daft_execution_config)
+            psets = {k: v.values() for k, v in self._part_set_cache.get_all_partition_sets().items()}
+            # Get executable tasks from planner.
+            tasks = plan_scheduler.to_partition_tasks(psets)
+            del psets
+            with profiler("profile_PyRunner.run_{datetime.now().isoformat()}.json"):
+                results_gen = self._physical_plan_to_partitions(tasks)
+                yield from results_gen
 
     def run_iter_tables(
         self, builder: LogicalPlanBuilder, results_buffer_size: int | None = None
@@ -214,8 +241,10 @@ class PyRunner(Runner[MicroPartition]):
                                 )
                             ):
                                 logger.debug("Running task synchronously in main thread: %s", next_step)
-                                partitions = self.build_partitions(next_step.instructions, *next_step.inputs)
-                                next_step.set_result([PyMaterializedResult(partition) for partition in partitions])
+                                materialized_results = self.build_partitions(
+                                    next_step.instructions, next_step.inputs, next_step.partial_metadatas
+                                )
+                                next_step.set_result(materialized_results)
 
                             else:
                                 # Submit the task for execution.
@@ -225,7 +254,10 @@ class PyRunner(Runner[MicroPartition]):
                                 pbar.mark_task_start(next_step)
 
                                 future = thread_pool.submit(
-                                    self.build_partitions, next_step.instructions, *next_step.inputs
+                                    self.build_partitions,
+                                    next_step.instructions,
+                                    next_step.inputs,
+                                    next_step.partial_metadatas,
                                 )
                                 # Register the inflight task and resources used.
                                 future_to_task[future] = next_step.id()
@@ -238,18 +270,19 @@ class PyRunner(Runner[MicroPartition]):
                     # Await at least one task and process the results.
                     assert (
                         len(future_to_task) > 0
-                    ), f"Scheduler deadlocked! This should never happen. Please file an issue."
+                    ), "Scheduler deadlocked! This should never happen. Please file an issue."
                     done_set, _ = futures.wait(list(future_to_task.keys()), return_when=futures.FIRST_COMPLETED)
                     for done_future in done_set:
                         done_id = future_to_task.pop(done_future)
                         del inflight_tasks_resources[done_id]
                         done_task = inflight_tasks.pop(done_id)
-                        partitions = done_future.result()
+                        materialized_results = done_future.result()
 
                         pbar.mark_task_done(done_task)
 
-                        logger.debug("Task completed: %s -> <%s partitions>", done_id, len(partitions))
-                        done_task.set_result([PyMaterializedResult(partition) for partition in partitions])
+                        logger.debug("Task completed: %s -> <%s partitions>", done_id, len(materialized_results))
+
+                        done_task.set_result(materialized_results)
 
                     if next_step is None:
                         next_step = next(plan)
@@ -283,17 +316,23 @@ class PyRunner(Runner[MicroPartition]):
         return all((cpus_okay, gpus_okay, memory_okay))
 
     @staticmethod
-    def build_partitions(instruction_stack: list[Instruction], *inputs: MicroPartition) -> list[MicroPartition]:
-        partitions = list(inputs)
+    def build_partitions(
+        instruction_stack: list[Instruction],
+        partitions: list[MicroPartition],
+        final_metadata: list[PartialPartitionMetadata],
+    ) -> list[MaterializedResult[MicroPartition]]:
         for instruction in instruction_stack:
             partitions = instruction.run(partitions)
-
-        return partitions
+        return [
+            PyMaterializedResult(part, PartitionMetadata.from_table(part).merge_with_partial(partial))
+            for part, partial in zip(partitions, final_metadata)
+        ]
 
 
-@dataclass(frozen=True)
+@dataclass
 class PyMaterializedResult(MaterializedResult[MicroPartition]):
     _partition: MicroPartition
+    _metadata: PartitionMetadata | None = None
 
     def partition(self) -> MicroPartition:
         return self._partition
@@ -302,7 +341,9 @@ class PyMaterializedResult(MaterializedResult[MicroPartition]):
         return self._partition
 
     def metadata(self) -> PartitionMetadata:
-        return PartitionMetadata.from_table(self._partition)
+        if self._metadata is None:
+            self._metadata = PartitionMetadata.from_table(self._partition)
+        return self._metadata
 
     def cancel(self) -> None:
         return None
