@@ -4,7 +4,6 @@ import logging
 import threading
 import time
 import uuid
-from dataclasses import dataclass
 from datetime import datetime
 from queue import Full, Queue
 from typing import TYPE_CHECKING, Any, Generator, Iterable, Iterator
@@ -22,7 +21,7 @@ try:
     import ray
 except ImportError:
     logger.error(
-        f"Error when importing Ray. Please ensure that getdaft was installed with the Ray extras tag: getdaft[ray] (https://www.getdaft.io/projects/docs/en/latest/learn/install.html)"
+        "Error when importing Ray. Please ensure that getdaft was installed with the Ray extras tag: getdaft[ray] (https://www.getdaft.io/projects/docs/en/latest/learn/install.html)"
     )
     raise
 
@@ -32,7 +31,6 @@ from daft.daft import (
     IOConfig,
     PyDaftExecutionConfig,
     ResourceRequest,
-    StorageConfig,
 )
 from daft.datatype import DataType
 from daft.execution.execution_step import (
@@ -41,6 +39,7 @@ from daft.execution.execution_step import (
     MultiOutputPartitionTask,
     PartitionTask,
     ReduceInstruction,
+    ScanWithTask,
     SingleOutputPartitionTask,
 )
 from daft.filesystem import glob_path_with_stats
@@ -126,31 +125,38 @@ def _to_pandas_ref(df: pd.DataFrame | ray.ObjectRef[pd.DataFrame]) -> ray.Object
         raise ValueError("Expected a Ray object ref or a Pandas DataFrame, " f"got {type(df)}")
 
 
-@ray.remote
-def sample_schema_from_filepath(
-    first_file_path: str,
-    file_format_config: FileFormatConfig,
-    storage_config: StorageConfig,
-) -> Schema:
-    """Ray remote function to run schema sampling on top of a MicroPartition containing a single filepath"""
-    # Currently just samples the Schema from the first file
-    return runner_io.sample_schema(first_file_path, file_format_config, storage_config)
-
-
-@dataclass
 class RayPartitionSet(PartitionSet[ray.ObjectRef]):
     _results: dict[PartID, RayMaterializedResult]
 
-    def items(self) -> list[tuple[PartID, ray.ObjectRef]]:
-        return [(pid, result.partition()) for pid, result in sorted(self._results.items())]
+    def __init__(self) -> None:
+        super().__init__()
+        self._results = {}
+
+    def items(self) -> list[tuple[PartID, MaterializedResult[ray.ObjectRef]]]:
+        return [(pid, result) for pid, result in sorted(self._results.items())]
 
     def _get_merged_vpartition(self) -> MicroPartition:
         ids_and_partitions = self.items()
         assert ids_and_partitions[0][0] == 0
         assert ids_and_partitions[-1][0] + 1 == len(ids_and_partitions)
-        all_partitions = ray.get([part for id, part in ids_and_partitions])
+        all_partitions = ray.get([part.partition() for id, part in ids_and_partitions])
         return MicroPartition.concat(all_partitions)
 
+    def _get_preview_vpartition(self, num_rows: int) -> list[MicroPartition]:
+        ids_and_partitions = self.items()
+        preview_parts = []
+        for _, mat_result in ids_and_partitions:
+            ref: ray.ObjectRef = mat_result.partition()
+            part: MicroPartition = ray.get(ref)
+            part_len = len(part)
+            if part_len >= num_rows:  # if this part has enough rows, take what we need and break
+                preview_parts.append(part.slice(0, num_rows))
+                break
+            else:  # otherwise, take the whole part and keep going
+                num_rows -= part_len
+                preview_parts.append(part)
+        return preview_parts
+
     def to_ray_dataset(self) -> RayDataset:
         if not _RAY_FROM_ARROW_REFS_AVAILABLE:
             raise ImportError(
@@ -230,24 +236,6 @@ class RayRunnerIO(runner_io.RunnerIO):
             ._table
         )
 
-    def get_schema_from_first_filepath(
-        self,
-        file_infos: FileInfos,
-        file_format_config: FileFormatConfig,
-        storage_config: StorageConfig,
-    ) -> Schema:
-        if len(file_infos) == 0:
-            raise ValueError("No files to get schema from")
-        # Naively retrieve the first filepath in the file info table.
-        first_path = file_infos[0].file_path
-        return ray.get(
-            sample_schema_from_filepath.remote(
-                first_path,
-                file_format_config,
-                storage_config,
-            )
-        )
-
     def partition_set_from_ray_dataset(
         self,
         ds: RayDataset,
@@ -281,9 +269,12 @@ class RayRunnerIO(runner_io.RunnerIO):
         daft_vpartitions = [
             _make_daft_partition_from_ray_dataset_blocks.remote(block, daft_schema) for block in block_refs
         ]
+        pset = RayPartitionSet()
 
+        for i, obj in enumerate(daft_vpartitions):
+            pset.set_partition(i, RayMaterializedResult(obj))
         return (
-            RayPartitionSet(_results={i: RayMaterializedResult(obj) for i, obj in enumerate(daft_vpartitions)}),
+            pset,
             daft_schema,
         )
 
@@ -307,8 +298,13 @@ class RayRunnerIO(runner_io.RunnerIO):
                 "Can't convert a Dask DataFrame with inconsistent schemas across partitions to a Daft DataFrame:",
                 schemas,
             )
+
+        pset = RayPartitionSet()
+
+        for i, obj in enumerate(daft_vpartitions):
+            pset.set_partition(i, RayMaterializedResult(obj))
         return (
-            RayPartitionSet(_results={i: RayMaterializedResult(obj) for i, obj in enumerate(daft_vpartitions)}),
+            pset,
             schemas[0],
         )
 
@@ -331,13 +327,15 @@ def _get_ray_task_options(resource_request: ResourceRequest) -> dict[str, Any]:
 
 
 def build_partitions(
-    instruction_stack: list[Instruction], *inputs: MicroPartition
+    instruction_stack: list[Instruction], partial_metadatas: list[PartitionMetadata], *inputs: MicroPartition
 ) -> list[list[PartitionMetadata] | MicroPartition]:
     partitions = list(inputs)
     for instruction in instruction_stack:
         partitions = instruction.run(partitions)
 
-    metadatas = [PartitionMetadata.from_table(p) for p in partitions]
+    assert len(partial_metadatas) == len(partitions), f"{len(partial_metadatas)} vs {len(partitions)}"
+
+    metadatas = [PartitionMetadata.from_table(p).merge_with_partial(m) for p, m in zip(partitions, partial_metadatas)]
 
     return [metadatas, *partitions]
 
@@ -347,38 +345,50 @@ def build_partitions(
 
 @ray.remote
 def single_partition_pipeline(
-    daft_execution_config: PyDaftExecutionConfig, instruction_stack: list[Instruction], *inputs: MicroPartition
+    daft_execution_config: PyDaftExecutionConfig,
+    instruction_stack: list[Instruction],
+    partial_metadatas: list[PartitionMetadata],
+    *inputs: MicroPartition,
 ) -> list[list[PartitionMetadata] | MicroPartition]:
     set_execution_config(daft_execution_config)
-    return build_partitions(instruction_stack, *inputs)
+    return build_partitions(instruction_stack, partial_metadatas, *inputs)
 
 
 @ray.remote
 def fanout_pipeline(
-    daft_execution_config: PyDaftExecutionConfig, instruction_stack: list[Instruction], *inputs: MicroPartition
+    daft_execution_config: PyDaftExecutionConfig,
+    instruction_stack: list[Instruction],
+    partial_metadatas: list[PartitionMetadata],
+    *inputs: MicroPartition,
 ) -> list[list[PartitionMetadata] | MicroPartition]:
     set_execution_config(daft_execution_config)
-    return build_partitions(instruction_stack, *inputs)
+    return build_partitions(instruction_stack, partial_metadatas, *inputs)
 
 
 @ray.remote(scheduling_strategy="SPREAD")
 def reduce_pipeline(
-    daft_execution_config: PyDaftExecutionConfig, instruction_stack: list[Instruction], inputs: list
+    daft_execution_config: PyDaftExecutionConfig,
+    instruction_stack: list[Instruction],
+    partial_metadatas: list[PartitionMetadata],
+    inputs: list,
 ) -> list[list[PartitionMetadata] | MicroPartition]:
     import ray
 
     set_execution_config(daft_execution_config)
-    return build_partitions(instruction_stack, *ray.get(inputs))
+    return build_partitions(instruction_stack, partial_metadatas, *ray.get(inputs))
 
 
 @ray.remote(scheduling_strategy="SPREAD")
 def reduce_and_fanout(
-    daft_execution_config: PyDaftExecutionConfig, instruction_stack: list[Instruction], inputs: list
+    daft_execution_config: PyDaftExecutionConfig,
+    instruction_stack: list[Instruction],
+    partial_metadatas: list[PartitionMetadata],
+    inputs: list,
 ) -> list[list[PartitionMetadata] | MicroPartition]:
     import ray
 
     set_execution_config(daft_execution_config)
-    return build_partitions(instruction_stack, *ray.get(inputs))
+    return build_partitions(instruction_stack, partial_metadatas, *ray.get(inputs))
 
 
 @ray.remote
@@ -493,7 +503,7 @@ class Scheduler:
         result_uuid: str,
     ) -> None:
         # Get executable tasks from plan scheduler.
-        tasks = plan_scheduler.to_partition_tasks(psets, is_ray_runner=True)
+        tasks = plan_scheduler.to_partition_tasks(psets)
 
         daft_execution_config = self.execution_configs_objref_by_df[result_uuid]
         inflight_tasks: dict[str, PartitionTask[ray.ObjectRef]] = dict()
@@ -526,7 +536,10 @@ class Scheduler:
                     while is_active():  # Loop: Dispatch (get tasks -> batch dispatch).
                         tasks_to_dispatch: list[PartitionTask] = []
 
-                        cores: int = max(next(num_cpus_provider) - self.reserved_cores, 0)
+                        # TODO: improve control loop code to be more understandable and dynamically adjust backlog
+                        cores: int = max(
+                            next(num_cpus_provider) - self.reserved_cores, 1
+                        )  # assume at least 1 CPU core for bootstrapping clusters that scale from zero
                         max_inflight_tasks = cores + self.max_task_backlog
                         dispatches_allowed = max_inflight_tasks - len(inflight_tasks)
                         dispatches_allowed = min(cores, dispatches_allowed)
@@ -547,9 +560,28 @@ class Scheduler:
                             # If it is a no-op task, just run it locally immediately.
                             elif len(next_step.instructions) == 0:
                                 logger.debug("Running task synchronously in main thread: %s", next_step)
-                                assert isinstance(next_step, SingleOutputPartitionTask)
+                                assert (
+                                    len(next_step.partial_metadatas) == 1
+                                ), "No-op tasks must have one output by definition, since there are no instructions to run"
+                                [single_partial] = next_step.partial_metadatas
+                                if single_partial.num_rows is None:
+                                    [single_meta] = ray.get(get_metas.remote(next_step.inputs))
+                                    accessor = PartitionMetadataAccessor.from_metadata_list(
+                                        [single_meta.merge_with_partial(single_partial)]
+                                    )
+                                else:
+                                    accessor = PartitionMetadataAccessor.from_metadata_list(
+                                        [
+                                            PartitionMetadata(
+                                                num_rows=single_partial.num_rows,
+                                                size_bytes=single_partial.size_bytes,
+                                                boundaries=single_partial.boundaries,
+                                            )
+                                        ]
+                                    )
+
                                 next_step.set_result(
-                                    [RayMaterializedResult(partition) for partition in next_step.inputs]
+                                    [RayMaterializedResult(partition, accessor, 0) for partition in next_step.inputs]
                                 )
                                 next_step = next(tasks)
 
@@ -640,6 +672,10 @@ class Scheduler:
         pbar.close()
 
 
+SCHEDULER_ACTOR_NAME = "scheduler"
+SCHEDULER_ACTOR_NAMESPACE = "daft"
+
+
 @ray.remote(num_cpus=1)
 class SchedulerActor(Scheduler):
     def __init__(self, *n, **kw) -> None:
@@ -657,17 +693,27 @@ def _build_partitions(
         ray_options = {**ray_options, **_get_ray_task_options(task.resource_request)}
 
     if isinstance(task.instructions[0], ReduceInstruction):
-        build_remote = reduce_and_fanout if isinstance(task.instructions[-1], FanoutInstruction) else reduce_pipeline
-        build_remote = build_remote.options(**ray_options)
-        [metadatas_ref, *partitions] = build_remote.remote(daft_execution_config_objref, task.instructions, task.inputs)
-
-    else:
         build_remote = (
-            fanout_pipeline if isinstance(task.instructions[-1], FanoutInstruction) else single_partition_pipeline
+            reduce_and_fanout
+            if task.instructions and isinstance(task.instructions[-1], FanoutInstruction)
+            else reduce_pipeline
         )
         build_remote = build_remote.options(**ray_options)
         [metadatas_ref, *partitions] = build_remote.remote(
-            daft_execution_config_objref, task.instructions, *task.inputs
+            daft_execution_config_objref, task.instructions, task.partial_metadatas, task.inputs
+        )
+
+    else:
+        build_remote = (
+            fanout_pipeline
+            if task.instructions and isinstance(task.instructions[-1], FanoutInstruction)
+            else single_partition_pipeline
+        )
+        if task.instructions and isinstance(task.instructions[0], ScanWithTask):
+            ray_options["scheduling_strategy"] = "SPREAD"
+        build_remote = build_remote.options(**ray_options)
+        [metadatas_ref, *partitions] = build_remote.remote(
+            daft_execution_config_objref, task.instructions, task.partial_metadatas, *task.inputs
         )
 
     metadatas_accessor = PartitionMetadataAccessor(metadatas_ref)
@@ -693,12 +739,16 @@ class RayRunner(Runner[ray.ObjectRef]):
     ) -> None:
         super().__init__()
         if ray.is_initialized():
-            logger.warning(f"Ray has already been initialized, Daft will reuse the existing Ray context.")
+            logger.warning("Ray has already been initialized, Daft will reuse the existing Ray context.")
         self.ray_context = ray.init(address=address, ignore_reinit_error=True)
 
         if isinstance(self.ray_context, ray.client_builder.ClientContext):
             # Run scheduler remotely if the cluster is connected remotely.
-            self.scheduler_actor = SchedulerActor.remote(  # type: ignore
+            self.scheduler_actor = SchedulerActor.options(  # type: ignore
+                name=SCHEDULER_ACTOR_NAME,
+                namespace=SCHEDULER_ACTOR_NAMESPACE,
+                get_if_exists=True,
+            ).remote(  # type: ignore
                 max_task_backlog=max_task_backlog,
                 use_ray_tqdm=True,
             )
@@ -714,24 +764,13 @@ class RayRunner(Runner[ray.ObjectRef]):
         else:
             return self.scheduler.active_plans()
 
-    def run_iter(
-        self, builder: LogicalPlanBuilder, results_buffer_size: int | None = None
-    ) -> Iterator[RayMaterializedResult]:
-        # Grab and freeze the current DaftExecutionConfig
-        daft_execution_config = get_context().daft_execution_config
-
-        # Optimize the logical plan.
-        builder = builder.optimize()
-
-        # Finalize the logical plan and get a physical plan scheduler for translating the
-        # physical plan to executable tasks.
-        plan_scheduler = builder.to_physical_plan_scheduler(daft_execution_config)
-
-        psets = {
-            key: entry.value.values()
-            for key, entry in self._part_set_cache._uuid_to_partition_set.items()
-            if entry.value is not None
-        }
+    def _start_plan(
+        self,
+        plan_scheduler: PhysicalPlanScheduler,
+        daft_execution_config: PyDaftExecutionConfig,
+        results_buffer_size: int | None = None,
+    ) -> str:
+        psets = {k: v.values() for k, v in self._part_set_cache.get_all_partition_sets().items()}
         result_uuid = str(uuid.uuid4())
         if isinstance(self.ray_context, ray.client_builder.ClientContext):
             ray.get(
@@ -751,7 +790,9 @@ class RayRunner(Runner[ray.ObjectRef]):
                 result_uuid=result_uuid,
                 results_buffer_size=results_buffer_size,
             )
+        return result_uuid
 
+    def _stream_plan(self, result_uuid: str) -> Iterator[RayMaterializedResult]:
         try:
             while True:
                 if isinstance(self.ray_context, ray.client_builder.ClientContext):
@@ -772,16 +813,51 @@ class RayRunner(Runner[ray.ObjectRef]):
             else:
                 self.scheduler.stop_plan(result_uuid)
 
+    def run_iter(
+        self, builder: LogicalPlanBuilder, results_buffer_size: int | None = None
+    ) -> Iterator[RayMaterializedResult]:
+        # Grab and freeze the current DaftExecutionConfig
+        daft_execution_config = get_context().daft_execution_config
+
+        # Optimize the logical plan.
+        builder = builder.optimize()
+
+        if daft_execution_config.enable_aqe:
+            adaptive_planner = builder.to_adaptive_physical_plan_scheduler(daft_execution_config)
+            while not adaptive_planner.is_done():
+                source_id, plan_scheduler = adaptive_planner.next()
+                # don't store partition sets in variable to avoid reference
+                result_uuid = self._start_plan(
+                    plan_scheduler, daft_execution_config, results_buffer_size=results_buffer_size
+                )
+                del plan_scheduler
+                results_iter = self._stream_plan(result_uuid)
+                # if source_id is None that means this is the final stage
+                if source_id is None:
+                    yield from results_iter
+                else:
+                    cache_entry = self._collect_into_cache(results_iter)
+                    adaptive_planner.update(source_id, cache_entry)
+                    del cache_entry
+        else:
+            # Finalize the logical plan and get a physical plan scheduler for translating the
+            # physical plan to executable tasks.
+            plan_scheduler = builder.to_physical_plan_scheduler(daft_execution_config)
+
+            result_uuid = self._start_plan(
+                plan_scheduler, daft_execution_config, results_buffer_size=results_buffer_size
+            )
+
+            yield from self._stream_plan(result_uuid)
+
     def run_iter_tables(
         self, builder: LogicalPlanBuilder, results_buffer_size: int | None = None
     ) -> Iterator[MicroPartition]:
         for result in self.run_iter(builder, results_buffer_size=results_buffer_size):
             yield ray.get(result.partition())
 
-    def run(self, builder: LogicalPlanBuilder) -> PartitionCacheEntry:
-        result_pset = RayPartitionSet(_results={})
-
-        results_iter = self.run_iter(builder)
+    def _collect_into_cache(self, results_iter: Iterator[RayMaterializedResult]) -> PartitionCacheEntry:
+        result_pset = RayPartitionSet()
 
         for i, result in enumerate(results_iter):
             result_pset.set_partition(i, result)
@@ -790,17 +866,19 @@ class RayRunner(Runner[ray.ObjectRef]):
 
         return pset_entry
 
+    def run(self, builder: LogicalPlanBuilder) -> PartitionCacheEntry:
+        results_iter = self.run_iter(builder)
+        return self._collect_into_cache(results_iter)
+
     def put_partition_set_into_cache(self, pset: PartitionSet) -> PartitionCacheEntry:
         if isinstance(pset, LocalPartitionSet):
-            pset = RayPartitionSet(
-                _results={
-                    pid: RayMaterializedResult(
-                        ray.put(val),
-                    )
-                    for pid, val in pset._partitions.items()
-                },
-            )
-
+            new_pset = RayPartitionSet()
+            metadata_accessor = PartitionMetadataAccessor.from_metadata_list([v.metadata() for v in pset.values()])
+            for i, (pid, py_mat_result) in enumerate(pset.items()):
+                new_pset.set_partition(
+                    pid, RayMaterializedResult(ray.put(py_mat_result.partition()), metadata_accessor, i)
+                )
+            pset = new_pset
         return self._part_set_cache.put_partition_set(pset=pset)
 
     def runner_io(self) -> RayRunnerIO:
@@ -852,3 +930,10 @@ class PartitionMetadataAccessor:
 
     def get_index(self, key) -> PartitionMetadata:
         return self._get_metadatas()[key]
+
+    @classmethod
+    def from_metadata_list(cls, meta: list[PartitionMetadata]) -> PartitionMetadataAccessor:
+        ref = ray.put(meta)
+        accessor = cls(ref)
+        accessor._metadatas = meta
+        return accessor
