@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import TypeVar
+from typing import Any, Literal, TypeVar
 
 import pyarrow as pa
 
@@ -279,7 +279,7 @@ class Series:
         """
         if self.datatype()._is_python_type():
             return self._series.to_pylist()
-        elif self.datatype()._is_logical_type():
+        elif self.datatype()._should_cast_to_python():
             return self._series.cast(DataType.python()._dtype).to_pylist()
         else:
             return self._series.to_arrow().to_pylist()
@@ -349,6 +349,73 @@ class Series:
     def __abs__(self) -> Series:
         return Series._from_pyseries(abs(self._series))
 
+    def ceil(self) -> Series:
+        return Series._from_pyseries(self._series.ceil())
+
+    def floor(self) -> Series:
+        return Series._from_pyseries(self._series.floor())
+
+    def sign(self) -> Series:
+        return Series._from_pyseries(self._series.sign())
+
+    def round(self, decimal: int) -> Series:
+        return Series._from_pyseries(self._series.round(decimal))
+
+    def sqrt(self) -> Series:
+        return Series._from_pyseries(self._series.sqrt())
+
+    def sin(self) -> Series:
+        """The elementwise sine of a numeric series."""
+        return Series._from_pyseries(self._series.sin())
+
+    def cos(self) -> Series:
+        """The elementwise cosine of a numeric series."""
+        return Series._from_pyseries(self._series.cos())
+
+    def tan(self) -> Series:
+        """The elementwise tangent of a numeric series."""
+        return Series._from_pyseries(self._series.tan())
+
+    def cot(self) -> Series:
+        """The elementwise cotangent of a numeric series"""
+        return Series._from_pyseries(self._series.cot())
+
+    def arcsin(self) -> Series:
+        """The elementwise arc sine of a numeric series"""
+        return Series._from_pyseries(self._series.arcsin())
+
+    def arccos(self) -> Series:
+        """The elementwise arc cosine of a numeric series"""
+        return Series._from_pyseries(self._series.arccos())
+
+    def arctan(self) -> Series:
+        """The elementwise arc tangent of a numeric series"""
+        return Series._from_pyseries(self._series.arctan())
+
+    def radians(self) -> Series:
+        """The elementwise radians of a numeric series"""
+        return Series._from_pyseries(self._series.radians())
+
+    def degrees(self) -> Series:
+        """The elementwise degrees of a numeric series"""
+        return Series._from_pyseries(self._series.degrees())
+
+    def log2(self) -> Series:
+        """The elementwise log2 of a numeric series"""
+        return Series._from_pyseries(self._series.log2())
+
+    def log10(self) -> Series:
+        """The elementwise log10 of a numeric series"""
+        return Series._from_pyseries(self._series.log10())
+
+    def ln(self) -> Series:
+        """The elementwise ln of a numeric series"""
+        return Series._from_pyseries(self._series.ln())
+
+    def exp(self) -> Series:
+        """The e^self of a numeric series"""
+        return Series._from_pyseries(self._series.exp())
+
     def __add__(self, other: object) -> Series:
         if not isinstance(other, Series):
             raise TypeError(f"expected another Series but got {type(other)}")
@@ -437,25 +504,25 @@ class Series:
         assert self._series is not None and other._series is not None
         return Series._from_pyseries(self._series ^ other._series)
 
-    def _count(self, mode: CountMode = CountMode.Valid) -> Series:
+    def count(self, mode: CountMode = CountMode.Valid) -> Series:
         assert self._series is not None
-        return Series._from_pyseries(self._series._count(mode))
+        return Series._from_pyseries(self._series.count(mode))
 
-    def _min(self) -> Series:
+    def min(self) -> Series:
         assert self._series is not None
-        return Series._from_pyseries(self._series._min())
+        return Series._from_pyseries(self._series.min())
 
-    def _max(self) -> Series:
+    def max(self) -> Series:
         assert self._series is not None
-        return Series._from_pyseries(self._series._max())
+        return Series._from_pyseries(self._series.max())
 
-    def _mean(self) -> Series:
+    def mean(self) -> Series:
         assert self._series is not None
-        return Series._from_pyseries(self._series._mean())
+        return Series._from_pyseries(self._series.mean())
 
-    def _sum(self) -> Series:
+    def sum(self) -> Series:
         assert self._series is not None
-        return Series._from_pyseries(self._series._sum())
+        return Series._from_pyseries(self._series.sum())
 
     def if_else(self, if_true: object, if_false: object) -> Series:
         if not isinstance(if_true, Series):
@@ -474,6 +541,15 @@ class Series:
         assert self._series is not None
         return Series._from_pyseries(self._series.not_null())
 
+    def fill_null(self, fill_value: object) -> Series:
+        if not isinstance(fill_value, Series):
+            raise ValueError(f"expected another Series but got {type(fill_value)}")
+        assert self._series is not None and fill_value._series is not None
+        return Series._from_pyseries(self._series.fill_null(fill_value._series))
+
+    def _to_str_values(self) -> Series:
+        return Series._from_pyseries(self._series.to_str_values())
+
     @property
     def float(self) -> SeriesFloatNamespace:
         return SeriesFloatNamespace.from_series(self)
@@ -490,6 +566,10 @@ class Series:
     def list(self) -> SeriesListNamespace:
         return SeriesListNamespace.from_series(self)
 
+    @property
+    def map(self) -> SeriesMapNamespace:
+        return SeriesMapNamespace.from_series(self)
+
     @property
     def image(self) -> SeriesImageNamespace:
         return SeriesImageNamespace.from_series(self)
@@ -512,6 +592,22 @@ class Series:
         return Series._from_pyseries(PySeries._debug_bincode_deserialize(b))
 
 
+def item_to_series(name: str, item: Any) -> Series:
+    if isinstance(item, list):
+        series = Series.from_pylist(item, name)
+    elif _NUMPY_AVAILABLE and isinstance(item, np.ndarray):
+        series = Series.from_numpy(item, name)
+    elif isinstance(item, Series):
+        series = item
+    elif isinstance(item, (pa.Array, pa.ChunkedArray)):
+        series = Series.from_arrow(item, name)
+    elif _PANDAS_AVAILABLE and isinstance(item, pd.Series):
+        series = Series.from_pandas(item, name)
+    else:
+        raise ValueError(f"Creating a Series from data of type {type(item)} not implemented")
+    return series
+
+
 SomeSeriesNamespace = TypeVar("SomeSeriesNamespace", bound="SeriesNamespace")
 
 
@@ -552,11 +648,17 @@ class SeriesStringNamespace(SeriesNamespace):
         assert self._series is not None and pattern._series is not None
         return Series._from_pyseries(self._series.utf8_contains(pattern._series))
 
-    def split(self, pattern: Series) -> Series:
+    def match(self, pattern: Series) -> Series:
         if not isinstance(pattern, Series):
             raise ValueError(f"expected another Series but got {type(pattern)}")
         assert self._series is not None and pattern._series is not None
-        return Series._from_pyseries(self._series.utf8_split(pattern._series))
+        return Series._from_pyseries(self._series.utf8_match(pattern._series))
+
+    def split(self, pattern: Series, regex: bool = False) -> Series:
+        if not isinstance(pattern, Series):
+            raise ValueError(f"expected another Series but got {type(pattern)}")
+        assert self._series is not None and pattern._series is not None
+        return Series._from_pyseries(self._series.utf8_split(pattern._series, regex))
 
     def concat(self, other: Series) -> Series:
         if not isinstance(other, Series):
@@ -564,10 +666,106 @@ class SeriesStringNamespace(SeriesNamespace):
         assert self._series is not None and other._series is not None
         return Series._from_pyseries(self._series) + other
 
+    def extract(self, pattern: Series, index: int = 0) -> Series:
+        if not isinstance(pattern, Series):
+            raise ValueError(f"expected another Series but got {type(pattern)}")
+        assert self._series is not None and pattern._series is not None
+        return Series._from_pyseries(self._series.utf8_extract(pattern._series, index))
+
+    def extract_all(self, pattern: Series, index: int = 0) -> Series:
+        if not isinstance(pattern, Series):
+            raise ValueError(f"expected another Series but got {type(pattern)}")
+        assert self._series is not None and pattern._series is not None
+        return Series._from_pyseries(self._series.utf8_extract_all(pattern._series, index))
+
+    def replace(self, pattern: Series, replacement: Series, regex: bool = False) -> Series:
+        if not isinstance(pattern, Series):
+            raise ValueError(f"expected another Series but got {type(pattern)}")
+        if not isinstance(replacement, Series):
+            raise ValueError(f"expected another Series but got {type(replacement)}")
+        assert self._series is not None and pattern._series is not None
+        return Series._from_pyseries(self._series.utf8_replace(pattern._series, replacement._series, regex))
+
     def length(self) -> Series:
         assert self._series is not None
         return Series._from_pyseries(self._series.utf8_length())
 
+    def lower(self) -> Series:
+        assert self._series is not None
+        return Series._from_pyseries(self._series.utf8_lower())
+
+    def upper(self) -> Series:
+        assert self._series is not None
+        return Series._from_pyseries(self._series.utf8_upper())
+
+    def lstrip(self) -> Series:
+        assert self._series is not None
+        return Series._from_pyseries(self._series.utf8_lstrip())
+
+    def rstrip(self) -> Series:
+        assert self._series is not None
+        return Series._from_pyseries(self._series.utf8_rstrip())
+
+    def reverse(self) -> Series:
+        assert self._series is not None
+        return Series._from_pyseries(self._series.utf8_reverse())
+
+    def capitalize(self) -> Series:
+        assert self._series is not None
+        return Series._from_pyseries(self._series.utf8_capitalize())
+
+    def left(self, nchars: Series) -> Series:
+        if not isinstance(nchars, Series):
+            raise ValueError(f"expected another Series but got {type(nchars)}")
+        assert self._series is not None and nchars._series is not None
+        return Series._from_pyseries(self._series.utf8_left(nchars._series))
+
+    def right(self, nchars: Series) -> Series:
+        if not isinstance(nchars, Series):
+            raise ValueError(f"expected another Series but got {type(nchars)}")
+        assert self._series is not None and nchars._series is not None
+        return Series._from_pyseries(self._series.utf8_right(nchars._series))
+
+    def find(self, substr: Series) -> Series:
+        if not isinstance(substr, Series):
+            raise ValueError(f"expected another Series but got {type(substr)}")
+        assert self._series is not None and substr._series is not None
+        return Series._from_pyseries(self._series.utf8_find(substr._series))
+
+    def rpad(self, length: Series, pad: Series) -> Series:
+        if not isinstance(length, Series):
+            raise ValueError(f"expected another Series but got {type(length)}")
+        if not isinstance(pad, Series):
+            raise ValueError(f"expected another Series but got {type(pad)}")
+        assert self._series is not None and length._series is not None and pad._series is not None
+        return Series._from_pyseries(self._series.utf8_rpad(length._series, pad._series))
+
+    def lpad(self, length: Series, pad: Series) -> Series:
+        if not isinstance(length, Series):
+            raise ValueError(f"expected another Series but got {type(length)}")
+        if not isinstance(pad, Series):
+            raise ValueError(f"expected another Series but got {type(pad)}")
+        assert self._series is not None and length._series is not None and pad._series is not None
+        return Series._from_pyseries(self._series.utf8_lpad(length._series, pad._series))
+
+    def repeat(self, n: Series) -> Series:
+        if not isinstance(n, Series):
+            raise ValueError(f"expected another Series but got {type(n)}")
+        assert self._series is not None and n._series is not None
+        return Series._from_pyseries(self._series.utf8_repeat(n._series))
+
+    def like(self, pattern: Series) -> Series:
+        if not isinstance(pattern, Series):
+            raise ValueError(f"expected another Series but got {type(pattern)}")
+        assert self._series is not None and pattern._series is not None
+        return Series._from_pyseries(self._series.utf8_like(pattern._series))
+
+    def ilike(self, pattern: Series) -> Series:
+        if not isinstance(pattern, Series):
+            raise ValueError(f"expected another Series but got {type(pattern)}")
+        assert self._series is not None and pattern._series is not None
+        return Series._from_pyseries(self._series.utf8_ilike(pattern._series))
+
 
 class SeriesDateNamespace(SeriesNamespace):
     def date(self) -> Series:
@@ -579,6 +777,15 @@ class SeriesDateNamespace(SeriesNamespace):
     def hour(self) -> Series:
         return Series._from_pyseries(self._series.dt_hour())
 
+    def minute(self) -> Series:
+        return Series._from_pyseries(self._series.dt_minute())
+
+    def second(self) -> Series:
+        return Series._from_pyseries(self._series.dt_second())
+
+    def time(self) -> Series:
+        return Series._from_pyseries(self._series.dt_time())
+
     def month(self) -> Series:
         return Series._from_pyseries(self._series.dt_month())
 
@@ -588,6 +795,13 @@ class SeriesDateNamespace(SeriesNamespace):
     def day_of_week(self) -> Series:
         return Series._from_pyseries(self._series.dt_day_of_week())
 
+    def truncate(self, interval: str, relative_to: Series | None = None) -> Series:
+        if relative_to is not None and not isinstance(relative_to, Series):
+            raise ValueError(f"expected another Series but got {type(relative_to)}")
+        if relative_to is None:
+            relative_to = Series.from_arrow(pa.array([None]))
+        return Series._from_pyseries(self._series.dt_truncate(interval, relative_to._series))
+
 
 class SeriesPartitioningNamespace(SeriesNamespace):
     def days(self) -> Series:
@@ -611,15 +825,27 @@ class SeriesPartitioningNamespace(SeriesNamespace):
 
 class SeriesListNamespace(SeriesNamespace):
     def lengths(self) -> Series:
-        return Series._from_pyseries(self._series.list_lengths())
+        return Series._from_pyseries(self._series.list_count(CountMode.All))
 
     def get(self, idx: Series, default: Series) -> Series:
         return Series._from_pyseries(self._series.list_get(idx._series, default._series))
 
 
+class SeriesMapNamespace(SeriesNamespace):
+    def get(self, key: Series) -> Series:
+        return Series._from_pyseries(self._series.map_get(key._series))
+
+
 class SeriesImageNamespace(SeriesNamespace):
-    def decode(self) -> Series:
-        return Series._from_pyseries(self._series.image_decode())
+    def decode(self, on_error: Literal["raise"] | Literal["null"] = "raise") -> Series:
+        raise_on_error = False
+        if on_error == "raise":
+            raise_on_error = True
+        elif on_error == "null":
+            raise_on_error = False
+        else:
+            raise NotImplementedError(f"Unimplemented on_error option: {on_error}.")
+        return Series._from_pyseries(self._series.image_decode(raise_error_on_failure=raise_on_error))
 
     def encode(self, image_format: str | ImageFormat) -> Series:
         if isinstance(image_format, str):
