@@ -222,6 +222,16 @@ class MicroPartition:
         group_by_pyexprs = [e._expr for e in group_by] if group_by is not None else []
         return MicroPartition._from_pymicropartition(self._micropartition.agg(to_agg_pyexprs, group_by_pyexprs))
 
+    def pivot(
+        self, group_by: ExpressionsProjection, pivot_column: Expression, values_column: Expression, names: list[str]
+    ) -> MicroPartition:
+        group_by_pyexprs = [e._expr for e in group_by]
+        pivot_column_pyexpr = pivot_column._expr
+        values_column_pyexpr = values_column._expr
+        return MicroPartition._from_pymicropartition(
+            self._micropartition.pivot(group_by_pyexprs, pivot_column_pyexpr, values_column_pyexpr, names)
+        )
+
     def quantiles(self, num: int) -> MicroPartition:
         return MicroPartition._from_pymicropartition(self._micropartition.quantiles(num))
 
@@ -230,12 +240,44 @@ class MicroPartition:
         to_explode_pyexprs = [e._expr for e in columns]
         return MicroPartition._from_pymicropartition(self._micropartition.explode(to_explode_pyexprs))
 
-    def join(
+    def unpivot(
+        self, ids: ExpressionsProjection, values: ExpressionsProjection, variable_name: str, value_name: str
+    ) -> MicroPartition:
+        ids_pyexprs = [e._expr for e in ids]
+        values_pyexprs = [e._expr for e in values]
+        return MicroPartition._from_pymicropartition(
+            self._micropartition.unpivot(ids_pyexprs, values_pyexprs, variable_name, value_name)
+        )
+
+    def hash_join(
         self,
         right: MicroPartition,
         left_on: ExpressionsProjection,
         right_on: ExpressionsProjection,
         how: JoinType = JoinType.Inner,
+    ) -> MicroPartition:
+        if len(left_on) != len(right_on):
+            raise ValueError(
+                f"Mismatch of number of join keys, left_on: {len(left_on)}, right_on: {len(right_on)}\nleft_on {left_on}\nright_on {right_on}"
+            )
+
+        if not isinstance(right, MicroPartition):
+            raise TypeError(f"Expected a MicroPartition for `right` in join but got {type(right)}")
+
+        left_exprs = [e._expr for e in left_on]
+        right_exprs = [e._expr for e in right_on]
+
+        return MicroPartition._from_pymicropartition(
+            self._micropartition.hash_join(right._micropartition, left_on=left_exprs, right_on=right_exprs, how=how)
+        )
+
+    def sort_merge_join(
+        self,
+        right: MicroPartition,
+        left_on: ExpressionsProjection,
+        right_on: ExpressionsProjection,
+        how: JoinType = JoinType.Inner,
+        is_sorted: bool = False,
     ) -> MicroPartition:
         if how != JoinType.Inner:
             raise NotImplementedError("TODO: [RUST] Implement Other Join types")
@@ -251,7 +293,9 @@ class MicroPartition:
         right_exprs = [e._expr for e in right_on]
 
         return MicroPartition._from_pymicropartition(
-            self._micropartition.join(right._micropartition, left_on=left_exprs, right_on=right_exprs)
+            self._micropartition.sort_merge_join(
+                right._micropartition, left_on=left_exprs, right_on=right_exprs, is_sorted=is_sorted
+            )
         )
 
     def partition_by_hash(self, exprs: ExpressionsProjection, num_partitions: int) -> list[MicroPartition]:
@@ -288,6 +332,18 @@ class MicroPartition:
             for t in self._micropartition.partition_by_random(num_partitions, seed)
         ]
 
+    def partition_by_value(self, partition_keys: ExpressionsProjection) -> tuple[list[MicroPartition], MicroPartition]:
+        exprs = [e._expr for e in partition_keys]
+        pytables, values = self._micropartition.partition_by_value(exprs)
+        return [MicroPartition._from_pymicropartition(t) for t in pytables], MicroPartition._from_pymicropartition(
+            values
+        )
+
+    def add_monotonically_increasing_id(self, partition_num: int, column_name: str) -> MicroPartition:
+        return MicroPartition._from_pymicropartition(
+            self._micropartition.add_monotonically_increasing_id(partition_num, column_name)
+        )
+
     ###
     # Compute methods (MicroPartition -> Series)
     ###
