@@ -27,7 +27,7 @@ from daft.daft import read_parquet_statistics as _read_parquet_statistics
 from daft.datatype import DataType, TimeUnit
 from daft.expressions import Expression, ExpressionsProjection
 from daft.logical.schema import Schema
-from daft.series import Series
+from daft.series import Series, item_to_series
 
 _NUMPY_AVAILABLE = True
 try:
@@ -148,18 +148,7 @@ class Table:
     def from_pydict(data: dict) -> Table:
         series_dict = dict()
         for k, v in data.items():
-            if isinstance(v, list):
-                series = Series.from_pylist(v, name=k)
-            elif _NUMPY_AVAILABLE and isinstance(v, np.ndarray):
-                series = Series.from_numpy(v, name=k)
-            elif isinstance(v, Series):
-                series = v
-            elif isinstance(v, (pa.Array, pa.ChunkedArray)):
-                series = Series.from_arrow(v, name=k)
-            elif _PANDAS_AVAILABLE and isinstance(v, pd.Series):
-                series = Series.from_pandas(v, name=k)
-            else:
-                raise ValueError(f"Creating a Series from data of type {type(v)} not implemented")
+            series = item_to_series(k, v)
             series_dict[k] = series._series
         return Table._from_pytable(_PyTable.from_pylist_series(series_dict))
 
@@ -317,6 +306,12 @@ class Table:
         group_by_pyexprs = [e._expr for e in group_by] if group_by is not None else []
         return Table._from_pytable(self._table.agg(to_agg_pyexprs, group_by_pyexprs))
 
+    def pivot(
+        self, group_by: ExpressionsProjection, pivot_column: Expression, values_column: Expression, names: list[str]
+    ) -> Table:
+        group_by_pyexprs = [e._expr for e in group_by]
+        return Table._from_pytable(self._table.pivot(group_by_pyexprs, pivot_column._expr, values_column._expr, names))
+
     def quantiles(self, num: int) -> Table:
         return Table._from_pytable(self._table.quantiles(num))
 
@@ -325,12 +320,35 @@ class Table:
         to_explode_pyexprs = [e._expr for e in columns]
         return Table._from_pytable(self._table.explode(to_explode_pyexprs))
 
-    def join(
+    def hash_join(
         self,
         right: Table,
         left_on: ExpressionsProjection,
         right_on: ExpressionsProjection,
         how: JoinType = JoinType.Inner,
+    ) -> Table:
+        if len(left_on) != len(right_on):
+            raise ValueError(
+                f"Mismatch of number of join keys, left_on: {len(left_on)}, right_on: {len(right_on)}\nleft_on {left_on}\nright_on {right_on}"
+            )
+
+        if not isinstance(right, Table):
+            raise TypeError(f"Expected a Table for `right` in join but got {type(right)}")
+
+        left_exprs = [e._expr for e in left_on]
+        right_exprs = [e._expr for e in right_on]
+
+        return Table._from_pytable(
+            self._table.hash_join(right._table, left_on=left_exprs, right_on=right_exprs, how=how)
+        )
+
+    def sort_merge_join(
+        self,
+        right: Table,
+        left_on: ExpressionsProjection,
+        right_on: ExpressionsProjection,
+        how: JoinType = JoinType.Inner,
+        is_sorted: bool = False,
     ) -> Table:
         if how != JoinType.Inner:
             raise NotImplementedError("TODO: [RUST] Implement Other Join types")
@@ -345,7 +363,9 @@ class Table:
         left_exprs = [e._expr for e in left_on]
         right_exprs = [e._expr for e in right_on]
 
-        return Table._from_pytable(self._table.join(right._table, left_on=left_exprs, right_on=right_exprs))
+        return Table._from_pytable(
+            self._table.sort_merge_join(right._table, left_on=left_exprs, right_on=right_exprs, is_sorted=is_sorted)
+        )
 
     def partition_by_hash(self, exprs: ExpressionsProjection, num_partitions: int) -> list[Table]:
         if not isinstance(num_partitions, int):
@@ -372,6 +392,15 @@ class Table:
 
         return [Table._from_pytable(t) for t in self._table.partition_by_random(num_partitions, seed)]
 
+    def partition_by_value(self, partition_keys: ExpressionsProjection) -> tuple[list[Table], Table]:
+        exprs = [e._expr for e in partition_keys]
+        pytables, values = self._table.partition_by_value(exprs)
+
+        return [Table._from_pytable(t) for t in pytables], Table._from_pytable(values)
+
+    def add_monotonically_increasing_id(self, partition_num: int, column_name: str) -> Table:
+        return Table._from_pytable(self._table.add_monotonically_increasing_id(partition_num, column_name))
+
     ###
     # Compute methods (Table -> Series)
     ###
@@ -548,6 +577,7 @@ def read_parquet_into_pyarrow(
     io_config: IOConfig | None = None,
     multithreaded_io: bool | None = None,
     coerce_int96_timestamp_unit: TimeUnit = TimeUnit.ns(),
+    file_timeout_ms: int | None = 900_000,  # 15 minutes
 ) -> pa.Table:
     fields, metadata, columns = _read_parquet_into_pyarrow(
         uri=path,
@@ -558,6 +588,7 @@ def read_parquet_into_pyarrow(
         io_config=io_config,
         multithreaded_io=multithreaded_io,
         coerce_int96_timestamp_unit=coerce_int96_timestamp_unit._timeunit,
+        file_timeout_ms=file_timeout_ms,
     )
     schema = pa.schema(fields, metadata=metadata)
     columns = [pa.chunked_array(c, type=f.type) for f, c in zip(schema, columns)]  # type: ignore
@@ -587,6 +618,9 @@ def read_parquet_into_pyarrow_bulk(
         coerce_int96_timestamp_unit=coerce_int96_timestamp_unit._timeunit,
     )
     return [
-        pa.table([pa.chunked_array(c, type=f.type) for f, c in zip(fields, columns)], schema=pa.schema(fields, metadata=metadata))  # type: ignore
+        pa.table(
+            [pa.chunked_array(c, type=f.type) for f, c in zip(fields, columns)],
+            schema=pa.schema(fields, metadata=metadata),
+        )  # type: ignore
         for fields, metadata, columns in bulk_result
     ]
