@@ -4,7 +4,7 @@ import dataclasses
 import functools
 import inspect
 import types
-from typing import Callable
+from typing import TYPE_CHECKING, Callable, Union
 
 from daft.datatype import DataType
 from daft.expressions import Expression
@@ -16,7 +16,17 @@ try:
 except ImportError:
     _NUMPY_AVAILABLE = False
 
-UserProvidedPythonFunction = Callable[..., Series]
+_PYARROW_AVAILABLE = True
+try:
+    import pyarrow as pa
+except ImportError:
+    _PYARROW_AVAILABLE = False
+
+if TYPE_CHECKING:
+    import numpy as np
+    import pyarrow as pa
+
+UserProvidedPythonFunction = Callable[..., Union[Series, "np.ndarray", list]]
 
 
 @dataclasses.dataclass(frozen=True)
@@ -27,11 +37,42 @@ class PartialUDF:
     bound_args: inspect.BoundArguments
 
     def expressions(self) -> dict[str, Expression]:
-        return {key: val for key, val in self.bound_args.arguments.items() if isinstance(val, Expression)}
+        parsed_expressions = {}
+        signature = self.bound_args.signature
+
+        for key, val in self.bound_args.arguments.items():
+            # If the argument is VAR_POSITIONAL (e.g. `*args`), we parse each
+            # entry in the tuple to find any expressions
+            if signature.parameters[key].kind == inspect.Parameter.VAR_POSITIONAL:
+                for idx, x in enumerate(val):
+                    if isinstance(x, Expression):
+                        parsed_expressions[f"{key}-{idx}"] = x
+            # If the key is VAR_KEYWORD (e.g. `**kwargs`), we parse each entry
+            # in the dict to find any expressions
+            elif signature.parameters[key].kind == inspect.Parameter.VAR_KEYWORD:
+                for kwarg_key, x in val.items():
+                    if isinstance(x, Expression):
+                        parsed_expressions[kwarg_key] = x
+            elif isinstance(val, Expression):
+                parsed_expressions[key] = val
+
+        return parsed_expressions
+
+    def arg_keys(self) -> list[str]:
+        parsed_arg_keys = []
+        signature = self.bound_args.signature
+        for key, value in self.bound_args.arguments.items():
+            if signature.parameters[key].kind == inspect.Parameter.VAR_POSITIONAL:
+                for idx, _ in enumerate(value):
+                    parsed_arg_keys.append(f"{key}-{idx}")
+            elif key not in self.bound_args.kwargs and signature.parameters[key].kind != inspect.Parameter.VAR_KEYWORD:
+                parsed_arg_keys.append(key)
+
+        return parsed_arg_keys
 
     def __call__(self, evaluated_expressions: list[Series]) -> PySeries:
         kwarg_keys = list(self.bound_args.kwargs.keys())
-        arg_keys = [k for k in self.bound_args.arguments.keys() if k not in self.bound_args.kwargs.keys()]
+        arg_keys = self.arg_keys()
         pyvalues = {key: val for key, val in self.bound_args.arguments.items() if not isinstance(val, Expression)}
         expressions = self.expressions()
         assert len(evaluated_expressions) == len(
@@ -80,6 +121,8 @@ class PartialUDF:
                 return Series.from_pylist(result, name=name, pyobj="allow").cast(self.udf.return_dtype)._series
         elif _NUMPY_AVAILABLE and isinstance(result, np.ndarray):
             return Series.from_numpy(result, name=name).cast(self.udf.return_dtype)._series
+        elif _PYARROW_AVAILABLE and isinstance(result, (pa.Array, pa.ChunkedArray)):
+            return Series.from_arrow(result, name=name).cast(self.udf.return_dtype)._series
         else:
             raise NotImplementedError(f"Return type not supported for UDF: {type(result)}")
 
@@ -98,7 +141,7 @@ class UDF:
     return_dtype: DataType
 
     def __post_init__(self):
-        """Analagous to the @functools.wraps(self.func) pattern
+        """Analogous to the @functools.wraps(self.func) pattern
 
         This will swap out identifiers on `self` to match `self.func`. Most notably, this swaps out
         self.__module__ and self.__qualname__, which is used in `__reduce__` during serialization.
@@ -115,7 +158,7 @@ class UDF:
             return_dtype=self.return_dtype,
         )
 
-    def bind_func(self, *args, **kwargs):
+    def bind_func(self, *args, **kwargs) -> inspect.BoundArguments:
         if isinstance(self.func, types.FunctionType):
             sig = inspect.signature(self.func)
             bound_args = sig.bind(*args, **kwargs)
