@@ -83,16 +83,31 @@ def map_operator_arrow_semantics_bool(
     right_pylist: list,
 ) -> list[bool | None]:
     return [
-        bool(operator(l, r)) if (l is not None and r is not None) else None for (l, r) in zip(left_pylist, right_pylist)
+        bool(operator(left, right)) if (left is not None and right is not None) else None
+        for (left, right) in zip(left_pylist, right_pylist)
     ]
 
 
+def python_list_membership_check(
+    left_pylist: list,
+    right_pylist: list,
+) -> list:
+    try:
+        right_pyset = set(right_pylist)
+        return [elem in right_pyset for elem in left_pylist]
+    except TypeError:
+        return [elem in right_pylist for elem in left_pylist]
+
+
 def map_operator_arrow_semantics(
     operator: Callable[[Any, Any], Any],
     left_pylist: list,
     right_pylist: list,
 ) -> list:
-    return [operator(l, r) if (l is not None and r is not None) else None for (l, r) in zip(left_pylist, right_pylist)]
+    return [
+        operator(left, right) if (left is not None and right is not None) else None
+        for (left, right) in zip(left_pylist, right_pylist)
+    ]
 
 
 def pyarrow_supports_fixed_shape_tensor() -> bool:
