@@ -18,7 +18,6 @@ if HAS_PILLOW:
 
 @dataclass(frozen=True)
 class DataFrameDisplay:
-
     preview: DataFramePreview
     schema: Schema
     # These formatting options are deprecated for now and not guaranteed to be supported.
@@ -57,7 +56,7 @@ class DataFrameDisplay:
         if self.preview.preview_partition is not None:
             res = repr(self.preview.preview_partition.to_table())
         else:
-            res = repr(self.schema)
+            res = self.schema._truncated_table_string()
 
         res += f"\n{self._get_user_message()}"
 
