@@ -21,8 +21,12 @@ use crate::config;
 ///     num_tries: Number of attempts to make a connection, defaults to 5
 ///     retry_mode: Retry Mode when a request fails, current supported values are `standard` and `adaptive`, defaults to `adaptive`
 ///     anonymous: Whether or not to use "anonymous mode", which will access S3 without any credentials
+///     use_ssl: Whether or not to use SSL, which require accessing S3 over HTTPS rather than HTTP, defaults to True
 ///     verify_ssl: Whether or not to verify ssl certificates, which will access S3 without checking if the certs are valid, defaults to True
 ///     check_hostname_ssl: Whether or not to verify the hostname when verifying ssl certificates, this was the legacy behavior for openssl, defaults to True
+///     requester_pays: Whether or not the authenticated user will assume transfer costs, which is required by some providers of bulk data, defaults to False
+///     force_virtual_addressing: Force S3 client to use virtual addressing in all cases. If False, virtual addressing will only be used if `endpoint_url` is empty, defaults to False
+///     profile_name: Name of AWS_PROFILE to load, defaults to None which will then check the Environment Variable `AWS_PROFILE` then fall back to `default`
 ///
 /// Example:
 ///     >>> io_config = IOConfig(s3=S3Config(key_id="xxx", access_key="xxx"))
@@ -180,8 +184,12 @@ impl S3Config {
         num_tries: Option<u32>,
         retry_mode: Option<String>,
         anonymous: Option<bool>,
+        use_ssl: Option<bool>,
         verify_ssl: Option<bool>,
         check_hostname_ssl: Option<bool>,
+        requester_pays: Option<bool>,
+        force_virtual_addressing: Option<bool>,
+        profile_name: Option<String>,
     ) -> Self {
         let def = crate::S3Config::default();
         S3Config {
@@ -200,8 +208,13 @@ impl S3Config {
                 num_tries: num_tries.unwrap_or(def.num_tries),
                 retry_mode: retry_mode.or(def.retry_mode),
                 anonymous: anonymous.unwrap_or(def.anonymous),
+                use_ssl: use_ssl.unwrap_or(def.use_ssl),
                 verify_ssl: verify_ssl.unwrap_or(def.verify_ssl),
                 check_hostname_ssl: check_hostname_ssl.unwrap_or(def.check_hostname_ssl),
+                requester_pays: requester_pays.unwrap_or(def.requester_pays),
+                force_virtual_addressing: force_virtual_addressing
+                    .unwrap_or(def.force_virtual_addressing),
+                profile_name: profile_name.or(def.profile_name),
             },
         }
     }
@@ -221,8 +234,12 @@ impl S3Config {
         num_tries: Option<u32>,
         retry_mode: Option<String>,
         anonymous: Option<bool>,
+        use_ssl: Option<bool>,
         verify_ssl: Option<bool>,
         check_hostname_ssl: Option<bool>,
+        requester_pays: Option<bool>,
+        force_virtual_addressing: Option<bool>,
+        profile_name: Option<String>,
     ) -> Self {
         S3Config {
             config: crate::S3Config {
@@ -240,12 +257,32 @@ impl S3Config {
                 num_tries: num_tries.unwrap_or(self.config.num_tries),
                 retry_mode: retry_mode.or_else(|| self.config.retry_mode.clone()),
                 anonymous: anonymous.unwrap_or(self.config.anonymous),
+                use_ssl: use_ssl.unwrap_or(self.config.use_ssl),
                 verify_ssl: verify_ssl.unwrap_or(self.config.verify_ssl),
                 check_hostname_ssl: check_hostname_ssl.unwrap_or(self.config.check_hostname_ssl),
+                requester_pays: requester_pays.unwrap_or(self.config.requester_pays),
+                force_virtual_addressing: force_virtual_addressing
+                    .unwrap_or(self.config.force_virtual_addressing),
+                profile_name: profile_name.or_else(|| self.config.profile_name.clone()),
             },
         }
     }
 
+    /// Creates an S3Config from the current environment, auto-discovering variables such as
+    /// credentials, regions and more.
+    #[staticmethod]
+    pub fn from_env(py: Python) -> PyResult<Self> {
+        let io_config_from_env_func = py
+            .import(pyo3::intern!(py, "daft"))?
+            .getattr(pyo3::intern!(py, "daft"))?
+            .getattr(pyo3::intern!(py, "s3_config_from_env"))?;
+        io_config_from_env_func.call0().map(|pyany| {
+            pyany
+                .extract()
+                .expect("s3_config_from_env function must return S3Config")
+        })
+    }
+
     pub fn __repr__(&self) -> PyResult<String> {
         Ok(format!("{}", self.config))
     }
@@ -322,6 +359,12 @@ impl S3Config {
         Ok(Some(self.config.anonymous))
     }
 
+    /// AWS Use SSL
+    #[getter]
+    pub fn use_ssl(&self) -> PyResult<Option<bool>> {
+        Ok(Some(self.config.use_ssl))
+    }
+
     /// AWS Verify SSL
     #[getter]
     pub fn verify_ssl(&self) -> PyResult<Option<bool>> {
@@ -333,6 +376,24 @@ impl S3Config {
     pub fn check_hostname_ssl(&self) -> PyResult<Option<bool>> {
         Ok(Some(self.config.check_hostname_ssl))
     }
+
+    /// AWS Requester Pays
+    #[getter]
+    pub fn requester_pays(&self) -> PyResult<Option<bool>> {
+        Ok(Some(self.config.requester_pays))
+    }
+
+    /// AWS force virtual addressing
+    #[getter]
+    pub fn force_virtual_addressing(&self) -> PyResult<Option<bool>> {
+        Ok(Some(self.config.force_virtual_addressing))
+    }
+
+    /// AWS profile name
+    #[getter]
+    pub fn profile_name(&self) -> PyResult<Option<String>> {
+        Ok(self.config.profile_name.clone())
+    }
 }
 
 #[pymethods]
@@ -343,6 +404,8 @@ impl AzureConfig {
         storage_account: Option<String>,
         access_key: Option<String>,
         anonymous: Option<bool>,
+        endpoint_url: Option<String>,
+        use_ssl: Option<bool>,
     ) -> Self {
         let def = crate::AzureConfig::default();
         AzureConfig {
@@ -350,6 +413,8 @@ impl AzureConfig {
                 storage_account: storage_account.or(def.storage_account),
                 access_key: access_key.or(def.access_key),
                 anonymous: anonymous.unwrap_or(def.anonymous),
+                endpoint_url: endpoint_url.or(def.endpoint_url),
+                use_ssl: use_ssl.unwrap_or(def.use_ssl),
             },
         }
     }
@@ -359,12 +424,16 @@ impl AzureConfig {
         storage_account: Option<String>,
         access_key: Option<String>,
         anonymous: Option<bool>,
+        endpoint_url: Option<String>,
+        use_ssl: Option<bool>,
     ) -> Self {
         AzureConfig {
             config: crate::AzureConfig {
                 storage_account: storage_account.or_else(|| self.config.storage_account.clone()),
                 access_key: access_key.or_else(|| self.config.access_key.clone()),
                 anonymous: anonymous.unwrap_or(self.config.anonymous),
+                endpoint_url: endpoint_url.or_else(|| self.config.endpoint_url.clone()),
+                use_ssl: use_ssl.unwrap_or(self.config.use_ssl),
             },
         }
     }
@@ -384,6 +453,24 @@ impl AzureConfig {
     pub fn access_key(&self) -> PyResult<Option<String>> {
         Ok(self.config.access_key.clone())
     }
+
+    /// Whether access is anonymous
+    #[getter]
+    pub fn anonymous(&self) -> PyResult<bool> {
+        Ok(self.config.anonymous)
+    }
+
+    /// Azure Secret Access Key
+    #[getter]
+    pub fn endpoint_url(&self) -> PyResult<Option<String>> {
+        Ok(self.config.endpoint_url.clone())
+    }
+
+    /// Whether SSL (HTTPS) is required
+    #[getter]
+    pub fn use_ssl(&self) -> PyResult<bool> {
+        Ok(self.config.use_ssl)
+    }
 }
 
 #[pymethods]
