@@ -14,7 +14,11 @@ pub struct FixedSizeListArray {
     validity: Option<arrow2::bitmap::Bitmap>,
 }
 
-impl DaftArrayType for FixedSizeListArray {}
+impl DaftArrayType for FixedSizeListArray {
+    fn data_type(&self) -> &DataType {
+        &self.field.as_ref().dtype
+    }
+}
 
 impl FixedSizeListArray {
     pub fn new<F: Into<Arc<Field>>>(
@@ -25,7 +29,9 @@ impl FixedSizeListArray {
         let field: Arc<Field> = field.into();
         match &field.as_ref().dtype {
             DataType::FixedSizeList(child_dtype, size) => {
-                if let Some(validity) = validity.as_ref() && (validity.len() * size) != flat_child.len() {
+                if let Some(validity) = validity.as_ref()
+                    && (validity.len() * size) != flat_child.len()
+                {
                     panic!(
                         "FixedSizeListArray::new received values with len {} but expected it to match len of validity * size: {}",
                         flat_child.len(),
@@ -63,7 +69,7 @@ impl FixedSizeListArray {
             ));
         }
 
-        let first_array = arrays.get(0).unwrap();
+        let first_array = arrays.first().unwrap();
         let mut growable = <Self as GrowableArray>::make_growable(
             first_array.field.name.as_str(),
             &first_array.field.dtype,
@@ -150,12 +156,14 @@ impl FixedSizeListArray {
     }
 
     pub fn with_validity(&self, validity: Option<arrow2::bitmap::Bitmap>) -> DaftResult<Self> {
-        if let Some(v) = &validity && v.len() != self.len() {
+        if let Some(v) = &validity
+            && v.len() != self.len()
+        {
             return Err(DaftError::ValueError(format!(
                 "validity mask length does not match FixedSizeListArray length, {} vs {}",
                 v.len(),
                 self.len()
-            )))
+            )));
         }
 
         Ok(Self::new(
@@ -166,6 +174,49 @@ impl FixedSizeListArray {
     }
 }
 
+impl<'a> IntoIterator for &'a FixedSizeListArray {
+    type Item = Option<Series>;
+
+    type IntoIter = FixedSizeListArrayIter<'a>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        FixedSizeListArrayIter {
+            array: self,
+            idx: 0,
+        }
+    }
+}
+
+pub struct FixedSizeListArrayIter<'a> {
+    array: &'a FixedSizeListArray,
+    idx: usize,
+}
+
+impl Iterator for FixedSizeListArrayIter<'_> {
+    type Item = Option<Series>;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.idx < self.array.len() {
+            if let Some(validity) = self.array.validity()
+                && !validity.get_bit(self.idx)
+            {
+                self.idx += 1;
+                Some(None)
+            } else {
+                let step = self.array.fixed_element_len();
+
+                let start = self.idx * step;
+                let end = (self.idx + 1) * step;
+
+                self.idx += 1;
+                Some(Some(self.array.flat_child.slice(start, end).unwrap()))
+            }
+        } else {
+            None
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use common_error::DaftResult;
