@@ -15,7 +15,11 @@ pub struct ListArray {
     validity: Option<arrow2::bitmap::Bitmap>,
 }
 
-impl DaftArrayType for ListArray {}
+impl DaftArrayType for ListArray {
+    fn data_type(&self) -> &DataType {
+        &self.field.as_ref().dtype
+    }
+}
 
 impl ListArray {
     pub fn new<F: Into<Arc<Field>>>(
@@ -27,7 +31,9 @@ impl ListArray {
         let field: Arc<Field> = field.into();
         match &field.as_ref().dtype {
             DataType::List(child_dtype) => {
-                if let Some(validity) = validity.as_ref() && validity.len() != offsets.len_proxy() {
+                if let Some(validity) = validity.as_ref()
+                    && validity.len() != offsets.len_proxy()
+                {
                     panic!("ListArray::new validity length does not match computed length from offsets")
                 }
                 if child_dtype.as_ref() != flat_child.data_type() {
@@ -76,7 +82,7 @@ impl ListArray {
             ));
         }
 
-        let first_array = arrays.get(0).unwrap();
+        let first_array = arrays.first().unwrap();
         let mut growable = <Self as GrowableArray>::make_growable(
             first_array.field.name.as_str(),
             &first_array.field.dtype,
@@ -162,12 +168,14 @@ impl ListArray {
     }
 
     pub fn with_validity(&self, validity: Option<arrow2::bitmap::Bitmap>) -> DaftResult<Self> {
-        if let Some(v) = &validity && v.len() != self.len() {
+        if let Some(v) = &validity
+            && v.len() != self.len()
+        {
             return Err(DaftError::ValueError(format!(
                 "validity mask length does not match ListArray length, {} vs {}",
                 v.len(),
                 self.len()
-            )))
+            )));
         }
 
         Ok(Self::new(
@@ -178,3 +186,44 @@ impl ListArray {
         ))
     }
 }
+
+impl<'a> IntoIterator for &'a ListArray {
+    type Item = Option<Series>;
+
+    type IntoIter = ListArrayIter<'a>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        ListArrayIter {
+            array: self,
+            idx: 0,
+        }
+    }
+}
+
+pub struct ListArrayIter<'a> {
+    array: &'a ListArray,
+    idx: usize,
+}
+
+impl Iterator for ListArrayIter<'_> {
+    type Item = Option<Series>;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.idx < self.array.len() {
+            if let Some(validity) = self.array.validity()
+                && !validity.get_bit(self.idx)
+            {
+                self.idx += 1;
+                Some(None)
+            } else {
+                let start = *self.array.offsets().get(self.idx).unwrap() as usize;
+                let end = *self.array.offsets().get(self.idx + 1).unwrap() as usize;
+
+                self.idx += 1;
+                Some(Some(self.array.flat_child.slice(start, end).unwrap()))
+            }
+        } else {
+            None
+        }
+    }
+}
