@@ -19,7 +19,7 @@ where
         let arrow_array = Box::new(arrow2::array::PrimitiveArray::<i64>::from_vec(data));
         let data_array = Int64Array::from((name.as_ref(), arrow_array));
         let casted_array = data_array.cast(&T::get_dtype())?;
-        let downcasted = casted_array.downcast::<T>()?;
+        let downcasted = casted_array.downcast::<DataArray<T>>()?;
         Ok(downcasted.clone())
     }
 }
