@@ -1,25 +1,26 @@
-use std::{iter::repeat, sync::Arc};
+use std::{iter::repeat, ops::Div, sync::Arc};
 
 use super::as_arrow::AsArrow;
 use crate::{
     array::{
         growable::make_growable,
-        ops::image::ImageArraySidecarData,
-        ops::{from_arrow::FromArrow, full::FullNull},
+        ops::{from_arrow::FromArrow, full::FullNull, image::ImageArraySidecarData},
         DataArray, FixedSizeListArray, ListArray, StructArray,
     },
     datatypes::{
         logical::{
             DateArray, Decimal128Array, DurationArray, EmbeddingArray, FixedShapeImageArray,
-            FixedShapeTensorArray, ImageArray, LogicalArray, LogicalArrayImpl, TensorArray,
-            TimestampArray,
+            FixedShapeTensorArray, ImageArray, LogicalArray, LogicalArrayImpl, MapArray,
+            TensorArray, TimeArray, TimestampArray,
         },
-        DaftArrowBackedType, DaftLogicalType, DataType, Field, ImageMode, Int64Array, TimeUnit,
-        UInt64Array, Utf8Array,
+        DaftArrayType, DaftArrowBackedType, DaftLogicalType, DataType, Field, ImageMode,
+        Int32Array, Int64Array, TimeUnit, UInt64Array, Utf8Array,
     },
     series::{IntoSeries, Series},
+    utils::display_table::display_time64,
     with_match_daft_logical_primitive_types,
 };
+
 use common_error::{DaftError, DaftResult};
 
 use arrow2::{
@@ -39,7 +40,6 @@ use {
     crate::datatypes::PythonArray,
     crate::ffi,
     crate::with_match_numeric_daft_types,
-    log,
     ndarray::IntoDimension,
     num_traits::{NumCast, ToPrimitive},
     numpy::{PyArray3, PyReadonlyArrayDyn},
@@ -72,7 +72,7 @@ where
         use DataType::*;
         let source_arrow_array = match source_dtype {
             // Wrapped primitives
-            Decimal128(..) | Date | Timestamp(..) | Duration(..) => {
+            Decimal128(..) | Date | Timestamp(..) | Duration(..) | Time(..) => {
                 with_match_daft_logical_primitive_types!(source_dtype, |$T| {
                     use arrow2::array::Array;
                     to_cast
@@ -115,7 +115,7 @@ where
             let target_physical_type = dtype.to_physical().to_arrow()?;
             match dtype {
                 // Primitive wrapper types: change the arrow2 array's type field to primitive
-                Decimal128(..) | Date | Timestamp(..) | Duration(..) => {
+                Decimal128(..) | Date | Timestamp(..) | Duration(..) | Time(..) => {
                     with_match_daft_logical_primitive_types!(dtype, |$P| {
                         use arrow2::array::Array;
                         result_arrow_array
@@ -127,10 +127,10 @@ where
                             .to_boxed()
                     })
                 }
-                _ => arrow2::compute::cast::cast(
+                _ => cast(
                     result_arrow_array.as_ref(),
                     &target_physical_type,
-                    arrow2::compute::cast::CastOptions {
+                    CastOptions {
                         wrapped: true,
                         partial: false,
                     },
@@ -302,60 +302,36 @@ impl DateArray {
     }
 }
 
-pub(super) fn decimal128_to_str(val: i128, _precision: u8, scale: i8) -> String {
-    if scale < 0 {
-        unimplemented!();
-    } else {
-        let modulus = i128::pow(10, scale as u32);
-        let integral = val / modulus;
-        if scale == 0 {
-            format!("{}", integral)
-        } else {
-            let sign = if val < 0 { "-" } else { "" };
-            let integral = integral.abs();
-            let decimals = (val % modulus).abs();
-            let scale = scale as usize;
-            format!("{}{}.{:0scale$}", sign, integral, decimals)
-        }
-    }
-}
-
+/// Formats a naive timestamp to a string in the format "%Y-%m-%d %H:%M:%S%.f".
+/// Example: 2021-01-01 00:00:00
+/// See https://docs.rs/chrono/latest/chrono/format/strftime/index.html for format string options.
 pub(crate) fn timestamp_to_str_naive(val: i64, unit: &TimeUnit) -> String {
-    let chrono_ts =
-        { arrow2::temporal_conversions::timestamp_to_naive_datetime(val, unit.to_arrow()) };
-    let format_str = match unit {
-        TimeUnit::Seconds => "%Y-%m-%dT%H:%M:%S",
-        TimeUnit::Milliseconds => "%Y-%m-%dT%H:%M:%S%.3f",
-        TimeUnit::Microseconds => "%Y-%m-%dT%H:%M:%S%.6f",
-        TimeUnit::Nanoseconds => "%Y-%m-%dT%H:%M:%S%.9f",
-    };
+    let chrono_ts = arrow2::temporal_conversions::timestamp_to_naive_datetime(val, unit.to_arrow());
+    let format_str = "%Y-%m-%d %H:%M:%S%.f";
     chrono_ts.format(format_str).to_string()
 }
 
+/// Formats a timestamp with an offset to a string in the format "%Y-%m-%d %H:%M:%S%.f %:z".
+/// Example: 2021-01-01 00:00:00 -07:00
+/// See https://docs.rs/chrono/latest/chrono/format/strftime/index.html for format string options.
 pub(crate) fn timestamp_to_str_offset(
     val: i64,
     unit: &TimeUnit,
     offset: &chrono::FixedOffset,
 ) -> String {
-    let seconds_format = match unit {
-        TimeUnit::Seconds => chrono::SecondsFormat::Secs,
-        TimeUnit::Milliseconds => chrono::SecondsFormat::Millis,
-        TimeUnit::Microseconds => chrono::SecondsFormat::Micros,
-        TimeUnit::Nanoseconds => chrono::SecondsFormat::Nanos,
-    };
-    arrow2::temporal_conversions::timestamp_to_datetime(val, unit.to_arrow(), offset)
-        .to_rfc3339_opts(seconds_format, false)
+    let chrono_ts =
+        arrow2::temporal_conversions::timestamp_to_datetime(val, unit.to_arrow(), offset);
+    let format_str = "%Y-%m-%d %H:%M:%S%.f %:z";
+    chrono_ts.format(format_str).to_string()
 }
 
+/// Formats a timestamp with a timezone to a string in the format "%Y-%m-%d %H:%M:%S%.f %Z".
+/// Example: 2021-01-01 00:00:00 PST
+/// See https://docs.rs/chrono/latest/chrono/format/strftime/index.html for format string options.
 pub(crate) fn timestamp_to_str_tz(val: i64, unit: &TimeUnit, tz: &chrono_tz::Tz) -> String {
-    let seconds_format = match unit {
-        TimeUnit::Seconds => chrono::SecondsFormat::Secs,
-        TimeUnit::Milliseconds => chrono::SecondsFormat::Millis,
-        TimeUnit::Microseconds => chrono::SecondsFormat::Micros,
-        TimeUnit::Nanoseconds => chrono::SecondsFormat::Nanos,
-    };
-    arrow2::temporal_conversions::timestamp_to_datetime(val, unit.to_arrow(), tz)
-        .to_rfc3339_opts(seconds_format, false)
+    let chrono_ts = arrow2::temporal_conversions::timestamp_to_datetime(val, unit.to_arrow(), tz);
+    let format_str = "%Y-%m-%d %H:%M:%S%.f %Z";
+    chrono_ts.format(format_str).to_string()
 }
 
 impl TimestampArray {
@@ -363,6 +339,7 @@ impl TimestampArray {
         match dtype {
             DataType::Timestamp(..) => arrow_logical_cast(self, dtype),
             DataType::Date => Ok(self.date()?.into_series()),
+            DataType::Time(tu) => Ok(self.time(tu)?.into_series()),
             DataType::Utf8 => {
                 let DataType::Timestamp(unit, timezone) = self.data_type() else {
                     panic!("Wrong dtype for TimestampArray: {}", self.data_type())
@@ -407,6 +384,35 @@ impl TimestampArray {
     }
 }
 
+impl TimeArray {
+    pub fn cast(&self, dtype: &DataType) -> DaftResult<Series> {
+        match dtype {
+            DataType::Time(..) => arrow_logical_cast(self, dtype),
+            DataType::Utf8 => {
+                let time_array = self.as_arrow();
+                let time_str: arrow2::array::Utf8Array<i64> = time_array
+                    .iter()
+                    .map(|val| {
+                        val.map(|val| {
+                            let DataType::Time(unit) = &self.field.dtype else {
+                                panic!("Wrong dtype for TimeArray: {}", self.field.dtype)
+                            };
+                            display_time64(*val, unit)
+                        })
+                    })
+                    .collect();
+                Ok(Utf8Array::from((self.name(), Box::new(time_str))).into_series())
+            }
+            DataType::Int64 => Ok(self.physical.clone().into_series()),
+            DataType::Float32 => self.cast(&DataType::Int64)?.cast(&DataType::Float32),
+            DataType::Float64 => self.cast(&DataType::Int64)?.cast(&DataType::Float64),
+            #[cfg(feature = "python")]
+            DataType::Python => cast_logical_to_python_array(self, dtype),
+            _ => arrow_cast(&self.physical, dtype),
+        }
+    }
+}
+
 impl DurationArray {
     pub fn cast(&self, dtype: &DataType) -> DaftResult<Series> {
         match dtype {
@@ -418,6 +424,39 @@ impl DurationArray {
             _ => arrow_cast(&self.physical, dtype),
         }
     }
+
+    pub fn cast_to_days(&self) -> DaftResult<Int32Array> {
+        let tu = match self.data_type() {
+            DataType::Duration(tu) => tu,
+            _ => panic!("Wrong dtype for DurationArray: {}", self.data_type()),
+        };
+        let days = match tu {
+            TimeUnit::Seconds => self
+                .physical
+                .div(&Int64Array::from(("SecondsInDay", vec![60 * 60 * 24])))?,
+            TimeUnit::Milliseconds => self.physical.div(&Int64Array::from((
+                "MillisecondsInDay",
+                vec![1_000 * 60 * 60 * 24],
+            )))?,
+            TimeUnit::Microseconds => self.physical.div(&Int64Array::from((
+                "MicrosecondsInDay",
+                vec![1_000_000 * 60 * 60 * 24],
+            )))?,
+            TimeUnit::Nanoseconds => self.physical.div(&Int64Array::from((
+                "NanosecondsInDay",
+                vec![1_000_000_000 * 60 * 60 * 24],
+            )))?,
+        };
+        let days_i32 = cast(
+            days.data(),
+            &arrow2::datatypes::DataType::Int32,
+            CastOptions {
+                wrapped: true,
+                partial: false,
+            },
+        )?;
+        Int32Array::from_arrow(Field::new(self.name(), DataType::Int32).into(), days_i32)
+    }
 }
 
 impl Decimal128Array {
@@ -425,6 +464,7 @@ impl Decimal128Array {
         match dtype {
             #[cfg(feature = "python")]
             DataType::Python => cast_logical_to_python_array(self, dtype),
+            DataType::Int128 => Ok(self.physical.clone().into_series()),
             _ => arrow_logical_cast(self, dtype),
         }
     }
@@ -1374,9 +1414,9 @@ impl TensorArray {
                             // If image is 2 dimensions, 8-bit grayscale is assumed.
                             return false;
                         }
-                        if let Some(mode) = mode && s.u64().unwrap().as_arrow()
-                            .get(s.len() - 1)
-                            .unwrap() != mode.num_channels() as u64
+                        if let Some(mode) = mode
+                            && s.u64().unwrap().as_arrow().get(s.len() - 1).unwrap()
+                                != mode.num_channels() as u64
                         {
                             // If type-level mode is defined, each image must have the implied number of channels.
                             return false;
@@ -1714,11 +1754,22 @@ impl ListArray {
                     }
                 }
             }
+            DataType::Map(..) => Ok(MapArray::new(
+                Field::new(self.name(), dtype.clone()),
+                self.clone(),
+            )
+            .into_series()),
             _ => unimplemented!("List casting not implemented for dtype: {}", dtype),
         }
     }
 }
 
+impl MapArray {
+    pub fn cast(&self, dtype: &DataType) -> DaftResult<Series> {
+        self.physical.cast(dtype)
+    }
+}
+
 impl StructArray {
     pub fn cast(&self, dtype: &DataType) -> DaftResult<Series> {
         match (self.data_type(), dtype) {
