@@ -1,7 +1,10 @@
 use crate::{
     array::{DataArray, FixedSizeListArray, ListArray},
     datatypes::{
-        logical::{DateArray, Decimal128Array, DurationArray, LogicalArrayImpl, TimestampArray},
+        logical::{
+            DateArray, Decimal128Array, DurationArray, LogicalArrayImpl, MapArray, TimeArray,
+            TimestampArray,
+        },
         BinaryArray, BooleanArray, DaftLogicalType, DaftNumericType, ExtensionArray, NullArray,
         Utf8Array,
     },
@@ -66,6 +69,7 @@ impl_array_arrow_get!(BooleanArray, bool);
 impl_array_arrow_get!(BinaryArray, &[u8]);
 impl_array_arrow_get!(Decimal128Array, i128);
 impl_array_arrow_get!(DateArray, i32);
+impl_array_arrow_get!(TimeArray, i64);
 impl_array_arrow_get!(DurationArray, i64);
 impl_array_arrow_get!(TimestampArray, i64);
 
@@ -156,6 +160,13 @@ impl ListArray {
     }
 }
 
+impl MapArray {
+    #[inline]
+    pub fn get(&self, idx: usize) -> Option<Series> {
+        self.physical.get(idx)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use common_error::DaftResult;
