@@ -1,11 +1,8 @@
 #[cfg(feature = "python")]
-use pyo3::{
-    exceptions::PyValueError, prelude::*, types::PyBytes, PyObject, PyTypeInfo, ToPyObject,
-};
+use pyo3::{exceptions::PyValueError, prelude::*, types::PyBytes, PyTypeInfo};
 use serde::{Deserialize, Serialize};
 use std::fmt::{Display, Formatter, Result};
 use std::str::FromStr;
-use std::string::ToString;
 
 use crate::impl_bincode_py_state_serialization;
 
