@@ -76,6 +76,8 @@ pub enum DataType {
     List(Box<DataType>),
     /// A nested [`DataType`] with a given number of [`Field`]s.
     Struct(Vec<Field>),
+    /// A nested [`DataType`] that is represented as List<entries: Struct<key: K, value: V>>.
+    Map(Box<DataType>),
     /// Extension type.
     Extension(String, Box<DataType>, Option<String>),
     // Stop ArrowTypes
@@ -156,6 +158,14 @@ impl DataType {
             DataType::List(field) => Ok(ArrowType::LargeList(Box::new(
                 arrow2::datatypes::Field::new("item", field.to_arrow()?, true),
             ))),
+            DataType::Map(field) => Ok(ArrowType::Map(
+                Box::new(arrow2::datatypes::Field::new(
+                    "item",
+                    field.to_arrow()?,
+                    true,
+                )),
+                false,
+            )),
             DataType::Struct(fields) => Ok({
                 let fields = fields
                     .iter()
@@ -201,6 +211,7 @@ impl DataType {
             FixedSizeList(child_dtype, size) => {
                 FixedSizeList(Box::new(child_dtype.to_physical()), *size)
             }
+            Map(child_dtype) => List(Box::new(child_dtype.to_physical())),
             Embedding(dtype, size) => FixedSizeList(Box::new(dtype.to_physical()), *size),
             Image(mode) => Struct(vec![
                 Field::new(
@@ -310,6 +321,11 @@ impl DataType {
         matches!(self, DataType::FixedShapeImage(..))
     }
 
+    #[inline]
+    pub fn is_map(&self) -> bool {
+        matches!(self, DataType::Map(..))
+    }
+
     #[inline]
     pub fn is_null(&self) -> bool {
         match self {
@@ -334,12 +350,50 @@ impl DataType {
         }
     }
 
+    pub fn estimate_size_bytes(&self) -> Option<f64> {
+        const VARIABLE_TYPE_SIZE: f64 = 20.;
+        const DEFAULT_LIST_LEN: f64 = 4.;
+
+        let elem_size = match self.to_physical() {
+            DataType::Null => Some(0.),
+            DataType::Boolean => Some(0.125),
+            DataType::Int8 => Some(1.),
+            DataType::Int16 => Some(2.),
+            DataType::Int32 => Some(4.),
+            DataType::Int64 => Some(8.),
+            DataType::Int128 => Some(16.),
+            DataType::UInt8 => Some(1.),
+            DataType::UInt16 => Some(2.),
+            DataType::UInt32 => Some(4.),
+            DataType::UInt64 => Some(8.),
+            DataType::Float32 => Some(4.),
+            DataType::Float64 => Some(8.),
+            DataType::Utf8 => Some(VARIABLE_TYPE_SIZE),
+            DataType::Binary => Some(VARIABLE_TYPE_SIZE),
+            DataType::FixedSizeList(dtype, len) => {
+                dtype.estimate_size_bytes().map(|b| b * (len as f64))
+            }
+            DataType::List(dtype) => dtype.estimate_size_bytes().map(|b| b * DEFAULT_LIST_LEN),
+            DataType::Struct(fields) => Some(
+                fields
+                    .iter()
+                    .map(|f| f.dtype.estimate_size_bytes().unwrap_or(0f64))
+                    .sum(),
+            ),
+            DataType::Extension(_, dtype, _) => dtype.estimate_size_bytes(),
+            _ => None,
+        };
+        // add bitmap
+        elem_size.map(|e| e + 0.125)
+    }
+
     #[inline]
     pub fn is_logical(&self) -> bool {
         matches!(
             self,
             DataType::Decimal128(..)
                 | DataType::Date
+                | DataType::Time(..)
                 | DataType::Timestamp(..)
                 | DataType::Duration(..)
                 | DataType::Embedding(..)
@@ -347,6 +401,7 @@ impl DataType {
                 | DataType::FixedShapeImage(..)
                 | DataType::Tensor(..)
                 | DataType::FixedShapeTensor(..)
+                | DataType::Map(..)
         )
     }
 
@@ -360,7 +415,10 @@ impl DataType {
         let p: DataType = self.to_physical();
         matches!(
             p,
-            DataType::List(..) | DataType::FixedSizeList(..) | DataType::Struct(..)
+            DataType::List(..)
+                | DataType::FixedSizeList(..)
+                | DataType::Struct(..)
+                | DataType::Map(..)
         )
     }
 
@@ -411,6 +469,7 @@ impl From<&ArrowType> for DataType {
             ArrowType::FixedSizeList(field, size) => {
                 DataType::FixedSizeList(Box::new(field.as_ref().data_type().into()), *size)
             }
+            ArrowType::Map(field, ..) => DataType::Map(Box::new(field.as_ref().data_type().into())),
             ArrowType::Struct(fields) => {
                 let fields: Vec<Field> = fields.iter().map(|fld| fld.into()).collect();
                 DataType::Struct(fields)
@@ -455,10 +514,19 @@ impl Display for DataType {
             DataType::FixedSizeList(inner, size) => {
                 write!(f, "FixedSizeList[{}; {}]", inner, size)
             }
+            DataType::Map(inner, ..) => {
+                write!(f, "Map[{}]", inner)
+            }
             DataType::Struct(fields) => {
                 let fields: String = fields
                     .iter()
-                    .map(|f| format!("{}: {}", f.name, f.dtype))
+                    .filter_map(|f| {
+                        if f.name.is_empty() && f.dtype == DataType::Null {
+                            None
+                        } else {
+                            Some(format!("{}: {}", f.name, f.dtype))
+                        }
+                    })
                     .collect::<Vec<String>>()
                     .join(", ");
                 write!(f, "Struct[{fields}]")
