@@ -1,3 +1,5 @@
+use std::fmt::{Display, Formatter};
+
 use arrow2::datatypes::TimeUnit as ArrowTimeUnit;
 
 use serde::{Deserialize, Serialize};
@@ -21,6 +23,12 @@ impl TimeUnit {
         }
     }
 }
+impl Display for TimeUnit {
+    fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
+        // Leverage Debug trait implementation, which will already return the enum variant as a string.
+        write!(f, "{:?}", self)
+    }
+}
 
 impl From<&ArrowTimeUnit> for TimeUnit {
     fn from(tu: &ArrowTimeUnit) -> Self {
