@@ -33,15 +33,14 @@ impl PySchema {
         Ok(new_schema.into())
     }
 
-    pub fn apply_hints(&self, hints: &PySchema) -> PyResult<PySchema> {
-        let new_schema = Arc::new(self.schema.apply_hints(&hints.schema)?);
-        Ok(new_schema.into())
-    }
-
     pub fn eq(&self, other: &PySchema) -> PyResult<bool> {
         Ok(self.schema.fields.eq(&other.schema.fields))
     }
 
+    pub fn estimate_row_size_bytes(&self) -> PyResult<f64> {
+        Ok(self.schema.estimate_row_size_bytes())
+    }
+
     #[staticmethod]
     pub fn from_field_name_and_types(
         names_and_types: Vec<(String, PyDataType)>,
@@ -70,6 +69,10 @@ impl PySchema {
     pub fn _repr_html_(&self) -> PyResult<String> {
         Ok(self.schema.repr_html())
     }
+
+    pub fn _truncated_table_string(&self) -> PyResult<String> {
+        Ok(self.schema.truncated_table_string())
+    }
 }
 
 impl_bincode_py_state_serialization!(PySchema);
