@@ -18,6 +18,7 @@ use crate::{
 
 use super::datatype::PyDataType;
 use crate::array::ops::as_arrow::AsArrow;
+use crate::array::ops::trigonometry::TrigonometricFunction;
 
 #[pyclass]
 #[derive(Clone)]
@@ -108,6 +109,110 @@ impl PySeries {
         Ok(self.series.xor(&other.series)?.into_series().into())
     }
 
+    pub fn ceil(&self) -> PyResult<Self> {
+        Ok(self.series.ceil()?.into())
+    }
+
+    pub fn floor(&self) -> PyResult<Self> {
+        Ok(self.series.floor()?.into())
+    }
+
+    pub fn sign(&self) -> PyResult<Self> {
+        Ok(self.series.sign()?.into())
+    }
+
+    pub fn round(&self, decimal: i32) -> PyResult<Self> {
+        if decimal < 0 {
+            return Err(PyValueError::new_err(format!(
+                "decimal value can not be negative: {decimal}"
+            )));
+        }
+        Ok(self.series.round(decimal)?.into())
+    }
+
+    pub fn sqrt(&self) -> PyResult<Self> {
+        Ok(self.series.sqrt()?.into())
+    }
+
+    pub fn sin(&self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .trigonometry(&TrigonometricFunction::Sin)?
+            .into())
+    }
+
+    pub fn cos(&self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .trigonometry(&TrigonometricFunction::Cos)?
+            .into())
+    }
+
+    pub fn tan(&self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .trigonometry(&TrigonometricFunction::Tan)?
+            .into())
+    }
+
+    pub fn cot(&self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .trigonometry(&TrigonometricFunction::Cot)?
+            .into())
+    }
+
+    pub fn arcsin(&self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .trigonometry(&TrigonometricFunction::ArcSin)?
+            .into())
+    }
+
+    pub fn arccos(&self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .trigonometry(&TrigonometricFunction::ArcCos)?
+            .into())
+    }
+
+    pub fn arctan(&self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .trigonometry(&TrigonometricFunction::ArcTan)?
+            .into())
+    }
+
+    pub fn degrees(&self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .trigonometry(&TrigonometricFunction::Degrees)?
+            .into())
+    }
+
+    pub fn radians(&self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .trigonometry(&TrigonometricFunction::Radians)?
+            .into())
+    }
+
+    pub fn log2(&self) -> PyResult<Self> {
+        Ok(self.series.log2()?.into())
+    }
+
+    pub fn log10(&self) -> PyResult<Self> {
+        Ok(self.series.log10()?.into())
+    }
+
+    pub fn ln(&self) -> PyResult<Self> {
+        Ok(self.series.ln()?.into())
+    }
+
+    pub fn exp(&self) -> PyResult<Self> {
+        Ok(self.series.exp()?.into())
+    }
+
     pub fn take(&self, idx: &Self) -> PyResult<Self> {
         Ok(self.series.take(&idx.series)?.into())
     }
@@ -182,27 +287,27 @@ impl PySeries {
         Ok((&self.series).not()?.into())
     }
 
-    pub fn _count(&self, mode: CountMode) -> PyResult<Self> {
+    pub fn count(&self, mode: CountMode) -> PyResult<Self> {
         Ok((self.series).count(None, mode)?.into())
     }
 
-    pub fn _sum(&self) -> PyResult<Self> {
+    pub fn sum(&self) -> PyResult<Self> {
         Ok((self.series).sum(None)?.into())
     }
 
-    pub fn _mean(&self) -> PyResult<Self> {
+    pub fn mean(&self) -> PyResult<Self> {
         Ok((self.series).mean(None)?.into())
     }
 
-    pub fn _min(&self) -> PyResult<Self> {
+    pub fn min(&self) -> PyResult<Self> {
         Ok((self.series).min(None)?.into())
     }
 
-    pub fn _max(&self) -> PyResult<Self> {
+    pub fn max(&self) -> PyResult<Self> {
         Ok((self.series).max(None)?.into())
     }
 
-    pub fn _agg_list(&self) -> PyResult<Self> {
+    pub fn agg_list(&self) -> PyResult<Self> {
         Ok((self.series).agg_list(None)?.into())
     }
 
@@ -252,14 +357,95 @@ impl PySeries {
         Ok(self.series.utf8_contains(&pattern.series)?.into())
     }
 
-    pub fn utf8_split(&self, pattern: &Self) -> PyResult<Self> {
-        Ok(self.series.utf8_split(&pattern.series)?.into())
+    pub fn utf8_match(&self, pattern: &Self) -> PyResult<Self> {
+        Ok(self.series.utf8_match(&pattern.series)?.into())
+    }
+
+    pub fn utf8_split(&self, pattern: &Self, regex: bool) -> PyResult<Self> {
+        Ok(self.series.utf8_split(&pattern.series, regex)?.into())
+    }
+
+    pub fn utf8_extract(&self, pattern: &Self, index: usize) -> PyResult<Self> {
+        Ok(self.series.utf8_extract(&pattern.series, index)?.into())
+    }
+
+    pub fn utf8_extract_all(&self, pattern: &Self, index: usize) -> PyResult<Self> {
+        Ok(self.series.utf8_extract_all(&pattern.series, index)?.into())
+    }
+
+    pub fn utf8_replace(&self, pattern: &Self, replacement: &Self, regex: bool) -> PyResult<Self> {
+        Ok(self
+            .series
+            .utf8_replace(&pattern.series, &replacement.series, regex)?
+            .into())
     }
 
     pub fn utf8_length(&self) -> PyResult<Self> {
         Ok(self.series.utf8_length()?.into())
     }
 
+    pub fn utf8_lower(&self) -> PyResult<Self> {
+        Ok(self.series.utf8_lower()?.into())
+    }
+
+    pub fn utf8_upper(&self) -> PyResult<Self> {
+        Ok(self.series.utf8_upper()?.into())
+    }
+
+    pub fn utf8_lstrip(&self) -> PyResult<Self> {
+        Ok(self.series.utf8_lstrip()?.into())
+    }
+
+    pub fn utf8_rstrip(&self) -> PyResult<Self> {
+        Ok(self.series.utf8_rstrip()?.into())
+    }
+
+    pub fn utf8_reverse(&self) -> PyResult<Self> {
+        Ok(self.series.utf8_reverse()?.into())
+    }
+
+    pub fn utf8_capitalize(&self) -> PyResult<Self> {
+        Ok(self.series.utf8_capitalize()?.into())
+    }
+
+    pub fn utf8_left(&self, nchars: &Self) -> PyResult<Self> {
+        Ok(self.series.utf8_left(&nchars.series)?.into())
+    }
+
+    pub fn utf8_right(&self, nchars: &Self) -> PyResult<Self> {
+        Ok(self.series.utf8_right(&nchars.series)?.into())
+    }
+
+    pub fn utf8_find(&self, substr: &Self) -> PyResult<Self> {
+        Ok(self.series.utf8_find(&substr.series)?.into())
+    }
+
+    pub fn utf8_rpad(&self, length: &Self, character: &Self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .utf8_rpad(&length.series, &character.series)?
+            .into())
+    }
+
+    pub fn utf8_lpad(&self, length: &Self, character: &Self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .utf8_lpad(&length.series, &character.series)?
+            .into())
+    }
+
+    pub fn utf8_repeat(&self, n: &Self) -> PyResult<Self> {
+        Ok(self.series.utf8_repeat(&n.series)?.into())
+    }
+
+    pub fn utf8_like(&self, pattern: &Self) -> PyResult<Self> {
+        Ok(self.series.utf8_like(&pattern.series)?.into())
+    }
+
+    pub fn utf8_ilike(&self, pattern: &Self) -> PyResult<Self> {
+        Ok(self.series.utf8_ilike(&pattern.series)?.into())
+    }
+
     pub fn is_nan(&self) -> PyResult<Self> {
         Ok(self.series.is_nan()?.into())
     }
@@ -276,6 +462,18 @@ impl PySeries {
         Ok(self.series.dt_hour()?.into())
     }
 
+    pub fn dt_minute(&self) -> PyResult<Self> {
+        Ok(self.series.dt_minute()?.into())
+    }
+
+    pub fn dt_second(&self) -> PyResult<Self> {
+        Ok(self.series.dt_second()?.into())
+    }
+
+    pub fn dt_time(&self) -> PyResult<Self> {
+        Ok(self.series.dt_time()?.into())
+    }
+
     pub fn dt_month(&self) -> PyResult<Self> {
         Ok(self.series.dt_month()?.into())
     }
@@ -288,6 +486,13 @@ impl PySeries {
         Ok(self.series.dt_day_of_week()?.into())
     }
 
+    pub fn dt_truncate(&self, interval: &str, relative_to: &Self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .dt_truncate(interval, &relative_to.series)?
+            .into())
+    }
+
     pub fn partitioning_days(&self) -> PyResult<Self> {
         Ok(self.series.partitioning_days()?.into())
     }
@@ -316,16 +521,20 @@ impl PySeries {
         Ok(self.series.murmur3_32()?.into_series().into())
     }
 
-    pub fn list_lengths(&self) -> PyResult<Self> {
-        Ok(self.series.list_lengths()?.into_series().into())
+    pub fn list_count(&self, mode: CountMode) -> PyResult<Self> {
+        Ok(self.series.list_count(mode)?.into_series().into())
     }
 
     pub fn list_get(&self, idx: &Self, default: &Self) -> PyResult<Self> {
         Ok(self.series.list_get(&idx.series, &default.series)?.into())
     }
 
-    pub fn image_decode(&self) -> PyResult<Self> {
-        Ok(self.series.image_decode()?.into())
+    pub fn map_get(&self, key: &Self) -> PyResult<Self> {
+        Ok(self.series.map_get(&key.series)?.into())
+    }
+
+    pub fn image_decode(&self, raise_error_on_failure: bool) -> PyResult<Self> {
+        Ok(self.series.image_decode(raise_error_on_failure)?.into())
     }
 
     pub fn image_encode(&self, image_format: ImageFormat) -> PyResult<Self> {
@@ -362,6 +571,10 @@ impl PySeries {
         Ok(self.series.not_null()?.into())
     }
 
+    pub fn fill_null(&self, fill_value: &Self) -> PyResult<Self> {
+        Ok(self.series.fill_null(&fill_value.series)?.into())
+    }
+
     pub fn _debug_bincode_serialize(&self, py: Python) -> PyResult<PyObject> {
         let values = bincode::serialize(&self.series).unwrap();
         Ok(PyBytes::new(py, &values).to_object(py))
@@ -372,6 +585,10 @@ impl PySeries {
         let values = bincode::deserialize::<Series>(bytes.as_bytes()).unwrap();
         Ok(Self { series: values })
     }
+
+    pub fn to_str_values(&self) -> PyResult<Self> {
+        Ok(self.series.to_str_values()?.into())
+    }
 }
 
 impl From<series::Series> for PySeries {
@@ -408,7 +625,9 @@ fn infer_daft_dtype_for_sequence(
     let mut dtype: Option<DataType> = None;
     for obj in vec_pyobj.iter() {
         let obj = obj.as_ref(py);
-        if let Ok(pil_image_type) = py_pil_image_type && obj.is_instance(pil_image_type)? {
+        if let Ok(pil_image_type) = py_pil_image_type
+            && obj.is_instance(pil_image_type)?
+        {
             let mode_str = obj
                 .getattr(pyo3::intern!(py, "mode"))?
                 .extract::<String>()?;
@@ -432,18 +651,33 @@ fn infer_daft_dtype_for_sequence(
                     break;
                 }
             }
-        } else if let Ok(np_ndarray_type) = np_ndarray_type && let Ok(np_generic_type) = np_generic_type && (obj.is_instance(np_ndarray_type)? || obj.is_instance(np_generic_type)?) {
+        } else if let Ok(np_ndarray_type) = np_ndarray_type
+            && let Ok(np_generic_type) = np_generic_type
+            && (obj.is_instance(np_ndarray_type)? || obj.is_instance(np_generic_type)?)
+        {
             let np_dtype = obj.getattr(pyo3::intern!(py, "dtype"))?;
-            let inferred_inner_dtype = from_numpy_dtype.call1((np_dtype,)).map(|dt| dt.getattr(pyo3::intern!(py, "_dtype")).unwrap().extract::<PyDataType>().unwrap().dtype);
+            let inferred_inner_dtype = from_numpy_dtype.call1((np_dtype,)).map(|dt| {
+                dt.getattr(pyo3::intern!(py, "_dtype"))
+                    .unwrap()
+                    .extract::<PyDataType>()
+                    .unwrap()
+                    .dtype
+            });
             let shape: Vec<u64> = obj.getattr(pyo3::intern!(py, "shape"))?.extract()?;
             let inferred_dtype = match inferred_inner_dtype {
-                Ok(inferred_inner_dtype) if shape.len() == 1 => Some(DataType::List(Box::new(inferred_inner_dtype))),
-                Ok(inferred_inner_dtype) if shape.len() > 1 => Some(DataType::Tensor(Box::new(inferred_inner_dtype))),
+                Ok(inferred_inner_dtype) if shape.len() == 1 => {
+                    Some(DataType::List(Box::new(inferred_inner_dtype)))
+                }
+                Ok(inferred_inner_dtype) if shape.len() > 1 => {
+                    Some(DataType::Tensor(Box::new(inferred_inner_dtype)))
+                }
                 _ => None,
             };
             match (&dtype, &inferred_dtype) {
                 // Tensors with mixed inner dtypes is not supported, short-circuit.
-                (Some(existing_dtype), Some(inferred_dtype)) if existing_dtype != inferred_dtype => {
+                (Some(existing_dtype), Some(inferred_dtype))
+                    if existing_dtype != inferred_dtype =>
+                {
                     // TODO(Clark): Do some basic type promotion here, e.g. (u32, u64) --> u64.
                     dtype = None;
                     break;
