@@ -9,7 +9,10 @@ use std::{
 use indexmap::IndexMap;
 use serde::{Deserialize, Serialize};
 
-use crate::{datatypes::Field, utils::display_table::make_comfy_table};
+use crate::{
+    datatypes::Field,
+    utils::display_table::{make_comfy_table, make_schema_vertical_table},
+};
 
 use common_error::{DaftError, DaftResult};
 
@@ -121,7 +124,9 @@ impl Schema {
         res.push_str("<thead><tr>");
 
         for (name, field) in &self.fields {
-            res.push_str("<th style=\"text-wrap: nowrap; max-width:192px; overflow:auto\">");
+            res.push_str(
+                "<th style=\"text-wrap: nowrap; max-width:192px; overflow:auto; text-align:left\">",
+            );
             res.push_str(&html_escape::encode_text(name));
             res.push_str("<br />");
             res.push_str(&html_escape::encode_text(&format!("{}", field.dtype)));
@@ -139,10 +144,47 @@ impl Schema {
     pub fn short_string(&self) -> String {
         self.fields
             .iter()
-            .map(|(name, field)| format!("{} ({:?})", name, field.dtype))
+            .map(|(name, field)| format!("{}#{:?}", name, field.dtype))
             .collect::<Vec<String>>()
             .join(", ")
     }
+
+    pub fn truncated_table_string(&self) -> String {
+        let table = make_comfy_table(
+            self.fields
+                .values()
+                .map(Cow::Borrowed)
+                .collect::<Vec<_>>()
+                .as_slice(),
+            None,
+            None,
+        );
+        format!("{}\n", table)
+    }
+
+    pub fn estimate_row_size_bytes(&self) -> f64 {
+        self.fields
+            .values()
+            .map(|f| f.dtype.estimate_size_bytes().unwrap_or(0.))
+            .sum()
+    }
+
+    /// Returns a new schema with only the specified columns in the new schema
+    pub fn project<S: AsRef<str>>(self: Arc<Self>, columns: &[S]) -> DaftResult<Schema> {
+        let new_fields = columns
+            .iter()
+            .map(|i| {
+                let key = i.as_ref();
+                self.fields.get(key).cloned().ok_or_else(|| {
+                    DaftError::SchemaMismatch(format!(
+                        "Column {} not found in schema: {:?}",
+                        key, self.fields
+                    ))
+                })
+            })
+            .collect::<DaftResult<Vec<_>>>()?;
+        Self::new(new_fields)
+    }
 }
 
 impl Eq for Schema {}
@@ -183,14 +225,12 @@ impl Default for Schema {
 impl Display for Schema {
     // Produces an ASCII table.
     fn fmt(&self, f: &mut Formatter) -> Result {
-        let table = make_comfy_table(
+        let table = make_schema_vertical_table(
             self.fields
                 .values()
                 .map(Cow::Borrowed)
                 .collect::<Vec<_>>()
                 .as_slice(),
-            None,
-            None,
         );
         writeln!(f, "{table}")
     }
