@@ -32,20 +32,13 @@ use crate::{metadata::read_csv_schema_single, CsvConvertOptions, CsvParseOptions
 use daft_compression::CompressionCodec;
 use daft_decoding::deserialize::deserialize_column;
 
-trait ByteRecordChunkStream = Stream<Item = super::Result<Vec<ByteRecord>>>;
-trait ColumnArrayChunkStream = Stream<
-    Item = super::Result<
-        Context<
-            JoinHandle<DaftResult<Vec<Box<dyn arrow2::array::Array>>>>,
-            super::JoinSnafu,
-            super::Error,
-        >,
-    >,
->;
+trait ByteRecordChunkStream: Stream<Item = super::Result<Vec<ByteRecord>>> {}
+impl<S> ByteRecordChunkStream for S where S: Stream<Item = super::Result<Vec<ByteRecord>>> {}
 
-trait TableStream = Stream<
-    Item = super::Result<Context<JoinHandle<DaftResult<Table>>, super::JoinSnafu, super::Error>>,
->;
+type TableChunkResult =
+    super::Result<Context<JoinHandle<DaftResult<Table>>, super::JoinSnafu, super::Error>>;
+trait TableStream: Stream<Item = TableChunkResult> {}
+impl<S> TableStream for S where S: Stream<Item = TableChunkResult> {}
 
 #[allow(clippy::too_many_arguments)]
 pub fn read_csv(
@@ -142,13 +135,6 @@ pub fn read_csv_bulk(
     tables.into_iter().collect::<DaftResult<Vec<_>>>()
 }
 
-#[inline]
-fn assert_stream_send<'u, R>(
-    s: impl 'u + Send + Stream<Item = R>,
-) -> impl 'u + Send + Stream<Item = R> {
-    s
-}
-
 // Parallel version of table concat
 // get rid of this once Table APIs are parallel
 fn tables_concat(mut tables: Vec<Table>) -> DaftResult<Table> {
@@ -238,8 +224,7 @@ async fn read_csv_single_into_table(
             .unwrap_or(NonZeroUsize::new(2).unwrap())
             .checked_mul(2.try_into().unwrap())
             .unwrap()
-            .try_into()
-            .unwrap()
+            .into()
     });
     // Collect all chunks in chunk x column form.
     let tables = chunk_stream
@@ -263,9 +248,9 @@ async fn read_csv_single_into_table(
     let schema: arrow2::datatypes::Schema = schema_fields.into();
     let schema = Arc::new(Schema::try_from(&schema)?);
 
-    let filtered_tables = assert_stream_send(tables.map_ok(move |table| {
+    let filtered_tables = tables.map_ok(move |table| {
         if let Some(predicate) = &predicate {
-            let filtered = table?.filter(&[predicate.as_ref()])?;
+            let filtered = table?.filter(&[predicate.clone()])?;
             if let Some(include_columns) = &include_columns {
                 filtered.get_columns(include_columns.as_slice())
             } else {
@@ -274,7 +259,7 @@ async fn read_csv_single_into_table(
         } else {
             table
         }
-    }));
+    });
     let mut remaining_rows = limit.map(|limit| limit as i64);
     let collected_tables = filtered_tables
         .try_take_while(|result| {
@@ -302,8 +287,10 @@ async fn read_csv_single_into_table(
 
     // // TODO(Clark): Don't concatenate all chunks from a file into a single table, since MicroPartition is natively chunked.
     let concated_table = tables_concat(collected_tables)?;
-    if let Some(limit) = limit && concated_table.len() > limit {
-        // apply head incase that last chunk went over limit
+    if let Some(limit) = limit
+        && concated_table.len() > limit
+    {
+        // apply head in case that last chunk went over limit
         concated_table.head(limit)
     } else {
         Ok(concated_table)
@@ -422,7 +409,7 @@ fn read_into_byterecord_chunk_stream<R>(
     chunk_size: usize,
     estimated_mean_row_size: Option<f64>,
     estimated_std_row_size: Option<f64>,
-) -> impl ByteRecordChunkStream + Send
+) -> impl ByteRecordChunkStream
 where
     R: AsyncRead + Unpin + Send + 'static,
 {
@@ -531,7 +518,7 @@ fn parse_into_column_array_chunk_stream(
 }
 
 fn fields_to_projection_indices(
-    fields: &Vec<arrow2::datatypes::Field>,
+    fields: &[arrow2::datatypes::Field],
     include_columns: &Option<Vec<String>>,
 ) -> Arc<Vec<usize>> {
     let field_name_to_idx = fields
@@ -576,6 +563,7 @@ mod tests {
 
     use super::read_csv;
 
+    #[allow(clippy::too_many_arguments)]
     fn check_equal_local_arrow2(
         path: &str,
         out: &Table,
@@ -599,7 +587,14 @@ mod tests {
             .unwrap();
         let (mut fields, _) = infer_schema(&mut reader, None, has_header, &infer).unwrap();
         if !has_header && let Some(column_names) = column_names {
-            fields = fields.into_iter().zip(column_names).map(|(field, name)| arrow2::datatypes::Field::new(name, field.data_type, true).with_metadata(field.metadata)).collect::<Vec<_>>();
+            fields = fields
+                .into_iter()
+                .zip(column_names)
+                .map(|(field, name)| {
+                    arrow2::datatypes::Field::new(name, field.data_type, true)
+                        .with_metadata(field.metadata)
+                })
+                .collect::<Vec<_>>();
         }
         let mut rows = vec![ByteRecord::default(); limit.unwrap_or(100)];
         let rows_read = read_rows(&mut reader, 0, &mut rows).unwrap();
