@@ -1,13 +1,17 @@
 use daft_core::{
     count_mode::CountMode,
-    datatypes::DataType,
-    datatypes::{Field, FieldID},
+    datatypes::{try_mean_supertype, try_sum_supertype, DataType, Field, FieldID},
     schema::Schema,
     utils::supertype::try_get_supertype,
 };
 
 use crate::{
-    functions::FunctionEvaluator,
+    functions::{
+        function_display, function_semantic_id,
+        sketch::{HashableVecPercentiles, SketchExpr},
+        struct_::StructExpr,
+        FunctionEvaluator,
+    },
     lit,
     optimization::{get_required_columns, requires_computation},
 };
@@ -17,6 +21,7 @@ use common_error::{DaftError, DaftResult};
 use serde::{Deserialize, Serialize};
 use std::{
     fmt::{Debug, Display, Formatter, Result},
+    io::{self, Write},
     sync::Arc,
 };
 
@@ -37,11 +42,13 @@ pub enum Expr {
     Column(Arc<str>),
     Function {
         func: FunctionExpr,
-        inputs: Vec<Expr>,
+        inputs: Vec<ExprRef>,
     },
     Not(ExprRef),
     IsNull(ExprRef),
     NotNull(ExprRef),
+    FillNull(ExprRef, ExprRef),
+    IsIn(ExprRef, ExprRef),
     Literal(lit::LiteralValue),
     IfElse {
         if_true: ExprRef,
@@ -50,40 +57,56 @@ pub enum Expr {
     },
 }
 
+#[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Hash, Eq)]
+pub struct ApproxPercentileParams {
+    pub child: ExprRef,
+    pub percentiles: Vec<daft_core::utils::hashable_float_wrapper::FloatWrapper<f64>>,
+    pub force_list_output: bool,
+}
+
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
 pub enum AggExpr {
     Count(ExprRef, CountMode),
     Sum(ExprRef),
+    ApproxSketch(ExprRef),
+    ApproxPercentile(ApproxPercentileParams),
+    MergeSketch(ExprRef),
     Mean(ExprRef),
     Min(ExprRef),
     Max(ExprRef),
+    AnyValue(ExprRef, bool),
     List(ExprRef),
     Concat(ExprRef),
+    MapGroups {
+        func: FunctionExpr,
+        inputs: Vec<ExprRef>,
+    },
 }
 
-pub fn col<S: Into<Arc<str>>>(name: S) -> Expr {
-    Expr::Column(name.into())
+pub fn col<S: Into<Arc<str>>>(name: S) -> ExprRef {
+    Expr::Column(name.into()).into()
 }
 
-pub fn binary_op(op: Operator, left: &Expr, right: &Expr) -> Expr {
-    Expr::BinaryOp {
-        op,
-        left: left.clone().into(),
-        right: right.clone().into(),
-    }
+pub fn binary_op(op: Operator, left: ExprRef, right: ExprRef) -> ExprRef {
+    Expr::BinaryOp { op, left, right }.into()
 }
 
 impl AggExpr {
-    pub fn name(&self) -> DaftResult<&str> {
+    pub fn name(&self) -> &str {
         use AggExpr::*;
         match self {
             Count(expr, ..)
             | Sum(expr)
+            | ApproxSketch(expr)
+            | ApproxPercentile(ApproxPercentileParams { child: expr, .. })
+            | MergeSketch(expr)
             | Mean(expr)
             | Min(expr)
             | Max(expr)
+            | AnyValue(expr, _)
             | List(expr)
             | Concat(expr) => expr.name(),
+            MapGroups { func: _, inputs } => inputs.first().unwrap().name(),
         }
     }
 
@@ -98,6 +121,25 @@ impl AggExpr {
                 let child_id = expr.semantic_id(schema);
                 FieldID::new(format!("{child_id}.local_sum()"))
             }
+            ApproxSketch(expr) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.local_approx_sketch()"))
+            }
+            ApproxPercentile(ApproxPercentileParams {
+                child: expr,
+                percentiles,
+                force_list_output,
+            }) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!(
+                    "{child_id}.local_approx_percentiles(percentiles={:?},force_list_output={force_list_output})",
+                    percentiles,
+                ))
+            }
+            MergeSketch(expr) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.local_merge_sketch()"))
+            }
             Mean(expr) => {
                 let child_id = expr.semantic_id(schema);
                 FieldID::new(format!("{child_id}.local_mean()"))
@@ -110,6 +152,12 @@ impl AggExpr {
                 let child_id = expr.semantic_id(schema);
                 FieldID::new(format!("{child_id}.local_max()"))
             }
+            AnyValue(expr, ignore_nulls) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!(
+                    "{child_id}.local_any_value(ignore_nulls={ignore_nulls})"
+                ))
+            }
             List(expr) => {
                 let child_id = expr.semantic_id(schema);
                 FieldID::new(format!("{child_id}.local_list()"))
@@ -118,19 +166,60 @@ impl AggExpr {
                 let child_id = expr.semantic_id(schema);
                 FieldID::new(format!("{child_id}.local_concat()"))
             }
+            MapGroups { func, inputs } => function_semantic_id(func, inputs, schema),
         }
     }
 
-    pub fn child(&self) -> ExprRef {
+    pub fn children(&self) -> Vec<ExprRef> {
         use AggExpr::*;
         match self {
             Count(expr, ..)
             | Sum(expr)
+            | ApproxSketch(expr)
+            | ApproxPercentile(ApproxPercentileParams { child: expr, .. })
+            | MergeSketch(expr)
             | Mean(expr)
             | Min(expr)
             | Max(expr)
+            | AnyValue(expr, _)
             | List(expr)
-            | Concat(expr) => expr.clone(),
+            | Concat(expr) => vec![expr.clone()],
+            MapGroups { func: _, inputs } => inputs.clone(),
+        }
+    }
+
+    pub fn with_new_children(&self, children: Vec<ExprRef>) -> Self {
+        use AggExpr::*;
+
+        if let MapGroups { func: _, inputs } = &self {
+            assert_eq!(children.len(), inputs.len());
+        } else {
+            assert_eq!(children.len(), 1);
+        }
+        match self {
+            Count(_, count_mode) => Count(children[0].clone(), *count_mode),
+            Sum(_) => Sum(children[0].clone()),
+            Mean(_) => Mean(children[0].clone()),
+            Min(_) => Min(children[0].clone()),
+            Max(_) => Mean(children[0].clone()),
+            AnyValue(_, ignore_nulls) => AnyValue(children[0].clone(), *ignore_nulls),
+            List(_) => List(children[0].clone()),
+            Concat(_) => Concat(children[0].clone()),
+            MapGroups { func, inputs: _ } => MapGroups {
+                func: func.clone(),
+                inputs: children,
+            },
+            ApproxPercentile(ApproxPercentileParams {
+                percentiles,
+                force_list_output,
+                ..
+            }) => ApproxPercentile(ApproxPercentileParams {
+                child: children[0].clone(),
+                percentiles: percentiles.clone(),
+                force_list_output: *force_list_output,
+            }),
+            ApproxSketch(_) => ApproxSketch(children[0].clone()),
+            MergeSketch(_) => MergeSketch(children[0].clone()),
         }
     }
 
@@ -145,26 +234,10 @@ impl AggExpr {
                 let field = expr.to_field(schema)?;
                 Ok(Field::new(
                     field.name.as_str(),
-                    match &field.dtype {
-                        DataType::Int8 | DataType::Int16 | DataType::Int32 | DataType::Int64 => {
-                            DataType::Int64
-                        }
-                        DataType::UInt8
-                        | DataType::UInt16
-                        | DataType::UInt32
-                        | DataType::UInt64 => DataType::UInt64,
-                        DataType::Float32 => DataType::Float32,
-                        DataType::Float64 => DataType::Float64,
-                        other => {
-                            return Err(DaftError::TypeError(format!(
-                                "Expected input to sum() to be numeric but received dtype {} for column \"{}\"",
-                                other, field.name,
-                            )))
-                        }
-                    },
+                    try_sum_supertype(&field.dtype)?,
                 ))
             }
-            Mean(expr) => {
+            ApproxSketch(expr) => {
                 let field = expr.to_field(schema)?;
                 Ok(Field::new(
                     field.name.as_str(),
@@ -178,17 +251,62 @@ impl AggExpr {
                         | DataType::UInt32
                         | DataType::UInt64
                         | DataType::Float32
-                        | DataType::Float64 => DataType::Float64,
+                        | DataType::Float64 => DataType::from(&*daft_sketch::ARROW2_DDSKETCH_DTYPE),
                         other => {
                             return Err(DaftError::TypeError(format!(
-                                "Numeric mean is not implemented for column \"{}\" of type {}",
-                                field.name, other,
+                                "Expected input to approx_sketch() to be numeric but received dtype {} for column \"{}\"",
+                                other, field.name,
                             )))
                         }
                     },
                 ))
             }
-            Min(expr) | Max(expr) => {
+            ApproxPercentile(ApproxPercentileParams {
+                child: expr,
+                percentiles,
+                force_list_output,
+            }) => {
+                let field = expr.to_field(schema)?;
+                Ok(Field::new(
+                    field.name.as_str(),
+                    match &field.dtype {
+                        dt if dt.is_numeric() => if percentiles.len() > 1 || *force_list_output {
+                            DataType::FixedSizeList(Box::new(DataType::Float64), percentiles.len())
+                        } else {
+                            DataType::Float64
+                        },
+                        other => {
+                            return Err(DaftError::TypeError(format!(
+                                "Expected input to approx_percentiles() to be numeric but received dtype {} for column \"{}\"",
+                                other, field.name,
+                            )))
+                        }
+                    },
+                ))
+            }
+            MergeSketch(expr) => {
+                let field = expr.to_field(schema)?;
+                Ok(Field::new(
+                  field.name.as_str(),
+                  match &field.dtype {
+                      DataType::Struct(fields) => DataType::Struct(fields.clone()),
+                      other => {
+                          return Err(DaftError::TypeError(format!(
+                              "Expected input to merge_sketch() to be struct but received dtype {} for column \"{}\"",
+                              other, field.name,
+                          )))
+                        }
+                    },
+                ))
+            }
+            Mean(expr) => {
+                let field = expr.to_field(schema)?;
+                Ok(Field::new(
+                    field.name.as_str(),
+                    try_mean_supertype(&field.dtype)?,
+                ))
+            }
+            Min(expr) | Max(expr) | AnyValue(expr, _) => {
                 let field = expr.to_field(schema)?;
                 Ok(Field::new(field.name.as_str(), field.dtype))
             }
@@ -205,18 +323,19 @@ impl AggExpr {
                     ))),
                 }
             }
+            MapGroups { func, inputs } => func.to_field(inputs.as_slice(), schema, func),
         }
     }
 
-    pub fn from_name_and_child_expr(name: &str, child: &Expr) -> DaftResult<AggExpr> {
+    pub fn from_name_and_child_expr(name: &str, child: ExprRef) -> DaftResult<AggExpr> {
         use AggExpr::*;
         match name {
-            "count" => Ok(Count(child.clone().into(), CountMode::Valid)),
-            "sum" => Ok(Sum(child.clone().into())),
-            "mean" => Ok(Mean(child.clone().into())),
-            "min" => Ok(Min(child.clone().into())),
-            "max" => Ok(Max(child.clone().into())),
-            "list" => Ok(List(child.clone().into())),
+            "count" => Ok(Count(child, CountMode::Valid)),
+            "sum" => Ok(Sum(child)),
+            "mean" => Ok(Mean(child)),
+            "min" => Ok(Min(child)),
+            "max" => Ok(Max(child)),
+            "list" => Ok(List(child)),
             _ => Err(DaftError::ValueError(format!(
                 "{} not a valid aggregation name",
                 name
@@ -232,91 +351,148 @@ impl AsRef<Expr> for Expr {
 }
 
 impl Expr {
-    pub fn alias<S: Into<Arc<str>>>(&self, name: S) -> Self {
-        Expr::Alias(self.clone().into(), name.into())
+    pub fn arced(self) -> ExprRef {
+        Arc::new(self)
     }
 
-    pub fn if_else(&self, if_true: &Self, if_false: &Self) -> Self {
+    pub fn alias<S: Into<Arc<str>>>(self: &ExprRef, name: S) -> ExprRef {
+        Expr::Alias(self.clone(), name.into()).into()
+    }
+
+    pub fn if_else(self: ExprRef, if_true: ExprRef, if_false: ExprRef) -> ExprRef {
         Expr::IfElse {
-            if_true: if_true.clone().into(),
-            if_false: if_false.clone().into(),
-            predicate: self.clone().into(),
+            if_true,
+            if_false,
+            predicate: self,
         }
+        .into()
     }
 
-    pub fn cast(&self, dtype: &DataType) -> Self {
-        Expr::Cast(self.clone().into(), dtype.clone())
+    pub fn cast(self: ExprRef, dtype: &DataType) -> ExprRef {
+        Expr::Cast(self, dtype.clone()).into()
     }
 
-    pub fn count(&self, mode: CountMode) -> Self {
-        Expr::Agg(AggExpr::Count(self.clone().into(), mode))
+    pub fn count(self: ExprRef, mode: CountMode) -> ExprRef {
+        Expr::Agg(AggExpr::Count(self, mode)).into()
     }
 
-    pub fn sum(&self) -> Self {
-        Expr::Agg(AggExpr::Sum(self.clone().into()))
+    pub fn sum(self: ExprRef) -> ExprRef {
+        Expr::Agg(AggExpr::Sum(self)).into()
     }
 
-    pub fn mean(&self) -> Self {
-        Expr::Agg(AggExpr::Mean(self.clone().into()))
+    pub fn approx_sketch(self: ExprRef) -> ExprRef {
+        Expr::Agg(AggExpr::ApproxSketch(self)).into()
     }
 
-    pub fn min(&self) -> Self {
-        Expr::Agg(AggExpr::Min(self.clone().into()))
+    pub fn approx_percentiles(
+        self: ExprRef,
+        percentiles: &[f64],
+        force_list_output: bool,
+    ) -> ExprRef {
+        Expr::Agg(AggExpr::ApproxPercentile(ApproxPercentileParams {
+            child: self,
+            percentiles: percentiles
+                .iter()
+                .map(|f| daft_core::utils::hashable_float_wrapper::FloatWrapper(*f))
+                .collect(),
+            force_list_output,
+        }))
+        .into()
     }
 
-    pub fn max(&self) -> Self {
-        Expr::Agg(AggExpr::Max(self.clone().into()))
+    pub fn sketch_percentile(
+        self: ExprRef,
+        percentiles: &[f64],
+        force_list_output: bool,
+    ) -> ExprRef {
+        Expr::Function {
+            func: FunctionExpr::Sketch(SketchExpr::Percentile {
+                percentiles: HashableVecPercentiles(percentiles.to_vec()),
+                force_list_output,
+            }),
+            inputs: vec![self],
+        }
+        .into()
     }
 
-    pub fn agg_list(&self) -> Self {
-        Expr::Agg(AggExpr::List(self.clone().into()))
+    pub fn merge_sketch(self: ExprRef) -> ExprRef {
+        Expr::Agg(AggExpr::MergeSketch(self)).into()
     }
 
-    pub fn agg_concat(&self) -> Self {
-        Expr::Agg(AggExpr::Concat(self.clone().into()))
+    pub fn mean(self: ExprRef) -> ExprRef {
+        Expr::Agg(AggExpr::Mean(self)).into()
     }
 
-    pub fn not(&self) -> Self {
-        Expr::Not(self.clone().into())
+    pub fn min(self: ExprRef) -> ExprRef {
+        Expr::Agg(AggExpr::Min(self)).into()
     }
 
-    pub fn is_null(&self) -> Self {
-        Expr::IsNull(self.clone().into())
+    pub fn max(self: ExprRef) -> ExprRef {
+        Expr::Agg(AggExpr::Max(self)).into()
     }
 
-    pub fn not_null(&self) -> Self {
-        Expr::NotNull(self.clone().into())
+    pub fn any_value(self: ExprRef, ignore_nulls: bool) -> ExprRef {
+        Expr::Agg(AggExpr::AnyValue(self, ignore_nulls)).into()
     }
 
-    pub fn eq(&self, other: &Self) -> Self {
+    pub fn agg_list(self: ExprRef) -> ExprRef {
+        Expr::Agg(AggExpr::List(self)).into()
+    }
+
+    pub fn agg_concat(self: ExprRef) -> ExprRef {
+        Expr::Agg(AggExpr::Concat(self)).into()
+    }
+
+    #[allow(clippy::should_implement_trait)]
+    pub fn not(self: ExprRef) -> ExprRef {
+        Expr::Not(self).into()
+    }
+
+    pub fn is_null(self: ExprRef) -> ExprRef {
+        Expr::IsNull(self).into()
+    }
+
+    pub fn not_null(self: ExprRef) -> ExprRef {
+        Expr::NotNull(self).into()
+    }
+
+    pub fn fill_null(self: ExprRef, fill_value: ExprRef) -> ExprRef {
+        Expr::FillNull(self, fill_value).into()
+    }
+
+    pub fn is_in(self: ExprRef, items: ExprRef) -> ExprRef {
+        Expr::IsIn(self, items).into()
+    }
+
+    pub fn eq(self: ExprRef, other: ExprRef) -> ExprRef {
         binary_op(Operator::Eq, self, other)
     }
 
-    pub fn not_eq(&self, other: &Self) -> Self {
+    pub fn not_eq(self: ExprRef, other: ExprRef) -> ExprRef {
         binary_op(Operator::NotEq, self, other)
     }
 
-    pub fn and(&self, other: &Self) -> Self {
+    pub fn and(self: ExprRef, other: ExprRef) -> ExprRef {
         binary_op(Operator::And, self, other)
     }
 
-    pub fn or(&self, other: &Self) -> Self {
+    pub fn or(self: ExprRef, other: ExprRef) -> ExprRef {
         binary_op(Operator::Or, self, other)
     }
 
-    pub fn lt(&self, other: &Self) -> Self {
+    pub fn lt(self: ExprRef, other: ExprRef) -> ExprRef {
         binary_op(Operator::Lt, self, other)
     }
 
-    pub fn lt_eq(&self, other: &Self) -> Self {
+    pub fn lt_eq(self: ExprRef, other: ExprRef) -> ExprRef {
         binary_op(Operator::LtEq, self, other)
     }
 
-    pub fn gt(&self, other: &Self) -> Self {
+    pub fn gt(self: ExprRef, other: ExprRef) -> ExprRef {
         binary_op(Operator::Gt, self, other)
     }
 
-    pub fn gt_eq(&self, other: &Self) -> Self {
+    pub fn gt_eq(self: ExprRef, other: ExprRef) -> ExprRef {
         binary_op(Operator::GtEq, self, other)
     }
 
@@ -347,15 +523,17 @@ impl Expr {
                 let child_id = expr.semantic_id(schema);
                 FieldID::new(format!("{child_id}.not_null()"))
             }
-            Function { func, inputs } => {
-                let inputs = inputs
-                    .iter()
-                    .map(|expr| expr.semantic_id(schema).id.to_string())
-                    .collect::<Vec<String>>()
-                    .join(", ");
-                // TODO: check for function idempotency here.
-                FieldID::new(format!("Function_{func:?}({inputs})"))
+            FillNull(expr, fill_value) => {
+                let child_id = expr.semantic_id(schema);
+                let fill_value_id = fill_value.semantic_id(schema);
+                FieldID::new(format!("{child_id}.fill_null({fill_value_id})"))
             }
+            IsIn(expr, items) => {
+                let child_id = expr.semantic_id(schema);
+                let items_id = items.semantic_id(schema);
+                FieldID::new(format!("{child_id}.is_in({items_id})"))
+            }
+            Function { func, inputs } => function_semantic_id(func, inputs, schema),
             BinaryOp { op, left, right } => {
                 let left_id = left.semantic_id(schema);
                 let right_id = right.semantic_id(schema);
@@ -393,20 +571,68 @@ impl Expr {
             Not(expr) | IsNull(expr) | NotNull(expr) | Cast(expr, ..) | Alias(expr, ..) => {
                 vec![expr.clone()]
             }
-            Agg(agg_expr) => vec![agg_expr.child()],
+            Agg(agg_expr) => agg_expr.children(),
 
             // Multiple children.
-            Function { inputs, .. } => inputs.iter().map(|e| e.clone().into()).collect(),
+            Function { inputs, .. } => inputs.clone(),
             BinaryOp { left, right, .. } => {
                 vec![left.clone(), right.clone()]
             }
+            IsIn(expr, items) => vec![expr.clone(), items.clone()],
             IfElse {
                 if_true,
                 if_false,
                 predicate,
             } => {
-                vec![predicate.clone(), if_true.clone(), if_false.clone()]
+                vec![if_true.clone(), if_false.clone(), predicate.clone()]
             }
+            FillNull(expr, fill_value) => vec![expr.clone(), fill_value.clone()],
+        }
+    }
+
+    pub(super) fn with_new_children(&self, children: Vec<ExprRef>) -> Expr {
+        use Expr::*;
+        match self {
+            // no children
+            Column(..) | Literal(..) => self.clone(),
+            // 1 child
+            Not(..) => Not(children.first().expect("Should have 1 child").clone()),
+            Alias(.., name) => Alias(
+                children.first().expect("Should have 1 child").clone(),
+                name.clone(),
+            ),
+            IsNull(..) => IsNull(children.first().expect("Should have 1 child").clone()),
+            NotNull(..) => NotNull(children.first().expect("Should have 1 child").clone()),
+            Cast(.., dtype) => Cast(
+                children.first().expect("Should have 1 child").clone(),
+                dtype.clone(),
+            ),
+            // 2 children
+            BinaryOp { op, .. } => BinaryOp {
+                op: *op,
+                left: children.first().expect("Should have 1 child").clone(),
+                right: children.get(1).expect("Should have 2 child").clone(),
+            },
+            IsIn(..) => IsIn(
+                children.first().expect("Should have 1 child").clone(),
+                children.get(1).expect("Should have 2 child").clone(),
+            ),
+            FillNull(..) => FillNull(
+                children.first().expect("Should have 1 child").clone(),
+                children.get(1).expect("Should have 2 child").clone(),
+            ),
+            // ternary
+            IfElse { .. } => IfElse {
+                if_true: children.first().expect("Should have 1 child").clone(),
+                if_false: children.get(1).expect("Should have 2 child").clone(),
+                predicate: children.get(2).expect("Should have 3 child").clone(),
+            },
+            // N-ary
+            Agg(agg_expr) => Agg(agg_expr.with_new_children(children)),
+            Function { func, .. } => Function {
+                func: func.clone(),
+                inputs: children,
+            },
         }
     }
 
@@ -415,21 +641,38 @@ impl Expr {
         match self {
             Alias(expr, name) => Ok(Field::new(name.as_ref(), expr.get_type(schema)?)),
             Agg(agg_expr) => agg_expr.to_field(schema),
-            Cast(expr, dtype) => Ok(Field::new(expr.name()?, dtype.clone())),
+            Cast(expr, dtype) => Ok(Field::new(expr.name(), dtype.clone())),
             Column(name) => Ok(schema.get_field(name).cloned()?),
             Not(expr) => {
                 let child_field = expr.to_field(schema)?;
                 match child_field.dtype {
-                    DataType::Boolean => Ok(Field::new(expr.name()?, DataType::Boolean)),
+                    DataType::Boolean => Ok(Field::new(expr.name(), DataType::Boolean)),
                     _ => Err(DaftError::TypeError(format!(
                         "Expected argument to be a Boolean expression, but received {child_field}",
                     ))),
                 }
             }
-            IsNull(expr) => Ok(Field::new(expr.name()?, DataType::Boolean)),
-            NotNull(expr) => Ok(Field::new(expr.name()?, DataType::Boolean)),
+            IsNull(expr) => Ok(Field::new(expr.name(), DataType::Boolean)),
+            NotNull(expr) => Ok(Field::new(expr.name(), DataType::Boolean)),
+            FillNull(expr, fill_value) => {
+                let expr_field = expr.to_field(schema)?;
+                let fill_value_field = fill_value.to_field(schema)?;
+                match try_get_supertype(&expr_field.dtype, &fill_value_field.dtype) {
+                    Ok(supertype) => Ok(Field::new(expr_field.name.as_str(), supertype)),
+                    Err(_) => Err(DaftError::TypeError(format!(
+                        "Expected expr and fill_value arguments for fill_null to be castable to the same supertype, but received {expr_field} and {fill_value_field}",
+                    )))
+                }
+            }
+            IsIn(left, right) => {
+                let left_field = left.to_field(schema)?;
+                let right_field = right.to_field(schema)?;
+                let (result_type, _intermediate, _comp_type) =
+                    left_field.dtype.membership_op(&right_field.dtype)?;
+                Ok(Field::new(left_field.name.as_str(), result_type))
+            }
             Literal(value) => Ok(Field::new("literal", value.get_type())),
-            Function { func, inputs } => func.to_field(inputs.as_slice(), schema, self),
+            Function { func, inputs } => func.to_field(inputs.as_slice(), schema, func),
             BinaryOp { op, left, right } => {
                 let left_field = left.to_field(schema)?;
                 let right_field = right.to_field(schema)?;
@@ -484,34 +727,47 @@ impl Expr {
                 if_false,
                 predicate,
             } => {
-                let if_true_field = if_true.to_field(schema)?;
-                let if_false_field = if_false.to_field(schema)?;
                 let predicate_field = predicate.to_field(schema)?;
                 if predicate_field.dtype != DataType::Boolean {
                     return Err(DaftError::TypeError(format!(
                         "Expected predicate for if_else to be boolean but received {predicate_field}",
                     )));
                 }
-                match try_get_supertype(&if_true_field.dtype, &if_false_field.dtype) {
-                    Ok(supertype) => Ok(Field::new(if_true_field.name, supertype)),
-                    Err(_) => Err(DaftError::TypeError(format!("Expected if_true and if_false arguments for if_else to be castable to the same supertype, but received {if_true_field} and {if_false_field}")))
+                match predicate.as_ref() {
+                    Expr::Literal(lit::LiteralValue::Boolean(true)) => if_true.to_field(schema),
+                    Expr::Literal(lit::LiteralValue::Boolean(false)) => {
+                        Ok(if_false.to_field(schema)?.rename(if_true.name()))
+                    }
+                    _ => {
+                        let if_true_field = if_true.to_field(schema)?;
+                        let if_false_field = if_false.to_field(schema)?;
+                        match try_get_supertype(&if_true_field.dtype, &if_false_field.dtype) {
+                            Ok(supertype) => Ok(Field::new(if_true_field.name, supertype)),
+                            Err(_) => Err(DaftError::TypeError(format!("Expected if_true and if_false arguments for if_else to be castable to the same supertype, but received {if_true_field} and {if_false_field}")))
+                        }
+                    }
                 }
             }
         }
     }
 
-    pub fn name(&self) -> DaftResult<&str> {
+    pub fn name(&self) -> &str {
         use Expr::*;
         match self {
-            Alias(.., name) => Ok(name.as_ref()),
+            Alias(.., name) => name.as_ref(),
             Agg(agg_expr) => agg_expr.name(),
             Cast(expr, ..) => expr.name(),
-            Column(name) => Ok(name.as_ref()),
+            Column(name) => name.as_ref(),
             Not(expr) => expr.name(),
             IsNull(expr) => expr.name(),
             NotNull(expr) => expr.name(),
-            Literal(..) => Ok("literal"),
-            Function { func: _, inputs } => inputs.first().unwrap().name(),
+            FillNull(expr, ..) => expr.name(),
+            IsIn(expr, ..) => expr.name(),
+            Literal(..) => "literal",
+            Function { func, inputs } => match func {
+                FunctionExpr::Struct(StructExpr::Get(name)) => name,
+                _ => inputs.first().unwrap().name(),
+            },
             BinaryOp {
                 op: _,
                 left,
@@ -525,7 +781,7 @@ impl Expr {
         Ok(self.to_field(schema)?.dtype)
     }
 
-    pub fn input_mapping(&self) -> Option<String> {
+    pub fn input_mapping(self: &Arc<Self>) -> Option<String> {
         let required_columns = get_required_columns(self);
         let requires_computation = requires_computation(self);
 
@@ -533,10 +789,83 @@ impl Expr {
         //   1. There is only one required column
         //   2. No computation is run on this required column
         match (&required_columns[..], requires_computation) {
-            ([required_col], false) => Some(required_col.clone()),
+            ([required_col], false) => Some(required_col.to_string()),
             _ => None,
         }
     }
+
+    pub fn to_sql(&self) -> Option<String> {
+        fn to_sql_inner<W: Write>(expr: &Expr, buffer: &mut W) -> io::Result<()> {
+            match expr {
+                Expr::Column(name) => write!(buffer, "{}", name),
+                Expr::Literal(lit) => lit.display_sql(buffer),
+                Expr::Alias(inner, ..) => to_sql_inner(inner, buffer),
+                Expr::BinaryOp { op, left, right } => {
+                    to_sql_inner(left, buffer)?;
+                    let op = match op {
+                        Operator::Eq => "=",
+                        Operator::NotEq => "!=",
+                        Operator::Lt => "<",
+                        Operator::LtEq => "<=",
+                        Operator::Gt => ">",
+                        Operator::GtEq => ">=",
+                        Operator::And => "AND",
+                        Operator::Or => "OR",
+                        _ => {
+                            return Err(io::Error::new(
+                                io::ErrorKind::Other,
+                                "Unsupported operator for SQL translation",
+                            ))
+                        }
+                    };
+                    write!(buffer, " {} ", op)?;
+                    to_sql_inner(right, buffer)
+                }
+                Expr::Not(inner) => {
+                    write!(buffer, "NOT (")?;
+                    to_sql_inner(inner, buffer)?;
+                    write!(buffer, ")")
+                }
+                Expr::IsNull(inner) => {
+                    write!(buffer, "(")?;
+                    to_sql_inner(inner, buffer)?;
+                    write!(buffer, ") IS NULL")
+                }
+                Expr::NotNull(inner) => {
+                    write!(buffer, "(")?;
+                    to_sql_inner(inner, buffer)?;
+                    write!(buffer, ") IS NOT NULL")
+                }
+                Expr::IfElse {
+                    if_true,
+                    if_false,
+                    predicate,
+                } => {
+                    write!(buffer, "CASE WHEN ")?;
+                    to_sql_inner(predicate, buffer)?;
+                    write!(buffer, " THEN ")?;
+                    to_sql_inner(if_true, buffer)?;
+                    write!(buffer, " ELSE ")?;
+                    to_sql_inner(if_false, buffer)?;
+                    write!(buffer, " END")
+                }
+                // TODO: Implement SQL translations for these expressions if possible
+                Expr::Agg(..)
+                | Expr::Cast(..)
+                | Expr::IsIn(..)
+                | Expr::Function { .. }
+                | Expr::FillNull(..) => Err(io::Error::new(
+                    io::ErrorKind::Other,
+                    "Unsupported expression for SQL translation",
+                )),
+            }
+        }
+
+        let mut buffer = Vec::new();
+        to_sql_inner(self, &mut buffer)
+            .ok()
+            .and_then(|_| String::from_utf8(buffer).ok())
+    }
 }
 
 impl Display for Expr {
@@ -563,18 +892,10 @@ impl Display for Expr {
             Not(expr) => write!(f, "not({expr})"),
             IsNull(expr) => write!(f, "is_null({expr})"),
             NotNull(expr) => write!(f, "not_null({expr})"),
+            FillNull(expr, fill_value) => write!(f, "fill_null({expr}, {fill_value})"),
+            IsIn(expr, items) => write!(f, "{expr} in {items}"),
             Literal(val) => write!(f, "lit({val})"),
-            Function { func, inputs } => {
-                write!(f, "{}(", func.fn_name())?;
-                for (i, input) in inputs.iter().enumerate() {
-                    if i != 0 {
-                        write!(f, ", ")?;
-                    }
-                    write!(f, "{input}")?;
-                }
-                write!(f, ")")?;
-                Ok(())
-            }
+            Function { func, inputs } => function_display(f, func, inputs),
             IfElse {
                 if_true,
                 if_false,
@@ -592,16 +913,25 @@ impl Display for AggExpr {
         match self {
             Count(expr, mode) => write!(f, "count({expr}, {mode})"),
             Sum(expr) => write!(f, "sum({expr})"),
+            ApproxSketch(expr) => write!(f, "approx_sketch({expr})"),
+            ApproxPercentile(ApproxPercentileParams { child, percentiles, force_list_output }) => write!(
+                f,
+                "approx_percentiles({child}, percentiles={percentiles:?}, force_list_output={force_list_output})"
+            ),
+            MergeSketch(expr) => write!(f, "merge_sketch({expr})"),
             Mean(expr) => write!(f, "mean({expr})"),
             Min(expr) => write!(f, "min({expr})"),
             Max(expr) => write!(f, "max({expr})"),
+            AnyValue(expr, ignore_nulls) => {
+                write!(f, "any_value({expr}, ignore_nulls={ignore_nulls})")
+            }
             List(expr) => write!(f, "list({expr})"),
             Concat(expr) => write!(f, "list({expr})"),
+            MapGroups { func, inputs } => function_display(f, func, inputs),
         }
     }
 }
 
-/// Based on Polars first class operators: https://github.com/pola-rs/polars/blob/master/polars/polars-lazy/polars-plan/src/dsl/expr.rs
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Serialize, Deserialize, Hash)]
 pub enum Operator {
     Eq,
@@ -671,7 +1001,6 @@ impl Operator {
 mod tests {
 
     use super::*;
-    use crate::lit;
     #[test]
     fn check_comparison_type() -> DaftResult<()> {
         let x = lit(10.);
@@ -679,8 +1008,8 @@ mod tests {
         let schema = Schema::empty();
 
         let z = Expr::BinaryOp {
-            left: x.into(),
-            right: y.into(),
+            left: x,
+            right: y,
             op: Operator::Lt,
         };
         assert_eq!(z.get_type(&schema)?, DataType::Boolean);
@@ -691,7 +1020,7 @@ mod tests {
     fn check_alias_type() -> DaftResult<()> {
         let a = col("a");
         let b = a.alias("b");
-        match b {
+        match b.as_ref() {
             Expr::Alias(..) => Ok(()),
             other => Err(common_error::DaftError::ValueError(format!(
                 "expected expression to be a alias, got {other:?}"
@@ -706,8 +1035,8 @@ mod tests {
         let schema = Schema::empty();
 
         let z = Expr::BinaryOp {
-            left: x.into(),
-            right: y.into(),
+            left: x,
+            right: y,
             op: Operator::Plus,
         };
         assert_eq!(z.get_type(&schema)?, DataType::Float64);
@@ -716,8 +1045,8 @@ mod tests {
         let y = lit(12);
 
         let z = Expr::BinaryOp {
-            left: y.into(),
-            right: x.into(),
+            left: y,
+            right: x,
             op: Operator::Plus,
         };
         assert_eq!(z.get_type(&schema)?, DataType::Float64);
@@ -735,8 +1064,8 @@ mod tests {
         ])?;
 
         let z = Expr::BinaryOp {
-            left: x.into(),
-            right: y.into(),
+            left: x,
+            right: y,
             op: Operator::Plus,
         };
         assert_eq!(z.get_type(&schema)?, DataType::Float64);
@@ -745,8 +1074,8 @@ mod tests {
         let y = col("y");
 
         let z = Expr::BinaryOp {
-            left: y.into(),
-            right: x.into(),
+            left: y,
+            right: x,
             op: Operator::Plus,
         };
         assert_eq!(z.get_type(&schema)?, DataType::Float64);
