@@ -5,7 +5,7 @@ use std::sync::Arc;
 use download::DownloadEvaluator;
 use serde::{Deserialize, Serialize};
 
-use crate::Expr;
+use crate::{Expr, ExprRef};
 
 use super::FunctionEvaluator;
 
@@ -32,12 +32,12 @@ impl UriExpr {
 }
 
 pub fn download(
-    input: &Expr,
+    input: ExprRef,
     max_connections: usize,
     raise_error_on_failure: bool,
     multi_thread: bool,
     config: Option<IOConfig>,
-) -> Expr {
+) -> ExprRef {
     Expr::Function {
         func: super::FunctionExpr::Uri(UriExpr::Download {
             max_connections,
@@ -45,6 +45,7 @@ pub fn download(
             multi_thread,
             config: config.unwrap_or_default().into(),
         }),
-        inputs: vec![input.clone()],
+        inputs: vec![input],
     }
+    .into()
 }
