@@ -1,6 +1,7 @@
 use async_trait::async_trait;
-use azure_storage::prelude::*;
+use azure_storage::{prelude::*, CloudLocation};
 use azure_storage_blobs::{
+    blob::operations::GetBlobResponse,
     container::{operations::BlobItem, Container},
     prelude::*,
 };
@@ -33,18 +34,8 @@ enum Error {
     // Generic client errors.
     #[snafu(display("Azure Storage Account not set and is required.\n Set either `AzureConfig.storage_account` or the `AZURE_STORAGE_ACCOUNT` environment variable."))]
     StorageAccountNotSet,
-    #[snafu(display("Unable to create Azure Client {}", source))]
-    UnableToCreateClient { source: azure_storage::Error },
     #[snafu(display("Azure client generic error: {}", source))]
     AzureGeneric { source: azure_storage::Error },
-
-    // Parameterized client errors.
-    #[snafu(display("Unable to connect to {}: {}", path, source))]
-    UnableToConnect {
-        path: String,
-        source: reqwest::Error,
-    },
-
     #[snafu(display("Unable to open {}: {}", path, source))]
     UnableToOpenFile {
         path: String,
@@ -66,11 +57,8 @@ enum Error {
     #[snafu(display("Not Found: \"{}\"", path))]
     NotFound { path: String },
 
-    #[snafu(display("Unable to access container: {}: {}", path, source))]
-    ContainerAccess {
-        path: String,
-        source: azure_storage::Error,
-    },
+    #[snafu(display("Not a File: \"{}\"", path))]
+    NotAFile { path: String },
 }
 
 impl From<Error> for super::Error {
@@ -98,6 +86,7 @@ impl From<Error> for super::Error {
                 path: path.into(),
                 source: error.into(),
             },
+            NotAFile { path } => super::Error::NotAFile { path },
             _ => super::Error::Generic {
                 store: super::SourceType::AzureBlob,
                 source: error.into(),
@@ -117,9 +106,9 @@ impl AzureBlobSource {
         } else if let Ok(storage_account) = std::env::var("AZURE_STORAGE_ACCOUNT") {
             storage_account
         } else {
+            // TODO(Clark): Allow no storage account + anonymous access + custom endpoint.
             return Err(Error::StorageAccountNotSet.into());
         };
-
         let storage_credentials = if config.anonymous {
             StorageCredentials::anonymous()
         } else if let Some(access_key) = &config.access_key {
@@ -130,7 +119,22 @@ impl AzureBlobSource {
             log::warn!("Azure access key not found, Set either `AzureConfig.access_key` or the `AZURE_STORAGE_KEY` environment variable. Defaulting to anonymous mode.");
             StorageCredentials::anonymous()
         };
-        let blob_client = BlobServiceClient::new(storage_account, storage_credentials);
+        let endpoint_url = if let Some(endpoint_url) = &config.endpoint_url {
+            Some(endpoint_url.clone())
+        } else if let Ok(endpoint_url) = std::env::var("AZURE_ENDPOINT_URL") {
+            Some(endpoint_url)
+        } else {
+            None
+        };
+        let blob_client = if let Some(endpoint_url) = endpoint_url {
+            ClientBuilder::with_location(
+                CloudLocation::Custom { uri: endpoint_url },
+                storage_credentials,
+            )
+            .blob_service_client()
+        } else {
+            BlobServiceClient::new(storage_account, storage_credentials)
+        };
 
         Ok(AzureBlobSource {
             blob_client: blob_client.into(),
@@ -414,6 +418,10 @@ impl ObjectSource for AzureBlobSource {
         }?;
         let key = parsed.path();
 
+        if key.is_empty() {
+            return Err(Error::NotAFile { path: uri.into() }.into());
+        }
+
         let container_client = self.blob_client.container_client(container);
         let blob_client = container_client.blob_client(key);
         let request_builder = blob_client.get();
@@ -426,7 +434,7 @@ impl ObjectSource for AzureBlobSource {
 
         let owned_string = uri.to_string();
         let stream = blob_stream
-            .and_then(async move |v| v.data.collect().await)
+            .and_then(async move |v: GetBlobResponse| v.data.collect().await)
             .map_err(move |e| {
                 UnableToReadBytesSnafu::<String> {
                     path: owned_string.clone(),
@@ -455,6 +463,10 @@ impl ObjectSource for AzureBlobSource {
         }?;
         let key = parsed.path();
 
+        if key.is_empty() {
+            return Err(Error::NotAFile { path: uri.into() }.into());
+        }
+
         let container_client = self.blob_client.container_client(container);
         let blob_client = container_client.blob_client(key);
         let metadata = blob_client
