@@ -1,6 +1,6 @@
 #![feature(async_closure)]
 #![feature(let_chains)]
-
+#![feature(io_error_more)]
 mod azure_blob;
 mod google_cloud;
 mod http;
@@ -54,6 +54,9 @@ pub enum Error {
     #[snafu(display("Invalid Argument: {:?}", msg))]
     InvalidArgument { msg: String },
 
+    #[snafu(display("Unable to expand home dir"))]
+    HomeDirError { path: String },
+
     #[snafu(display("Unable to open file {}: {:?}", path, source))]
     UnableToOpenFile { path: String, source: DynError },
 
@@ -63,6 +66,23 @@ pub enum Error {
         source: std::io::Error,
     },
 
+    #[snafu(display(
+        "Connection timed out when trying to connect to {}\nDetails:\n{:?}",
+        path,
+        source
+    ))]
+    ConnectTimeout { path: String, source: DynError },
+
+    #[snafu(display("Read timed out when trying to read {}\nDetails:\n{:?}", path, source))]
+    ReadTimeout { path: String, source: DynError },
+
+    #[snafu(display(
+        "Socket error occurred when trying to read {}\nDetails:\n{:?}",
+        path,
+        source
+    ))]
+    SocketError { path: String, source: DynError },
+
     #[snafu(display("Unable to convert URL \"{}\" to path", path))]
     InvalidUrl {
         path: String,
@@ -72,6 +92,9 @@ pub enum Error {
     #[snafu(display("Not a File: \"{}\"", path))]
     NotAFile { path: String },
 
+    #[snafu(display("Unable to determine size of {}", path))]
+    UnableToDetermineSize { path: String },
+
     #[snafu(display("Unable to load Credentials for store: {store}\nDetails:\n{source:?}"))]
     UnableToLoadCredentials { store: SourceType, source: DynError },
 
@@ -99,6 +122,9 @@ pub enum Error {
 
     #[snafu(display("Error joining spawned task: {}", source), context(false))]
     JoinError { source: tokio::task::JoinError },
+
+    #[snafu(display("Cached error: {}", source))]
+    CachedError { source: Arc<Error> },
 }
 
 impl From<Error> for DaftError {
@@ -106,6 +132,22 @@ impl From<Error> for DaftError {
         use Error::*;
         match err {
             NotFound { path, source } => DaftError::FileNotFound { path, source },
+            ConnectTimeout { .. } => DaftError::ConnectTimeout(err.into()),
+            ReadTimeout { .. } => DaftError::ReadTimeout(err.into()),
+            UnableToReadBytes { .. } => DaftError::ByteStreamError(err.into()),
+            SocketError { .. } => DaftError::SocketError(err.into()),
+            // We have to repeat everything above for the case we have an Arc since we can't move the error.
+            CachedError { ref source } => match source.as_ref() {
+                NotFound { path, source: _ } => DaftError::FileNotFound {
+                    path: path.clone(),
+                    source: err.into(),
+                },
+                ConnectTimeout { .. } => DaftError::ConnectTimeout(err.into()),
+                ReadTimeout { .. } => DaftError::ReadTimeout(err.into()),
+                UnableToReadBytes { .. } => DaftError::ByteStreamError(err.into()),
+                SocketError { .. } => DaftError::SocketError(err.into()),
+                _ => DaftError::External(err.into()),
+            },
             _ => DaftError::External(err.into()),
         }
     }
@@ -261,11 +303,25 @@ impl std::fmt::Display for SourceType {
 
 pub fn parse_url(input: &str) -> Result<(SourceType, Cow<'_, str>)> {
     let mut fixed_input = Cow::Borrowed(input);
+    // handle tilde `~` expansion
+    if input.starts_with("~/") {
+        return home::home_dir()
+            .and_then(|home_dir| {
+                let expanded = home_dir.join(&input[2..]);
+                let input = expanded.to_str()?;
+
+                Some((SourceType::File, Cow::Owned(format!("file://{}", input))))
+            })
+            .ok_or_else(|| crate::Error::InvalidArgument {
+                msg: "Could not convert expanded path to string".to_string(),
+            });
+    }
 
     let url = match url::Url::parse(input) {
         Ok(url) => Ok(url),
         Err(ParseError::RelativeUrlWithoutBase) => {
             fixed_input = Cow::Owned(format!("file://{input}"));
+
             url::Url::parse(fixed_input.as_ref())
         }
         Err(err) => Err(err),
@@ -277,7 +333,7 @@ pub fn parse_url(input: &str) -> Result<(SourceType, Cow<'_, str>)> {
         "file" => Ok((SourceType::File, fixed_input)),
         "http" | "https" => Ok((SourceType::Http, fixed_input)),
         "s3" | "s3a" => Ok((SourceType::S3, fixed_input)),
-        "az" | "abfs" => Ok((SourceType::AzureBlob, fixed_input)),
+        "az" | "abfs" | "abfss" => Ok((SourceType::AzureBlob, fixed_input)),
         "gcs" | "gs" => Ok((SourceType::GCS, fixed_input)),
         #[cfg(target_env = "msvc")]
         _ if scheme.len() == 1 && ("a" <= scheme.as_str() && (scheme.as_str() <= "z")) => {
