@@ -31,10 +31,10 @@ where
         None => Ok(first),
         Some(second) => {
             let size_hint = size_hint.unwrap_or_else(|| first.len() + second.len());
-
             let mut buf = Vec::with_capacity(size_hint);
             buf.extend_from_slice(&first);
             buf.extend_from_slice(&second);
+
             while let Some(maybe_bytes) = stream.next().await {
                 buf.extend_from_slice(&maybe_bytes?);
             }
@@ -101,14 +101,7 @@ pub(crate) trait ObjectSource: Sync + Send {
         range: Option<Range<usize>>,
         io_stats: Option<IOStatsRef>,
     ) -> super::Result<GetResult>;
-    async fn get_range(
-        &self,
-        uri: &str,
-        range: Range<usize>,
-        io_stats: Option<IOStatsRef>,
-    ) -> super::Result<GetResult> {
-        self.get(uri, Some(range), io_stats).await
-    }
+
     async fn get_size(&self, uri: &str, io_stats: Option<IOStatsRef>) -> super::Result<usize>;
 
     async fn glob(
@@ -146,7 +139,7 @@ pub(crate) trait ObjectSource: Sync + Send {
             let mut continuation_token = lsr.continuation_token.clone();
             while continuation_token.is_some() {
                 let lsr = self.ls(&uri, posix, continuation_token.as_deref(), page_size, io_stats.clone()).await?;
-                continuation_token = lsr.continuation_token.clone();
+                continuation_token.clone_from(&lsr.continuation_token);
                 for fm in lsr.files {
                     yield Ok(fm);
                 }
