@@ -33,6 +33,7 @@ use aws_sdk_s3::primitives::ByteStreamError;
 
 use std::collections::HashMap;
 
+use std::io;
 use std::ops::Range;
 use std::string::FromUtf8Error;
 use std::sync::Arc;
@@ -68,12 +69,6 @@ enum Error {
         source: SdkError<ListObjectsV2Error, Response>,
     },
 
-    #[snafu(display("Unable to query the region for {}: {}", path, source))]
-    UnableToQueryRegion {
-        path: String,
-        source: reqwest::Error,
-    },
-
     #[snafu(display("Unable missing header: {header} when performing request for: {path}"))]
     MissingHeader { path: String, header: String },
 
@@ -97,9 +92,6 @@ enum Error {
     #[snafu(display("Unable to load Credentials: {}", source))]
     UnableToLoadCredentials { source: CredentialsError },
 
-    #[snafu(display("Unable to create http client. {}", source))]
-    UnableToCreateClient { source: reqwest::Error },
-
     #[snafu(display("Unable to grab semaphore. {}", source))]
     UnableToGrabSemaphore { source: tokio::sync::AcquireError },
 
@@ -116,63 +108,142 @@ enum Error {
 impl From<Error> for super::Error {
     fn from(error: Error) -> Self {
         use Error::*;
+
         match error {
-            UnableToOpenFile { path, source } => match source.into_service_error() {
-                GetObjectError::NoSuchKey(no_such_key) => super::Error::NotFound {
+            UnableToOpenFile { path, source } => match source {
+                SdkError::TimeoutError(_) => super::Error::ReadTimeout {
                     path,
-                    source: no_such_key.into(),
+                    source: source.into(),
                 },
-                GetObjectError::Unhandled(v) => super::Error::Unhandled {
-                    path,
-                    msg: DisplayErrorContext(v).to_string(),
-                },
-                err => super::Error::UnableToOpenFile {
-                    path,
-                    source: err.into(),
+                SdkError::DispatchFailure(ref dispatch) => {
+                    if dispatch.is_timeout() {
+                        super::Error::ConnectTimeout {
+                            path,
+                            source: source.into(),
+                        }
+                    } else if dispatch.is_io() {
+                        super::Error::SocketError {
+                            path,
+                            source: source.into(),
+                        }
+                    } else {
+                        super::Error::UnableToOpenFile {
+                            path,
+                            source: source.into(),
+                        }
+                    }
+                }
+                _ => match source.into_service_error() {
+                    GetObjectError::NoSuchKey(no_such_key) => super::Error::NotFound {
+                        path,
+                        source: no_such_key.into(),
+                    },
+                    GetObjectError::Unhandled(v) => super::Error::Unhandled {
+                        path,
+                        msg: DisplayErrorContext(v).to_string(),
+                    },
+                    err => super::Error::UnableToOpenFile {
+                        path,
+                        source: err.into(),
+                    },
                 },
             },
-            UnableToHeadFile { path, source } => match source.into_service_error() {
-                HeadObjectError::NotFound(no_such_key) => super::Error::NotFound {
+            UnableToHeadFile { path, source } => match source {
+                SdkError::TimeoutError(_) => super::Error::ReadTimeout {
                     path,
-                    source: no_such_key.into(),
+                    source: source.into(),
                 },
-                HeadObjectError::Unhandled(v) => super::Error::Unhandled {
-                    path,
-                    msg: DisplayErrorContext(v).to_string(),
-                },
-                err => super::Error::UnableToOpenFile {
-                    path,
-                    source: err.into(),
+                SdkError::DispatchFailure(ref dispatch) => {
+                    if dispatch.is_timeout() {
+                        super::Error::ConnectTimeout {
+                            path,
+                            source: source.into(),
+                        }
+                    } else if dispatch.is_io() {
+                        super::Error::SocketError {
+                            path,
+                            source: source.into(),
+                        }
+                    } else {
+                        super::Error::UnableToOpenFile {
+                            path,
+                            source: source.into(),
+                        }
+                    }
+                }
+                _ => match source.into_service_error() {
+                    HeadObjectError::NotFound(no_such_key) => super::Error::NotFound {
+                        path,
+                        source: no_such_key.into(),
+                    },
+                    HeadObjectError::Unhandled(v) => super::Error::Unhandled {
+                        path,
+                        msg: DisplayErrorContext(v).to_string(),
+                    },
+                    err => super::Error::UnableToOpenFile {
+                        path,
+                        source: err.into(),
+                    },
                 },
             },
-            UnableToListObjects { path, source } => match source.into_service_error() {
-                ListObjectsV2Error::NoSuchBucket(no_such_key) => super::Error::NotFound {
+            UnableToListObjects { path, source } => match source {
+                SdkError::TimeoutError(_) => super::Error::ReadTimeout {
                     path,
-                    source: no_such_key.into(),
+                    source: source.into(),
                 },
-                ListObjectsV2Error::Unhandled(v) => super::Error::Unhandled {
-                    path,
-                    msg: DisplayErrorContext(v).to_string(),
-                },
-                err => super::Error::UnableToOpenFile {
-                    path,
-                    source: err.into(),
+                SdkError::DispatchFailure(ref dispatch) => {
+                    if dispatch.is_timeout() {
+                        super::Error::ConnectTimeout {
+                            path,
+                            source: source.into(),
+                        }
+                    } else if dispatch.is_io() {
+                        super::Error::SocketError {
+                            path,
+                            source: source.into(),
+                        }
+                    } else {
+                        super::Error::UnableToOpenFile {
+                            path,
+                            source: source.into(),
+                        }
+                    }
+                }
+                _ => match source.into_service_error() {
+                    ListObjectsV2Error::NoSuchBucket(no_such_key) => super::Error::NotFound {
+                        path,
+                        source: no_such_key.into(),
+                    },
+                    ListObjectsV2Error::Unhandled(v) => super::Error::Unhandled {
+                        path,
+                        msg: DisplayErrorContext(v).to_string(),
+                    },
+                    err => super::Error::UnableToOpenFile {
+                        path,
+                        source: err.into(),
+                    },
                 },
             },
             InvalidUrl { path, source } => super::Error::InvalidUrl { path, source },
-            UnableToReadBytes { path, source } => super::Error::UnableToReadBytes {
-                path,
-                source: source.into(),
-            },
+            UnableToReadBytes { path, source } => {
+                use std::error::Error;
+                let io_error = if let Some(source) = source.source() {
+                    // if we have a source, lets extract out the error as a string rather than rely on the aws-sdk fmt.
+                    let source_as_string = source.to_string();
+                    std::io::Error::new(io::ErrorKind::Other, source_as_string)
+                } else {
+                    std::io::Error::new(io::ErrorKind::Other, source)
+                };
+                super::Error::UnableToReadBytes {
+                    path,
+                    source: io_error,
+                }
+            }
             NotAFile { path } => super::Error::NotAFile { path },
             UnableToLoadCredentials { source } => super::Error::UnableToLoadCredentials {
                 store: SourceType::S3,
                 source: source.into(),
             },
-            UnableToCreateClient { source } => super::Error::UnableToCreateClient {
-                store: SourceType::S3,
-                source: source.into(),
-            },
             NotFound { ref path } => super::Error::NotFound {
                 path: path.into(),
                 source: error.into(),
@@ -185,6 +256,33 @@ impl From<Error> for super::Error {
     }
 }
 
+/// Retrieves an S3Config from the environment by leveraging the AWS SDK's credentials chain
+pub(crate) async fn s3_config_from_env() -> super::Result<S3Config> {
+    let default_s3_config = S3Config::default();
+    let (anonymous, s3_conf) = build_s3_conf(&default_s3_config, None).await?;
+    let creds = s3_conf
+        .credentials_cache()
+        .provide_cached_credentials()
+        .await
+        .with_context(|_| UnableToLoadCredentialsSnafu {})?;
+    let key_id = Some(creds.access_key_id().to_string());
+    let access_key = Some(creds.secret_access_key().to_string());
+    let session_token = creds.session_token().map(|t| t.to_string());
+    let region_name = s3_conf.region().map(|r| r.to_string());
+    Ok(S3Config {
+        // Do not perform auto-discovery of endpoint_url. This is possible, but requires quite a bit
+        // of work that our current implementation of `build_s3_conf` does not yet do. See smithy-rs code:
+        // https://github.com/smithy-lang/smithy-rs/blob/94ecd38c2518583042796b2b45c37947237e31dd/aws/rust-runtime/aws-config/src/lib.rs#L824-L849
+        endpoint_url: None,
+        region_name,
+        key_id,
+        session_token,
+        access_key,
+        anonymous,
+        ..default_s3_config
+    })
+}
+
 /// Helper to parse S3 URLs, returning (scheme, bucket, key)
 fn parse_url(uri: &str) -> super::Result<(String, String, String)> {
     let parsed = url::Url::parse(uri).with_context(|_| InvalidUrlSnafu { path: uri })?;
@@ -236,24 +334,80 @@ fn handle_https_client_settings(
     Ok(builder)
 }
 
-async fn build_s3_client(
+async fn build_s3_conf(
     config: &S3Config,
     credentials_cache: Option<SharedCredentialsCache>,
-) -> super::Result<(bool, s3::Client)> {
+) -> super::Result<(bool, s3::Config)> {
     const DEFAULT_REGION: Region = Region::from_static("us-east-1");
 
     let mut anonymous = config.anonymous;
 
-    let conf: SdkConfig = if anonymous {
-        aws_config::SdkConfig::builder().build()
+    let cached_creds = if let Some(credentials_cache) = credentials_cache {
+        let creds = credentials_cache.provide_cached_credentials().await;
+        creds.ok()
     } else {
-        aws_config::load_from_env().await
+        None
     };
+
+    let provider = if let Some(cached_creds) = cached_creds {
+        let provider = CredentialsProviderChain::first_try("different_region_cache", cached_creds)
+            .or_default_provider()
+            .await;
+        Some(aws_credential_types::provider::SharedCredentialsProvider::new(provider))
+    } else if config.access_key.is_some() && config.key_id.is_some() {
+        let creds = Credentials::from_keys(
+            config.key_id.clone().unwrap(),
+            config.access_key.clone().unwrap(),
+            config.session_token.clone(),
+        );
+        Some(aws_credential_types::provider::SharedCredentialsProvider::new(creds))
+    } else if config.access_key.is_some() || config.key_id.is_some() {
+        return Err(super::Error::InvalidArgument {
+            msg: "Must provide both access_key and key_id when building S3-Like Client".to_string(),
+        });
+    } else {
+        None
+    };
+
+    let conf: SdkConfig = if anonymous {
+        let mut builder = aws_config::SdkConfig::builder();
+        builder.set_credentials_provider(provider);
+        builder.build()
+    } else {
+        let loader = aws_config::from_env();
+        let loader = if let Some(profile_name) = &config.profile_name {
+            loader.profile_name(profile_name)
+        } else {
+            loader
+        };
+
+        // Set region now to avoid imds
+        let loader = if let Some(region) = &config.region_name {
+            loader.region(Region::new(region.to_owned()))
+        } else {
+            loader
+        };
+        // Set creds now to avoid imds
+        let loader = if let Some(provider) = provider {
+            loader.credentials_provider(provider)
+        } else {
+            loader
+        };
+
+        loader.load().await
+    };
+
     let builder = aws_sdk_s3::config::Builder::from(&conf);
     let builder = match &config.endpoint_url {
         None => builder,
-        Some(endpoint) => builder.endpoint_url(endpoint).force_path_style(true),
+        Some(endpoint) => builder.endpoint_url(endpoint),
     };
+    let builder = if config.endpoint_url.is_some() && !config.force_virtual_addressing {
+        builder.force_path_style(true)
+    } else {
+        builder.force_path_style(false)
+    };
+
     let builder = if let Some(region) = &config.region_name {
         builder.region(Region::new(region.to_owned()))
     } else {
@@ -294,33 +448,6 @@ async fn build_s3_client(
         .build();
     let builder = builder.timeout_config(timeout_config);
 
-    let cached_creds = if let Some(credentials_cache) = credentials_cache {
-        let creds = credentials_cache.provide_cached_credentials().await;
-        creds.ok()
-    } else {
-        None
-    };
-
-    let builder = if let Some(cached_creds) = cached_creds {
-        let provider = CredentialsProviderChain::first_try("different_region_cache", cached_creds)
-            .or_default_provider()
-            .await;
-        builder.credentials_provider(provider)
-    } else if config.access_key.is_some() && config.key_id.is_some() {
-        let creds = Credentials::from_keys(
-            config.key_id.clone().unwrap(),
-            config.access_key.clone().unwrap(),
-            config.session_token.clone(),
-        );
-        builder.credentials_provider(creds)
-    } else if config.access_key.is_some() || config.key_id.is_some() {
-        return Err(super::Error::InvalidArgument {
-            msg: "Must provide both access_key and key_id when building S3-Like Client".to_string(),
-        });
-    } else {
-        builder
-    };
-
     let builder_copy = builder.clone();
     let s3_conf = builder.build();
     const CRED_TRIES: u64 = 4;
@@ -370,6 +497,15 @@ async fn build_s3_client(
     } else {
         s3_conf
     };
+
+    Ok((anonymous, s3_conf))
+}
+
+async fn build_s3_client(
+    config: &S3Config,
+    credentials_cache: Option<SharedCredentialsCache>,
+) -> super::Result<(bool, s3::Client)> {
+    let (anonymous, s3_conf) = build_s3_conf(config, credentials_cache).await?;
     Ok((anonymous, s3::Client::from_conf(s3_conf)))
 }
 
@@ -448,6 +584,12 @@ impl S3LikeSource {
                 .bucket(bucket)
                 .key(key);
 
+            let request = if self.s3_config.requester_pays {
+                request.request_payer(s3::types::RequestPayer::Requester)
+            } else {
+                request
+            };
+
             let request = match &range {
                 None => request,
                 Some(range) => request.range(format!(
@@ -552,6 +694,12 @@ impl S3LikeSource {
                 .bucket(bucket)
                 .key(key);
 
+            let request = if self.s3_config.requester_pays {
+                request.request_payer(s3::types::RequestPayer::Requester)
+            } else {
+                request
+            };
+
             let response = if self.anonymous {
                 request
                     .customize_middleware()
@@ -643,6 +791,12 @@ impl S3LikeSource {
         } else {
             request
         };
+        let request = if self.s3_config.requester_pays {
+            request.request_payer(s3::types::RequestPayer::Requester)
+        } else {
+            request
+        };
+
         let response = if self.anonymous {
             request
                 .customize_middleware()
@@ -894,7 +1048,7 @@ impl ObjectSource for S3LikeSource {
                 lsr.files.retain(|f| f.filepath == target_path);
 
                 if lsr.files.is_empty() {
-                    // Isnt a file or a directory
+                    // Isn't a file or a directory
                     return Err(Error::NotFound { path: path.into() }.into());
                 }
                 Ok(lsr)
@@ -956,7 +1110,7 @@ mod tests {
         assert_eq!(checksum, parquet_expected_md5);
 
         let first_bytes = client
-            .get_range(parquet_file_path, 0..10, None)
+            .get(parquet_file_path, Some(0..10), None)
             .await?
             .bytes()
             .await?;
@@ -964,7 +1118,7 @@ mod tests {
         assert_eq!(first_bytes.as_ref(), &all_bytes[..10]);
 
         let first_bytes = client
-            .get_range(parquet_file_path, 10..100, None)
+            .get(parquet_file_path, Some(10..100), None)
             .await?
             .bytes()
             .await?;
@@ -972,9 +1126,9 @@ mod tests {
         assert_eq!(first_bytes.as_ref(), &all_bytes[10..100]);
 
         let last_bytes = client
-            .get_range(
+            .get(
                 parquet_file_path,
-                (all_bytes.len() - 10)..(all_bytes.len() + 10),
+                Some((all_bytes.len() - 10)..(all_bytes.len() + 10)),
                 None,
             )
             .await?
