@@ -3,7 +3,7 @@ use std::{collections::HashMap, num::NonZeroUsize, sync::Arc};
 use common_error::{DaftError, DaftResult};
 use daft_core::{schema::Schema, utils::arrow::cast_array_for_daft_if_needed, Series};
 use daft_dsl::optimization::get_required_columns;
-use daft_io::{get_runtime, GetResult, IOClient, IOStatsRef};
+use daft_io::{get_runtime, parse_url, GetResult, IOClient, IOStatsRef, SourceType};
 use daft_table::Table;
 use futures::{Stream, StreamExt, TryStreamExt};
 use rayon::prelude::{IntoParallelIterator, ParallelIterator};
@@ -18,26 +18,22 @@ use tokio::{
 };
 use tokio_util::io::StreamReader;
 
-use crate::{decoding::deserialize_records, ArrowSnafu, ChunkSnafu};
+use crate::{decoding::deserialize_records, local::read_json_local, ArrowSnafu, ChunkSnafu};
 use crate::{
     schema::read_json_schema_single, JsonConvertOptions, JsonParseOptions, JsonReadOptions,
 };
 use daft_compression::CompressionCodec;
 
-trait LineChunkStream = Stream<Item = super::Result<Vec<String>>>;
-trait ColumnArrayChunkStream = Stream<
-    Item = super::Result<
-        Context<
-            JoinHandle<DaftResult<Vec<Box<dyn arrow2::array::Array>>>>,
-            super::JoinSnafu,
-            super::Error,
-        >,
-    >,
->;
+type TableChunkResult =
+    super::Result<Context<JoinHandle<DaftResult<Table>>, super::JoinSnafu, super::Error>>;
 
-trait TableChunkStream = Stream<
-    Item = super::Result<Context<JoinHandle<DaftResult<Table>>, super::JoinSnafu, super::Error>>,
->;
+type LineChunkResult = super::Result<Vec<String>>;
+
+trait TableChunkStream: Stream<Item = TableChunkResult> {}
+impl<T> TableChunkStream for T where T: Stream<Item = TableChunkResult> {}
+
+trait LineChunkStream: Stream<Item = LineChunkResult> {}
+impl<T> LineChunkStream for T where T: Stream<Item = LineChunkResult> {}
 
 #[allow(clippy::too_many_arguments)]
 pub fn read_json(
@@ -136,7 +132,7 @@ pub fn read_json_bulk(
 
 // Parallel version of table concat
 // get rid of this once Table APIs are parallel
-fn tables_concat(mut tables: Vec<Table>) -> DaftResult<Table> {
+pub(crate) fn tables_concat(mut tables: Vec<Table>) -> DaftResult<Table> {
     if tables.is_empty() {
         return Err(DaftError::ValueError(
             "Need at least 1 Table to perform concat".to_string(),
@@ -170,13 +166,6 @@ fn tables_concat(mut tables: Vec<Table>) -> DaftResult<Table> {
     Table::new(first_table.schema.clone(), new_series)
 }
 
-#[inline]
-fn assert_stream_send<'u, R>(
-    s: impl 'u + Send + Stream<Item = R>,
-) -> impl 'u + Send + Stream<Item = R> {
-    s
-}
-
 async fn read_json_single_into_table(
     uri: &str,
     convert_options: Option<JsonConvertOptions>,
@@ -186,6 +175,18 @@ async fn read_json_single_into_table(
     io_stats: Option<IOStatsRef>,
     max_chunks_in_flight: Option<usize>,
 ) -> DaftResult<Table> {
+    let (source_type, fixed_uri) = parse_url(uri)?;
+    let is_compressed = CompressionCodec::from_uri(uri).is_some();
+    if matches!(source_type, SourceType::File) && !is_compressed {
+        return read_json_local(
+            fixed_uri.as_ref(),
+            convert_options,
+            parse_options,
+            read_options,
+            max_chunks_in_flight,
+        );
+    }
+
     let predicate = convert_options.as_ref().and_then(|p| p.predicate.clone());
 
     let limit = convert_options.as_ref().and_then(|opts| opts.limit);
@@ -228,16 +229,15 @@ async fn read_json_single_into_table(
             .unwrap_or(NonZeroUsize::new(2).unwrap())
             .checked_mul(2.try_into().unwrap())
             .unwrap()
-            .try_into()
-            .unwrap()
+            .into()
     });
     let tables = table_stream
         // Limit the number of chunks we have in flight at any given time.
         .try_buffered(max_chunks_in_flight);
 
-    let filtered_tables = assert_stream_send(tables.map_ok(move |table| {
+    let filtered_tables = tables.map_ok(move |table| {
         if let Some(predicate) = &predicate {
-            let filtered = table?.filter(&[predicate.as_ref()])?;
+            let filtered = table?.filter(&[predicate.clone()])?;
             if let Some(include_columns) = &include_columns {
                 filtered.get_columns(include_columns.as_slice())
             } else {
@@ -246,7 +246,7 @@ async fn read_json_single_into_table(
         } else {
             table
         }
-    }));
+    });
     let mut remaining_rows = limit.map(|limit| limit as i64);
     let collected_tables = filtered_tables
         .try_take_while(|result| {
@@ -274,8 +274,10 @@ async fn read_json_single_into_table(
     }
     // // TODO(Clark): Don't concatenate all chunks from a file into a single table, since MicroPartition is natively chunked.
     let concated_table = tables_concat(collected_tables)?;
-    if let Some(limit) = limit && concated_table.len() > limit {
-        // apply head incase that last chunk went over limit
+    if let Some(limit) = limit
+        && concated_table.len() > limit
+    {
+        // apply head in case that last chunk went over limit
         concated_table.head(limit)
     } else {
         Ok(concated_table)
@@ -418,7 +420,7 @@ fn parse_into_column_array_chunk_stream(
     );
     // Parsing stream: we spawn background tokio + rayon tasks so we can pipeline chunk parsing with chunk reading, and
     // we further parse each chunk column in parallel on the rayon threadpool.
-    Ok(stream.map_ok(move |records| {
+    Ok(stream.map_ok(move |mut records| {
         let schema = schema.clone();
         let daft_schema = daft_schema.clone();
         let daft_fields = daft_fields.clone();
@@ -428,16 +430,15 @@ fn parse_into_column_array_chunk_stream(
                 let result = (move || {
                     // TODO(Clark): Switch to streaming parse + array construction?
                     let parsed = records
-                        .iter()
+                        .iter_mut()
                         .map(|unparsed_record| {
-                            json_deserializer::parse(unparsed_record.as_bytes()).map_err(|e| {
-                                super::Error::JsonDeserializationError {
+                            crate::deserializer::to_value(unsafe { unparsed_record.as_bytes_mut() })
+                                .map_err(|e| super::Error::JsonDeserializationError {
                                     string: e.to_string(),
-                                }
-                            })
+                                })
                         })
                         .collect::<super::Result<Vec<_>>>()?;
-                    let chunk = deserialize_records(parsed, schema.as_ref(), schema_is_projection)
+                    let chunk = deserialize_records(&parsed, schema.as_ref(), schema_is_projection)
                         .context(ArrowSnafu)?;
                     let all_series = chunk
                         .into_iter()
@@ -491,11 +492,11 @@ mod tests {
         projection: Option<Vec<String>>,
     ) {
         let reader = std::io::BufReader::new(std::fs::File::open(path).unwrap());
-        let lines = reader.lines().collect::<Vec<_>>();
+        let mut lines = reader.lines().map(|l| l.unwrap()).collect::<Vec<_>>();
         let parsed = lines
-            .iter()
+            .iter_mut()
             .take(limit.unwrap_or(usize::MAX))
-            .map(|record| json_deserializer::parse(record.as_ref().unwrap().as_bytes()).unwrap())
+            .map(|record| crate::deserializer::to_value(unsafe { record.as_bytes_mut() }).unwrap())
             .collect::<Vec<_>>();
         // Get consolidated schema from parsed JSON.
         let mut column_types: IndexMap<String, HashSet<arrow2::datatypes::DataType>> =
@@ -535,7 +536,7 @@ mod tests {
             None => (field_map.into_values().collect::<Vec<_>>().into(), false),
         };
         // Deserialize JSON records into Arrow2 column arrays.
-        let columns = deserialize_records(parsed, &schema, is_projection).unwrap();
+        let columns = deserialize_records(&parsed, &schema, is_projection).unwrap();
         // Roundtrip columns with Daft for casting.
         let columns = columns
             .into_iter()
@@ -1019,7 +1020,7 @@ mod tests {
             Schema::new(vec![
                 Field::new("sepalLength", DataType::Float64),
                 Field::new("sepalWidth", DataType::Float64),
-                // All null column should have hte provided dtype.
+                // All null column should have the provided dtype.
                 Field::new("petalLength", DataType::Float64),
                 Field::new("petalWidth", DataType::Float64),
                 Field::new("species", DataType::Utf8),
