@@ -5,6 +5,7 @@ use std::{
 
 use common_error::DaftResult;
 use daft_core::{
+    join::JoinType,
     python::{datatype::PyTimeUnit, schema::PySchema, PySeries},
     schema::Schema,
     Series,
@@ -17,7 +18,7 @@ use daft_parquet::read::ParquetSchemaInferenceOptions;
 use daft_scan::{python::pylib::PyScanTask, storage_config::PyStorageConfig, ScanTask};
 use daft_stats::TableStatistics;
 use daft_table::python::PyTable;
-use pyo3::{exceptions::PyValueError, prelude::*, types::PyBytes, Python};
+use pyo3::{exceptions::PyValueError, prelude::*, types::PyBytes};
 
 use crate::micropartition::{MicroPartition, TableState};
 
@@ -160,7 +161,7 @@ impl PyMicroPartition {
     }
 
     pub fn eval_expression_list(&self, py: Python, exprs: Vec<PyExpr>) -> PyResult<Self> {
-        let converted_exprs: Vec<daft_dsl::Expr> = exprs.into_iter().map(|e| e.into()).collect();
+        let converted_exprs: Vec<daft_dsl::ExprRef> = exprs.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
                 .inner
@@ -174,7 +175,7 @@ impl PyMicroPartition {
     }
 
     pub fn filter(&self, py: Python, exprs: Vec<PyExpr>) -> PyResult<Self> {
-        let converted_exprs: Vec<daft_dsl::Expr> = exprs.into_iter().map(|e| e.into()).collect();
+        let converted_exprs: Vec<daft_dsl::ExprRef> = exprs.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| Ok(self.inner.filter(converted_exprs.as_slice())?.into()))
     }
 
@@ -184,7 +185,7 @@ impl PyMicroPartition {
         sort_keys: Vec<PyExpr>,
         descending: Vec<bool>,
     ) -> PyResult<Self> {
-        let converted_exprs: Vec<daft_dsl::Expr> =
+        let converted_exprs: Vec<daft_dsl::ExprRef> =
             sort_keys.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
@@ -200,7 +201,7 @@ impl PyMicroPartition {
         sort_keys: Vec<PyExpr>,
         descending: Vec<bool>,
     ) -> PyResult<PySeries> {
-        let converted_exprs: Vec<daft_dsl::Expr> =
+        let converted_exprs: Vec<daft_dsl::ExprRef> =
             sort_keys.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
@@ -211,8 +212,9 @@ impl PyMicroPartition {
     }
 
     pub fn agg(&self, py: Python, to_agg: Vec<PyExpr>, group_by: Vec<PyExpr>) -> PyResult<Self> {
-        let converted_to_agg: Vec<daft_dsl::Expr> = to_agg.into_iter().map(|e| e.into()).collect();
-        let converted_group_by: Vec<daft_dsl::Expr> =
+        let converted_to_agg: Vec<daft_dsl::ExprRef> =
+            to_agg.into_iter().map(|e| e.into()).collect();
+        let converted_group_by: Vec<daft_dsl::ExprRef> =
             group_by.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
@@ -222,30 +224,108 @@ impl PyMicroPartition {
         })
     }
 
-    pub fn join(
+    pub fn pivot(
+        &self,
+        py: Python,
+        group_by: Vec<PyExpr>,
+        pivot_col: PyExpr,
+        values_col: PyExpr,
+        names: Vec<String>,
+    ) -> PyResult<Self> {
+        let converted_group_by: Vec<daft_dsl::ExprRef> =
+            group_by.into_iter().map(|e| e.into()).collect();
+        let converted_pivot_col: daft_dsl::ExprRef = pivot_col.into();
+        let converted_values_col: daft_dsl::ExprRef = values_col.into();
+        py.allow_threads(|| {
+            Ok(self
+                .inner
+                .pivot(
+                    converted_group_by.as_slice(),
+                    converted_pivot_col,
+                    converted_values_col,
+                    names,
+                )?
+                .into())
+        })
+    }
+
+    pub fn hash_join(
         &self,
         py: Python,
         right: &Self,
         left_on: Vec<PyExpr>,
         right_on: Vec<PyExpr>,
+        how: JoinType,
     ) -> PyResult<Self> {
-        let left_exprs: Vec<daft_dsl::Expr> = left_on.into_iter().map(|e| e.into()).collect();
-        let right_exprs: Vec<daft_dsl::Expr> = right_on.into_iter().map(|e| e.into()).collect();
+        let left_exprs: Vec<daft_dsl::ExprRef> = left_on.into_iter().map(|e| e.into()).collect();
+        let right_exprs: Vec<daft_dsl::ExprRef> = right_on.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
                 .inner
-                .join(&right.inner, left_exprs.as_slice(), right_exprs.as_slice())?
+                .hash_join(
+                    &right.inner,
+                    left_exprs.as_slice(),
+                    right_exprs.as_slice(),
+                    how,
+                )?
+                .into())
+        })
+    }
+
+    pub fn sort_merge_join(
+        &self,
+        py: Python,
+        right: &Self,
+        left_on: Vec<PyExpr>,
+        right_on: Vec<PyExpr>,
+        is_sorted: bool,
+    ) -> PyResult<Self> {
+        let left_exprs: Vec<daft_dsl::ExprRef> = left_on.into_iter().map(|e| e.into()).collect();
+        let right_exprs: Vec<daft_dsl::ExprRef> = right_on.into_iter().map(|e| e.into()).collect();
+        py.allow_threads(|| {
+            Ok(self
+                .inner
+                .sort_merge_join(
+                    &right.inner,
+                    left_exprs.as_slice(),
+                    right_exprs.as_slice(),
+                    is_sorted,
+                )?
                 .into())
         })
     }
 
     pub fn explode(&self, py: Python, to_explode: Vec<PyExpr>) -> PyResult<Self> {
-        let converted_to_explode: Vec<daft_dsl::Expr> =
+        let converted_to_explode: Vec<daft_dsl::ExprRef> =
             to_explode.into_iter().map(|e| e.expr).collect();
 
         py.allow_threads(|| Ok(self.inner.explode(converted_to_explode.as_slice())?.into()))
     }
 
+    pub fn unpivot(
+        &self,
+        py: Python,
+        ids: Vec<PyExpr>,
+        values: Vec<PyExpr>,
+        variable_name: &str,
+        value_name: &str,
+    ) -> PyResult<Self> {
+        let converted_ids: Vec<daft_dsl::ExprRef> = ids.into_iter().map(|e| e.into()).collect();
+        let converted_values: Vec<daft_dsl::ExprRef> =
+            values.into_iter().map(|e| e.into()).collect();
+        py.allow_threads(|| {
+            Ok(self
+                .inner
+                .unpivot(
+                    converted_ids.as_slice(),
+                    converted_values.as_slice(),
+                    variable_name,
+                    value_name,
+                )?
+                .into())
+        })
+    }
+
     pub fn head(&self, py: Python, num: i64) -> PyResult<Self> {
         py.allow_threads(|| {
             if num < 0 {
@@ -324,7 +404,7 @@ impl PyMicroPartition {
                 "Can not partition into negative number of partitions: {num_partitions}"
             )));
         }
-        let exprs: Vec<daft_dsl::Expr> = exprs.into_iter().map(|e| e.into()).collect();
+        let exprs: Vec<daft_dsl::ExprRef> = exprs.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
                 .inner
@@ -369,7 +449,7 @@ impl PyMicroPartition {
         boundaries: &PyTable,
         descending: Vec<bool>,
     ) -> PyResult<Vec<Self>> {
-        let exprs: Vec<daft_dsl::Expr> = partition_keys.into_iter().map(|e| e.into()).collect();
+        let exprs: Vec<daft_dsl::ExprRef> = partition_keys.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
                 .inner
@@ -380,6 +460,34 @@ impl PyMicroPartition {
         })
     }
 
+    pub fn partition_by_value(
+        &self,
+        py: Python,
+        partition_keys: Vec<PyExpr>,
+    ) -> PyResult<(Vec<Self>, Self)> {
+        let exprs: Vec<daft_dsl::ExprRef> = partition_keys.into_iter().map(|e| e.into()).collect();
+        py.allow_threads(|| {
+            let (mps, values) = self.inner.partition_by_value(exprs.as_slice())?;
+            let mps = mps.into_iter().map(|m| m.into()).collect::<Vec<Self>>();
+            let values = values.into();
+            Ok((mps, values))
+        })
+    }
+
+    pub fn add_monotonically_increasing_id(
+        &self,
+        py: Python,
+        partition_num: u64,
+        column_name: &str,
+    ) -> PyResult<Self> {
+        py.allow_threads(|| {
+            Ok(self
+                .inner
+                .add_monotonically_increasing_id(partition_num, column_name)?
+                .into())
+        })
+    }
+
     #[staticmethod]
     pub fn read_json(
         py: Python,
@@ -486,12 +594,15 @@ impl PyMicroPartition {
                 start_offset,
                 num_rows,
                 row_groups.map(|rg| vec![Some(rg)]),
-                predicate.map(|e| e.expr.into()),
+                predicate.map(|e| e.expr),
+                None,
                 io_config,
                 Some(io_stats),
                 1,
                 multithreaded_io.unwrap_or(true),
                 &schema_infer_options,
+                None,
+                None,
             )
         })?;
         Ok(mp.into())
@@ -526,12 +637,15 @@ impl PyMicroPartition {
                 start_offset,
                 num_rows,
                 row_groups,
-                predicate.map(|e| e.expr.into()),
+                predicate.map(|e| e.expr),
+                None,
                 io_config,
                 Some(io_stats),
                 num_parallel_tasks.unwrap_or(128) as usize,
                 multithreaded_io.unwrap_or(true),
                 &schema_infer_options,
+                None,
+                None,
             )
         })?;
         Ok(mp.into())
@@ -732,6 +846,42 @@ pub(crate) fn read_parquet_into_py_table(
         .extract()
 }
 
+pub(crate) fn read_sql_into_py_table(
+    py: Python,
+    sql: &str,
+    conn: &PyObject,
+    predicate: Option<PyExpr>,
+    schema: PySchema,
+    include_columns: Option<Vec<String>>,
+    num_rows: Option<usize>,
+) -> PyResult<PyTable> {
+    let py_schema = py
+        .import(pyo3::intern!(py, "daft.logical.schema"))?
+        .getattr(pyo3::intern!(py, "Schema"))?
+        .getattr(pyo3::intern!(py, "_from_pyschema"))?
+        .call1((schema,))?;
+    let py_predicate = match predicate {
+        Some(p) => Some(
+            py.import(pyo3::intern!(py, "daft.expressions.expressions"))?
+                .getattr(pyo3::intern!(py, "Expression"))?
+                .getattr(pyo3::intern!(py, "_from_pyexpr"))?
+                .call1((p,))?,
+        ),
+        None => None,
+    };
+    let read_options = py
+        .import(pyo3::intern!(py, "daft.runners.partitioning"))?
+        .getattr(pyo3::intern!(py, "TableReadOptions"))?
+        .call1((num_rows, include_columns))?;
+    py.import(pyo3::intern!(py, "daft.table.table_io"))?
+        .getattr(pyo3::intern!(py, "read_sql"))?
+        .call1((sql, conn, py_schema, read_options, py_predicate))?
+        .getattr(pyo3::intern!(py, "to_table"))?
+        .call0()?
+        .getattr(pyo3::intern!(py, "_table"))?
+        .extract()
+}
+
 impl From<MicroPartition> for PyMicroPartition {
     fn from(value: MicroPartition) -> Self {
         PyMicroPartition {
