@@ -2,19 +2,20 @@ use std::sync::Arc;
 
 use daft_core::datatypes::FieldID;
 use daft_core::schema::{Schema, SchemaRef};
-use daft_dsl::{optimization, AggExpr, Expr, ExprRef};
+use daft_dsl::{optimization, AggExpr, ApproxPercentileParams, Expr, ExprRef};
 use indexmap::{IndexMap, IndexSet};
+use itertools::Itertools;
 use snafu::ResultExt;
 
+use crate::logical_optimization::Transformed;
 use crate::logical_plan::{CreationSnafu, Result};
-use crate::optimization::Transformed;
 use crate::{LogicalPlan, ResourceRequest};
 
 #[derive(Clone, Debug, PartialEq, Eq, Hash)]
 pub struct Project {
     // Upstream node.
     pub input: Arc<LogicalPlan>,
-    pub projection: Vec<Expr>,
+    pub projection: Vec<ExprRef>,
     pub resource_request: ResourceRequest,
     pub projected_schema: SchemaRef,
 }
@@ -22,7 +23,7 @@ pub struct Project {
 impl Project {
     pub(crate) fn try_new(
         input: Arc<LogicalPlan>,
-        projection: Vec<Expr>,
+        projection: Vec<ExprRef>,
         resource_request: ResourceRequest,
     ) -> Result<Self> {
         // Factor the projection and see if there are any substitutions to factor out.
@@ -49,19 +50,15 @@ impl Project {
     pub fn multiline_display(&self) -> Vec<String> {
         vec![format!(
             "Project: {}",
-            self.projection
-                .iter()
-                .map(|e| e.to_string())
-                .collect::<Vec<_>>()
-                .join(", ")
+            self.projection.iter().map(|e| e.to_string()).join(", ")
         )]
     }
 
     fn try_factor_subexpressions(
         input: Arc<LogicalPlan>,
-        projection: Vec<Expr>,
+        projection: Vec<ExprRef>,
         resource_request: &ResourceRequest,
-    ) -> Result<(Arc<LogicalPlan>, Vec<Expr>)> {
+    ) -> Result<(Arc<LogicalPlan>, Vec<ExprRef>)> {
         // Given construction parameters for a projection,
         // see if we can factor out common subexpressions.
         // Returns a new set of projection parameters
@@ -80,7 +77,7 @@ impl Project {
                 .into_iter()
                 .map(|colname| {
                     let expr = &substitutions[&colname];
-                    Expr::Alias(expr.clone().into(), colname.clone().into())
+                    expr.clone().alias(colname)
                 })
                 .collect::<Vec<_>>();
 
@@ -92,9 +89,9 @@ impl Project {
     }
 
     fn factor_expressions(
-        exprs: Vec<Expr>,
+        exprs: Vec<ExprRef>,
         schema: &Schema,
-    ) -> (Vec<Expr>, IndexMap<String, Expr>) {
+    ) -> (Vec<ExprRef>, IndexMap<String, ExprRef>) {
         // Returns
         //  1. original expressions with substitutions
         //  2. substitution definitions
@@ -113,7 +110,7 @@ impl Project {
         // all existing names must also be converted to semantic IDs.
         let mut column_name_substitutions = IndexMap::new();
 
-        let mut exprs_to_walk: Vec<Arc<Expr>> = exprs.iter().map(|e| e.clone().into()).collect();
+        let mut exprs_to_walk: Vec<Arc<Expr>> = exprs.to_vec();
         while !exprs_to_walk.is_empty() {
             exprs_to_walk = exprs_to_walk
                 .iter()
@@ -158,22 +155,17 @@ impl Project {
             let substituted_expressions = exprs
                 .iter()
                 .map(|e| {
-                    let new_expr = replace_column_with_semantic_id(
-                        e.clone().into(),
-                        &subexprs_to_replace,
-                        schema,
-                    )
-                    .unwrap()
-                    .as_ref()
-                    .clone();
+                    let new_expr =
+                        replace_column_with_semantic_id(e.clone(), &subexprs_to_replace, schema);
+                    let new_expr = new_expr.unwrap();
                     // The substitution can unintentionally change the expression's name
                     // (since the name depends on the first column referenced, which can be substituted away)
                     // so re-alias the original name here if it has changed.
-                    let old_name = e.name().unwrap();
-                    if new_expr.name().unwrap() != old_name {
+                    let old_name = e.name();
+                    if new_expr.name() != old_name {
                         new_expr.alias(old_name)
                     } else {
-                        new_expr
+                        new_expr.clone()
                     }
                 })
                 .collect::<Vec<_>>();
@@ -181,7 +173,7 @@ impl Project {
             let substitutions = subexpressions_to_cache
                 .iter()
                 .chain(column_name_substitutions.iter())
-                .map(|(k, v)| (k.id.as_ref().to_string(), v.as_ref().clone()))
+                .map(|(k, v)| (k.id.as_ref().to_string(), v.clone()))
                 .collect::<IndexMap<_, _>>();
 
             (substituted_expressions, substitutions)
@@ -195,7 +187,7 @@ fn replace_column_with_semantic_id(
     schema: &Schema,
 ) -> Transformed<ExprRef> {
     // Constructs a new copy of this expression
-    // with all occurences of subexprs_to_replace replaced with a column selection.
+    // with all occurrences of subexprs_to_replace replaced with a column selection.
     // e.g. e := (a+b)+c, subexprs := {FieldID("(a + b)")}
     //  -> Col("(a + b)") + c
 
@@ -254,6 +246,35 @@ fn replace_column_with_semantic_id(
                         |_| e,
                     )
             }
+            Expr::FillNull(child, fill_value) => {
+                let child =
+                    replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema);
+                let fill_value = replace_column_with_semantic_id(
+                    fill_value.clone(),
+                    subexprs_to_replace,
+                    schema,
+                );
+                if child.is_no() && fill_value.is_no() {
+                    Transformed::No(e)
+                } else {
+                    Transformed::Yes(
+                        Expr::FillNull(child.unwrap().clone(), fill_value.unwrap().clone()).into(),
+                    )
+                }
+            }
+            Expr::IsIn(child, items) => {
+                let child =
+                    replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema);
+                let items =
+                    replace_column_with_semantic_id(items.clone(), subexprs_to_replace, schema);
+                if child.is_no() && items.is_no() {
+                    Transformed::No(e)
+                } else {
+                    Transformed::Yes(
+                        Expr::IsIn(child.unwrap().clone(), items.unwrap().clone()).into(),
+                    )
+                }
+            }
             Expr::BinaryOp { op, left, right } => {
                 let left =
                     replace_column_with_semantic_id(left.clone(), subexprs_to_replace, schema);
@@ -300,11 +321,7 @@ fn replace_column_with_semantic_id(
                 let transforms = inputs
                     .iter()
                     .map(|e| {
-                        replace_column_with_semantic_id(
-                            e.clone().into(),
-                            subexprs_to_replace,
-                            schema,
-                        )
+                        replace_column_with_semantic_id(e.clone(), subexprs_to_replace, schema)
                     })
                     .collect::<Vec<_>>();
                 if transforms.iter().all(|e| e.is_no()) {
@@ -313,11 +330,7 @@ fn replace_column_with_semantic_id(
                     Transformed::Yes(
                         Expr::Function {
                             func: func.clone(),
-                            inputs: transforms
-                                .iter()
-                                .map(|t| t.unwrap().as_ref())
-                                .cloned()
-                                .collect(),
+                            inputs: transforms.iter().map(|t| t.unwrap()).cloned().collect(),
                         }
                         .into(),
                     )
@@ -333,7 +346,7 @@ fn replace_column_with_semantic_id_aggexpr(
     schema: &Schema,
 ) -> Transformed<AggExpr> {
     // Constructs a new copy of this expression
-    // with all occurences of subexprs_to_replace replaced with a column selection.
+    // with all occurrences of subexprs_to_replace replaced with a column selection.
     // e.g. e := (a+b)+c, subexprs := {FieldID("(a + b)")}
     //  -> Col("(a + b)") + c
 
@@ -348,6 +361,29 @@ fn replace_column_with_semantic_id_aggexpr(
             replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
                 .map_yes_no(AggExpr::Sum, |_| e.clone())
         }
+        AggExpr::ApproxSketch(ref child) => {
+            replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
+                .map_yes_no(AggExpr::ApproxSketch, |_| e.clone())
+        }
+        AggExpr::ApproxPercentile(ApproxPercentileParams {
+            ref child,
+            ref percentiles,
+            ref force_list_output,
+        }) => replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
+            .map_yes_no(
+                |transformed_child| {
+                    AggExpr::ApproxPercentile(ApproxPercentileParams {
+                        child: transformed_child,
+                        percentiles: percentiles.clone(),
+                        force_list_output: *force_list_output,
+                    })
+                },
+                |_| e.clone(),
+            ),
+        AggExpr::MergeSketch(ref child) => {
+            replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
+                .map_yes_no(AggExpr::MergeSketch, |_| e.clone())
+        }
         AggExpr::Mean(ref child) => {
             replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
                 .map_yes_no(AggExpr::Mean, |_| e.clone())
@@ -360,6 +396,12 @@ fn replace_column_with_semantic_id_aggexpr(
             replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
                 .map_yes_no(AggExpr::Max, |_| e.clone())
         }
+        AggExpr::AnyValue(ref child, ignore_nulls) => {
+            replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema).map_yes_no(
+                |transformed_child| AggExpr::AnyValue(transformed_child, ignore_nulls),
+                |_| e.clone(),
+            )
+        }
         AggExpr::List(ref child) => {
             replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
                 .map_yes_no(AggExpr::List, |_| e.clone())
@@ -368,6 +410,20 @@ fn replace_column_with_semantic_id_aggexpr(
             replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
                 .map_yes_no(AggExpr::Concat, |_| e.clone())
         }
+        AggExpr::MapGroups { func, inputs } => {
+            let transforms = inputs
+                .iter()
+                .map(|e| replace_column_with_semantic_id(e.clone(), subexprs_to_replace, schema))
+                .collect::<Vec<_>>();
+            if transforms.iter().all(|e| e.is_no()) {
+                Transformed::No(AggExpr::MapGroups { func, inputs })
+            } else {
+                Transformed::Yes(AggExpr::MapGroups {
+                    func: func.clone(),
+                    inputs: transforms.iter().map(|t| t.unwrap()).cloned().collect(),
+                })
+            }
+        }
     }
 }
 
@@ -377,7 +433,11 @@ mod tests {
     use daft_core::{datatypes::Field, DataType};
     use daft_dsl::{binary_op, col, lit, Operator};
 
-    use crate::{logical_ops::Project, test::dummy_scan_node, LogicalPlan};
+    use crate::{
+        logical_ops::Project,
+        test::{dummy_scan_node, dummy_scan_operator},
+        LogicalPlan,
+    };
 
     /// Test that nested common subexpressions are correctly split
     /// into multiple levels of projections.
@@ -389,27 +449,31 @@ mod tests {
     /// 3: a+a as aa
     #[test]
     fn test_nested_subexpression() -> DaftResult<()> {
-        let source = dummy_scan_node(vec![
+        let source = dummy_scan_node(dummy_scan_operator(vec![
             Field::new("a", DataType::Int64),
             Field::new("b", DataType::Int64),
-        ])
+        ]))
         .build();
-        let a2 = binary_op(Operator::Plus, &col("a"), &col("a"));
-        let a4 = binary_op(Operator::Plus, &a2, &a2);
-        let a8 = binary_op(Operator::Plus, &a4, &a4);
+        let a2 = binary_op(Operator::Plus, col("a"), col("a"));
+        let a2_colname = a2.semantic_id(&source.schema()).id;
+
+        let a4 = binary_op(Operator::Plus, a2.clone(), a2.clone());
+        let a4_colname = a4.semantic_id(&source.schema()).id;
+
+        let a8 = binary_op(Operator::Plus, a4.clone(), a4.clone());
         let expressions = vec![a8.alias("x")];
         let result_projection = Project::try_new(source.clone(), expressions, Default::default())?;
 
-        let a4_colname = a4.semantic_id(&source.schema()).id;
         let a4_col = col(a4_colname.clone());
         let expected_result_projection =
-            vec![binary_op(Operator::Plus, &a4_col, &a4_col).alias("x")];
+            vec![binary_op(Operator::Plus, a4_col.clone(), a4_col.clone()).alias("x")];
         assert_eq!(result_projection.projection, expected_result_projection);
 
-        let a2_colname = a2.semantic_id(&source.schema()).id;
         let a2_col = col(a2_colname.clone());
         let expected_subprojection =
-            vec![binary_op(Operator::Plus, &a2_col, &a2_col).alias(a4_colname.clone())];
+            vec![
+                binary_op(Operator::Plus, a2_col.clone(), a2_col.clone()).alias(a4_colname.clone())
+            ];
         let LogicalPlan::Project(subprojection) = result_projection.input.as_ref() else {
             panic!()
         };
@@ -433,27 +497,29 @@ mod tests {
     /// 2. a+a as aa, a
     #[test]
     fn test_shared_subexpression() -> DaftResult<()> {
-        let source = dummy_scan_node(vec![
+        let source = dummy_scan_node(dummy_scan_operator(vec![
             Field::new("a", DataType::Int64),
             Field::new("b", DataType::Int64),
-        ])
+        ]))
         .build();
-        let a2 = binary_op(Operator::Plus, &col("a"), &col("a"));
+        let a2 = binary_op(Operator::Plus, col("a"), col("a"));
+        let a2_colname = a2.semantic_id(&source.schema()).id;
+
         let expressions = vec![
-            a2.alias("x"),
-            binary_op(Operator::Plus, &a2, &col("a")).alias("y"),
+            a2.clone().alias("x"),
+            binary_op(Operator::Plus, a2.clone(), col("a")).alias("y"),
         ];
         let result_projection = Project::try_new(source.clone(), expressions, Default::default())?;
 
-        let a2_colname = a2.semantic_id(&source.schema()).id;
         let a2_col = col(a2_colname.clone());
         let expected_result_projection = vec![
             a2_col.alias("x"),
-            binary_op(Operator::Plus, &a2_col, &col("a")).alias("y"),
+            binary_op(Operator::Plus, a2_col, col("a")).alias("y"),
         ];
         assert_eq!(result_projection.projection, expected_result_projection);
 
-        let expected_subprojection = vec![a2.alias(a2_colname.clone()), col("a").alias("a")];
+        let expected_subprojection =
+            vec![a2.clone().alias(a2_colname.clone()), col("a").alias("a")];
         let LogicalPlan::Project(subprojection) = result_projection.input.as_ref() else {
             panic!()
         };
@@ -470,10 +536,10 @@ mod tests {
     /// (unchanged)
     #[test]
     fn test_vacuous_subexpression() -> DaftResult<()> {
-        let source = dummy_scan_node(vec![
+        let source = dummy_scan_node(dummy_scan_operator(vec![
             Field::new("a", DataType::Int64),
             Field::new("b", DataType::Int64),
-        ])
+        ]))
         .build();
         let expressions = vec![
             lit(3).alias("x"),
