@@ -3,10 +3,10 @@ use std::sync::Arc;
 use daft_core::schema::SchemaRef;
 use daft_scan::ScanExternalInfo;
 
-use crate::source_info::{ExternalInfo, LegacyExternalInfo, SourceInfo};
+use crate::source_info::SourceInfo;
 
-#[cfg(feature = "python")]
 use crate::source_info::InMemoryInfo;
+use crate::source_info::PlaceHolderInfo;
 
 #[derive(Clone, Debug, PartialEq, Eq, Hash)]
 pub struct Source {
@@ -30,32 +30,14 @@ impl Source {
         let mut res = vec![];
 
         match self.source_info.as_ref() {
-            SourceInfo::ExternalInfo(ExternalInfo::Legacy(LegacyExternalInfo {
-                source_schema,
-                file_infos,
-                file_format_config,
-                storage_config,
-                pushdowns,
-            })) => {
-                res.push(format!("Source: {}", file_format_config.var_name()));
-                res.push(format!(
-                    "File paths = [{}]",
-                    file_infos.file_paths.join(", ")
-                ));
-                res.push(format!("File schema = {}", source_schema.short_string()));
-                res.push(format!("Format-specific config = {:?}", file_format_config));
-                res.push(format!("Storage config = {:?}", storage_config));
-                res.extend(pushdowns.multiline_display());
-            }
-            SourceInfo::ExternalInfo(ExternalInfo::Scan(ScanExternalInfo {
+            SourceInfo::External(ScanExternalInfo {
                 source_schema,
                 scan_op,
                 partitioning_keys,
                 pushdowns,
-            })) => {
+            }) => {
                 use itertools::Itertools;
-                res.push("Source:".to_string());
-                res.push(format!("{}", scan_op));
+                res.extend(scan_op.0.multiline_display());
 
                 res.push(format!("File schema = {}", source_schema.short_string()));
                 res.push(format!(
@@ -64,11 +46,19 @@ impl Source {
                 ));
                 res.extend(pushdowns.multiline_display());
             }
-            #[cfg(feature = "python")]
-            SourceInfo::InMemoryInfo(InMemoryInfo { num_partitions, .. }) => {
+            SourceInfo::InMemory(InMemoryInfo { num_partitions, .. }) => {
                 res.push("Source:".to_string());
                 res.push(format!("Number of partitions = {}", num_partitions));
             }
+            SourceInfo::PlaceHolder(PlaceHolderInfo {
+                source_id,
+                clustering_spec,
+                ..
+            }) => {
+                res.push("PlaceHolder:".to_string());
+                res.push(format!("Source ID = {}", source_id));
+                res.extend(clustering_spec.multiline_display());
+            }
         }
         res.push(format!(
             "Output schema = {}",
