@@ -1,51 +1,33 @@
 use std::sync::Arc;
 
 use daft_core::{datatypes::Field, schema::Schema};
-use daft_scan::{file_format::FileFormatConfig, storage_config::StorageConfig, Pushdowns};
+use daft_scan::{
+    file_format::FileFormatConfig, storage_config::NativeStorageConfig,
+    storage_config::StorageConfig, AnonymousScanOperator, Pushdowns, ScanOperator,
+};
 
-use crate::{builder::LogicalPlanBuilder, source_info::FileInfos, NativeStorageConfig};
-
-/// Create a dummy scan node containing the provided fields in its schema.
-pub fn dummy_scan_node(fields: Vec<Field>) -> LogicalPlanBuilder {
-    dummy_scan_node_with_pushdowns(fields, Default::default())
-}
+use crate::builder::LogicalPlanBuilder;
 
 /// Create a dummy scan node containing the provided fields in its schema and the provided limit.
-pub fn dummy_scan_node_with_pushdowns(
-    fields: Vec<Field>,
-    pushdowns: Pushdowns,
-) -> LogicalPlanBuilder {
+pub fn dummy_scan_operator(fields: Vec<Field>) -> Arc<dyn ScanOperator> {
     let schema = Arc::new(Schema::new(fields).unwrap());
-
-    LogicalPlanBuilder::table_scan_with_pushdowns(
-        FileInfos::new_internal(vec!["/foo".to_string()], vec![None], vec![None]),
-        schema,
-        FileFormatConfig::Json(Default::default()).into(),
-        StorageConfig::Native(NativeStorageConfig::new_internal(true, None).into()).into(),
-        pushdowns,
-    )
-    .unwrap()
-}
-
-pub fn dummy_scan_operator_node(fields: Vec<Field>) -> LogicalPlanBuilder {
-    dummy_scan_operator_node_with_pushdowns(fields, Default::default())
-}
-
-/// Create a dummy scan node containing the provided fields in its schema and the provided limit.
-pub fn dummy_scan_operator_node_with_pushdowns(
-    fields: Vec<Field>,
-    pushdowns: Pushdowns,
-) -> LogicalPlanBuilder {
-    let schema = Arc::new(Schema::new(fields).unwrap());
-    let anon = daft_scan::AnonymousScanOperator::new(
+    Arc::new(AnonymousScanOperator::new(
         vec!["/foo".to_string()],
         schema,
         FileFormatConfig::Json(Default::default()).into(),
         StorageConfig::Native(NativeStorageConfig::new_internal(true, None).into()).into(),
-    );
-    LogicalPlanBuilder::table_scan_with_scan_operator(
-        daft_scan::ScanOperatorRef(Arc::new(anon)),
-        Some(pushdowns),
-    )
-    .unwrap()
+    ))
+}
+
+/// Create a dummy scan node containing the provided fields in its schema.
+pub fn dummy_scan_node(scan_op: Arc<dyn ScanOperator>) -> LogicalPlanBuilder {
+    dummy_scan_node_with_pushdowns(scan_op, Default::default())
+}
+
+/// Create a dummy scan node containing the provided fields in its schema and the provided limit.
+pub fn dummy_scan_node_with_pushdowns(
+    scan_op: Arc<dyn ScanOperator>,
+    pushdowns: Pushdowns,
+) -> LogicalPlanBuilder {
+    LogicalPlanBuilder::table_scan(daft_scan::ScanOperatorRef(scan_op), Some(pushdowns)).unwrap()
 }
