@@ -1,4 +1,42 @@
-use pyo3::prelude::*;
+use pyo3::{prelude::*, types::PyTuple, AsPyPointer};
+use serde::{Deserialize, Serialize};
+
+use crate::py_object_serde::{deserialize_py_object, serialize_py_object};
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+struct PyObjectSerializableWrapper(
+    #[serde(
+        serialize_with = "serialize_py_object",
+        deserialize_with = "deserialize_py_object"
+    )]
+    pub PyObject,
+);
+
+/// Python arguments to a Python function that produces Tables
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct PythonTablesFactoryArgs(Vec<PyObjectSerializableWrapper>);
+
+impl PythonTablesFactoryArgs {
+    pub fn new(args: Vec<PyObject>) -> Self {
+        Self(args.into_iter().map(PyObjectSerializableWrapper).collect())
+    }
+
+    pub fn to_pytuple<'a>(&self, py: Python<'a>) -> &'a PyTuple {
+        pyo3::types::PyTuple::new(py, self.0.iter().map(|x| x.0.as_ref(py)))
+    }
+}
+
+impl PartialEq for PythonTablesFactoryArgs {
+    fn eq(&self, other: &Self) -> bool {
+        if self.0.len() != other.0.len() {
+            return false;
+        }
+        self.0
+            .iter()
+            .zip(other.0.iter())
+            .all(|(s, o)| (s.0.as_ptr() as isize) == (o.0.as_ptr() as isize))
+    }
+}
 
 pub mod pylib {
     use common_error::DaftResult;
@@ -9,6 +47,7 @@ pub mod pylib {
     use daft_core::impl_bincode_py_state_serialization;
     use daft_stats::PartitionSpec;
     use daft_stats::TableMetadata;
+    use daft_stats::TableStatistics;
     use daft_table::python::PyTable;
     use daft_table::Table;
     use pyo3::prelude::*;
@@ -17,8 +56,6 @@ pub mod pylib {
     use pyo3::types::PyList;
     use pyo3::PyTypeInfo;
 
-    use std::fmt::Display;
-
     use std::sync::Arc;
 
     use daft_core::python::schema::PySchema;
@@ -27,6 +64,8 @@ pub mod pylib {
     use serde::{Deserialize, Serialize};
 
     use crate::anonymous::AnonymousScanOperator;
+    use crate::file_format::FileFormatConfig;
+    use crate::storage_config::PythonStorageConfig;
     use crate::DataFileSource;
     use crate::PartitionField;
     use crate::Pushdowns;
@@ -37,7 +76,9 @@ pub mod pylib {
     use crate::file_format::PyFileFormatConfig;
     use crate::glob::GlobScanOperator;
     use crate::storage_config::PyStorageConfig;
+    use common_daft_config::PyDaftExecutionConfig;
 
+    use super::PythonTablesFactoryArgs;
     #[pyclass(module = "daft.daft", frozen)]
     #[derive(Debug, Clone)]
     pub struct ScanOperatorHandle {
@@ -175,12 +216,6 @@ pub mod pylib {
         }
     }
 
-    impl Display for PythonScanOperatorBridge {
-        fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-            write!(f, "PythonScanOperator: {}", self.display_name,)
-        }
-    }
-
     impl ScanOperator for PythonScanOperatorBridge {
         fn partitioning_keys(&self) -> &[crate::PartitionField] {
             &self.partitioning_keys
@@ -198,6 +233,11 @@ pub mod pylib {
             self.can_absorb_select
         }
 
+        fn multiline_display(&self) -> Vec<String> {
+            let lines = vec![format!("PythonScanOperator: {}", self.display_name)];
+            lines
+        }
+
         fn to_scan_tasks(
             &self,
             pushdowns: Pushdowns,
@@ -248,6 +288,17 @@ pub mod pylib {
         pub fn size_bytes(&self) -> PyResult<Option<i64>> {
             Ok(self.0.size_bytes().map(i64::try_from).transpose()?)
         }
+
+        pub fn estimate_in_memory_size_bytes(
+            &self,
+            cfg: PyDaftExecutionConfig,
+        ) -> PyResult<Option<i64>> {
+            Ok(self
+                .0
+                .estimate_in_memory_size_bytes(Some(cfg.config.as_ref()))
+                .map(i64::try_from)
+                .transpose()?)
+        }
     }
 
     #[pymethods]
@@ -263,10 +314,14 @@ pub mod pylib {
             size_bytes: Option<u64>,
             pushdowns: Option<PyPushdowns>,
             partition_values: Option<PyTable>,
+            stats: Option<PyTable>,
         ) -> PyResult<Option<Self>> {
-            if let Some(ref pvalues) = partition_values && let Some(Some(ref partition_filters)) = pushdowns.as_ref().map(|p| &p.0.partition_filters) {
+            if let Some(ref pvalues) = partition_values
+                && let Some(Some(ref partition_filters)) =
+                    pushdowns.as_ref().map(|p| &p.0.partition_filters)
+            {
                 let table = &pvalues.table;
-                let eval_pred = table.eval_expression_list(&[partition_filters.as_ref().clone()])?;
+                let eval_pred = table.eval_expression_list(&[partition_filters.clone()])?;
                 assert_eq!(eval_pred.num_columns(), 1);
                 let series = eval_pred.get_column_by_index(0)?;
                 assert_eq!(series.data_type(), &daft_core::DataType::Boolean);
@@ -278,12 +333,16 @@ pub mod pylib {
                     None | Some(true) => {}
                 }
             }
+            // TODO(Clark): Filter out scan tasks with pushed down filters + table stats?
 
             let pspec = PartitionSpec {
                 keys: partition_values
                     .map(|p| p.table)
                     .unwrap_or_else(|| Table::empty(None).unwrap()),
             };
+            let statistics = stats
+                .map(|s| TableStatistics::from_stats_table(&s.table))
+                .transpose()?;
             let data_source = DataFileSource::CatalogDataFile {
                 path: file,
                 chunk_spec: None,
@@ -292,7 +351,7 @@ pub mod pylib {
                     length: num_rows as usize,
                 },
                 partition_spec: pspec,
-                statistics: None,
+                statistics,
             };
 
             let scan_task = ScanTask::new(
@@ -305,6 +364,84 @@ pub mod pylib {
             Ok(Some(PyScanTask(scan_task.into())))
         }
 
+        #[allow(clippy::too_many_arguments)]
+        #[staticmethod]
+        pub fn sql_scan_task(
+            url: String,
+            file_format: PyFileFormatConfig,
+            schema: PySchema,
+            storage_config: PyStorageConfig,
+            num_rows: Option<i64>,
+            size_bytes: Option<u64>,
+            pushdowns: Option<PyPushdowns>,
+            stats: Option<PyTable>,
+        ) -> PyResult<Self> {
+            let statistics = stats
+                .map(|s| TableStatistics::from_stats_table(&s.table))
+                .transpose()?;
+            let data_source = DataFileSource::DatabaseDataSource {
+                path: url,
+                chunk_spec: None,
+                size_bytes,
+                metadata: num_rows.map(|n| TableMetadata { length: n as usize }),
+                partition_spec: None,
+                statistics,
+            };
+
+            let scan_task = ScanTask::new(
+                vec![data_source],
+                file_format.into(),
+                schema.schema,
+                storage_config.into(),
+                pushdowns.map(|p| p.0.as_ref().clone()).unwrap_or_default(),
+            );
+            Ok(PyScanTask(scan_task.into()))
+        }
+
+        #[allow(clippy::too_many_arguments)]
+        #[staticmethod]
+        pub fn python_factory_func_scan_task(
+            py: Python,
+            module: String,
+            func_name: String,
+            func_args: Vec<&PyAny>,
+            schema: PySchema,
+            num_rows: Option<i64>,
+            size_bytes: Option<u64>,
+            pushdowns: Option<PyPushdowns>,
+            stats: Option<PyTable>,
+        ) -> PyResult<Self> {
+            let statistics = stats
+                .map(|s| TableStatistics::from_stats_table(&s.table))
+                .transpose()?;
+            let data_source = DataFileSource::PythonFactoryFunction {
+                module,
+                func_name,
+                func_args: PythonTablesFactoryArgs::new(
+                    func_args.iter().map(|pyany| pyany.into_py(py)).collect(),
+                ),
+                size_bytes,
+                metadata: num_rows.map(|num_rows| TableMetadata {
+                    length: num_rows as usize,
+                }),
+                statistics,
+                partition_spec: None,
+            };
+
+            let scan_task = ScanTask::new(
+                vec![data_source],
+                Arc::new(FileFormatConfig::PythonFunction),
+                schema.schema,
+                // HACK: StorageConfig isn't used when running the Python function but this is a non-optional arg for
+                // ScanTask creation, so we just put in a placeholder here
+                Arc::new(crate::storage_config::StorageConfig::Python(Arc::new(
+                    PythonStorageConfig { io_config: None },
+                ))),
+                pushdowns.map(|p| p.0.as_ref().clone()).unwrap_or_default(),
+            );
+            Ok(PyScanTask(scan_task.into()))
+        }
+
         pub fn __repr__(&self) -> PyResult<String> {
             Ok(format!("{:?}", self.0))
         }
@@ -420,22 +557,28 @@ pub mod pylib {
 
         #[getter]
         pub fn filters(&self) -> Option<PyExpr> {
-            self.0.filters.as_ref().map(|e| PyExpr {
-                expr: e.as_ref().clone(),
-            })
+            self.0.filters.as_ref().map(|e| PyExpr { expr: e.clone() })
         }
 
         #[getter]
         pub fn partition_filters(&self) -> Option<PyExpr> {
-            self.0.partition_filters.as_ref().map(|e| PyExpr {
-                expr: e.as_ref().clone(),
-            })
+            self.0
+                .partition_filters
+                .as_ref()
+                .map(|e| PyExpr { expr: e.clone() })
         }
 
         #[getter]
         pub fn columns(&self) -> Option<Vec<String>> {
             self.0.columns.as_deref().cloned()
         }
+
+        pub fn filter_required_column_names(&self) -> Option<Vec<String>> {
+            self.0
+                .filters
+                .as_ref()
+                .map(daft_dsl::optimization::get_required_columns)
+        }
     }
 }
 
