@@ -1,4 +1,7 @@
+use std::collections::HashMap;
+
 use daft_core::array::ops::DaftCompare;
+use daft_dsl::{ExprRef, Literal};
 use daft_table::Table;
 
 #[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
@@ -6,6 +9,28 @@ pub struct PartitionSpec {
     pub keys: Table,
 }
 
+impl PartitionSpec {
+    pub fn multiline_display(&self) -> Vec<String> {
+        let mut res = vec![];
+        res.push(format!("Keys = {}", self.keys));
+        res
+    }
+
+    pub fn to_fill_map(&self) -> HashMap<&str, ExprRef> {
+        self.keys
+            .schema
+            .fields
+            .iter()
+            .map(|(col, _)| {
+                (
+                    col.as_str(),
+                    self.keys.get_column(col).unwrap().clone().lit(),
+                )
+            })
+            .collect()
+    }
+}
+
 impl PartialEq for PartitionSpec {
     fn eq(&self, other: &Self) -> bool {
         // If the names of fields or types of fields don't match, return False
