@@ -1,7 +1,8 @@
 #![feature(hash_raw_entry)]
+#![feature(let_chains)]
 
 use std::borrow::Cow;
-use std::collections::HashSet;
+use std::collections::{HashMap, HashSet};
 use std::fmt::{Display, Formatter, Result};
 
 use daft_core::array::ops::full::FullNull;
@@ -16,7 +17,7 @@ use daft_core::schema::{Schema, SchemaRef};
 use daft_core::series::{IntoSeries, Series};
 
 use daft_dsl::functions::FunctionEvaluator;
-use daft_dsl::{col, null_lit, AggExpr, Expr};
+use daft_dsl::{col, null_lit, AggExpr, ApproxPercentileParams, Expr, ExprRef, LiteralValue};
 #[cfg(feature = "python")]
 pub mod ffi;
 mod ops;
@@ -173,6 +174,20 @@ impl Table {
         }
     }
 
+    pub fn add_monotonically_increasing_id(
+        &self,
+        partition_num: u64,
+        offset: u64,
+        column_name: &str,
+    ) -> DaftResult<Self> {
+        // Use the leftmost 28 bits for the partition number and the rightmost 36 bits for the row number
+        let start = (partition_num << 36) + offset;
+        let end = start + self.len() as u64;
+        let ids = (start..end).step_by(1).collect::<Vec<_>>();
+        let id_series = UInt64Array::from((column_name, ids)).into_series();
+        Self::from_columns([&[id_series], &self.columns[..]].concat())
+    }
+
     pub fn quantiles(&self, num: usize) -> DaftResult<Self> {
         if self.is_empty() {
             return Ok(self.clone());
@@ -204,21 +219,21 @@ impl Table {
         Ok(column_sizes?.iter().sum())
     }
 
-    pub fn filter<E: AsRef<Expr>>(&self, predicate: &[E]) -> DaftResult<Self> {
+    pub fn filter(&self, predicate: &[ExprRef]) -> DaftResult<Self> {
         if predicate.is_empty() {
             Ok(self.clone())
         } else if predicate.len() == 1 {
-            let mask = self.eval_expression(predicate.get(0).unwrap().as_ref())?;
+            let mask = self.eval_expression(predicate.first().unwrap().as_ref())?;
             self.mask_filter(&mask)
         } else {
             let mut expr = predicate
-                .get(0)
+                .first()
                 .unwrap()
-                .as_ref()
-                .and(predicate.get(1).unwrap().as_ref());
+                .clone()
+                .and(predicate.get(1).unwrap().clone());
             for i in 2..predicate.len() {
-                let next = predicate.get(i).unwrap().as_ref();
-                expr = expr.and(next);
+                let next = predicate.get(i).unwrap();
+                expr = expr.and(next.clone());
             }
             let mask = self.eval_expression(&expr)?;
             self.mask_filter(&mask)
@@ -307,11 +322,28 @@ impl Table {
         match agg_expr {
             Count(expr, mode) => Series::count(&self.eval_expression(expr)?, groups, *mode),
             Sum(expr) => Series::sum(&self.eval_expression(expr)?, groups),
+            ApproxSketch(expr) => Series::approx_sketch(&self.eval_expression(expr)?, groups),
+            ApproxPercentile(ApproxPercentileParams {
+                child: expr,
+                percentiles,
+                force_list_output,
+            }) => {
+                let percentiles = percentiles.iter().map(|p| p.0).collect::<Vec<f64>>();
+                Series::approx_sketch(&self.eval_expression(expr)?, groups)?
+                    .sketch_percentile(&percentiles, *force_list_output)
+            }
+            MergeSketch(expr) => Series::merge_sketch(&self.eval_expression(expr)?, groups),
             Mean(expr) => Series::mean(&self.eval_expression(expr)?, groups),
             Min(expr) => Series::min(&self.eval_expression(expr)?, groups),
             Max(expr) => Series::max(&self.eval_expression(expr)?, groups),
+            AnyValue(expr, ignore_nulls) => {
+                Series::any_value(&self.eval_expression(expr)?, groups, *ignore_nulls)
+            }
             List(expr) => Series::agg_list(&self.eval_expression(expr)?, groups),
             Concat(expr) => Series::agg_concat(&self.eval_expression(expr)?, groups),
+            MapGroups { .. } => Err(DaftError::ValueError(
+                "MapGroups not supported via aggregation, use map_groups instead".to_string(),
+            )),
         }
     }
 
@@ -326,6 +358,13 @@ impl Table {
             Not(child) => !(self.eval_expression(child)?),
             IsNull(child) => self.eval_expression(child)?.is_null(),
             NotNull(child) => self.eval_expression(child)?.not_null(),
+            FillNull(child, fill_value) => {
+                let fill_value = self.eval_expression(fill_value)?;
+                self.eval_expression(child)?.fill_null(&fill_value)
+            }
+            IsIn(child, items) => self
+                .eval_expression(child)?
+                .is_in(&self.eval_expression(items)?),
             BinaryOp { op, left, right } => {
                 let lhs = self.eval_expression(left)?;
                 let rhs = self.eval_expression(right)?;
@@ -354,19 +393,25 @@ impl Table {
                     .iter()
                     .map(|e| self.eval_expression(e))
                     .collect::<DaftResult<Vec<_>>>()?;
-                func.evaluate(evaluated_inputs.as_slice(), expr)
+                func.evaluate(evaluated_inputs.as_slice(), func)
             }
             Literal(lit_value) => Ok(lit_value.to_series()),
             IfElse {
                 if_true,
                 if_false,
                 predicate,
-            } => {
-                let if_true_series = self.eval_expression(if_true)?;
-                let if_false_series = self.eval_expression(if_false)?;
-                let predicate_series = self.eval_expression(predicate)?;
-                Ok(if_true_series.if_else(&if_false_series, &predicate_series)?)
-            }
+            } => match predicate.as_ref() {
+                Expr::Literal(LiteralValue::Boolean(true)) => self.eval_expression(if_true),
+                Expr::Literal(LiteralValue::Boolean(false)) => {
+                    Ok(self.eval_expression(if_false)?.rename(if_true.name()))
+                }
+                _ => {
+                    let if_true_series = self.eval_expression(if_true)?;
+                    let if_false_series = self.eval_expression(if_false)?;
+                    let predicate_series = self.eval_expression(predicate)?;
+                    Ok(if_true_series.if_else(&if_false_series, &predicate_series)?)
+                }
+            },
         }?;
         if expected_field.name != series.field().name {
             return Err(DaftError::ComputeError(format!(
@@ -381,7 +426,7 @@ impl Table {
         Ok(series)
     }
 
-    pub fn eval_expression_list(&self, exprs: &[Expr]) -> DaftResult<Self> {
+    pub fn eval_expression_list(&self, exprs: &[ExprRef]) -> DaftResult<Self> {
         let result_series = exprs
             .iter()
             .map(|e| self.eval_expression(e))
@@ -410,7 +455,16 @@ impl Table {
     }
 
     pub fn cast_to_schema(&self, schema: &Schema) -> DaftResult<Self> {
+        self.cast_to_schema_with_fill(schema, None)
+    }
+
+    pub fn cast_to_schema_with_fill(
+        &self,
+        schema: &Schema,
+        fill_map: Option<&HashMap<&str, ExprRef>>,
+    ) -> DaftResult<Self> {
         let current_col_names = HashSet::<_>::from_iter(self.column_names());
+        let null_lit = null_lit();
         let exprs: Vec<_> = schema
             .fields
             .iter()
@@ -419,8 +473,15 @@ impl Table {
                     // For any fields already in the table, perform a cast
                     col(name.clone()).cast(&field.dtype)
                 } else {
-                    // For any fields in schema that are not in self.schema, create all-null arrays
-                    null_lit().alias(name.clone()).cast(&field.dtype)
+                    // For any fields in schema that are not in self.schema, use fill map to fill with an expression.
+                    // If no entry for column name, fall back to null literal (i.e. create a null array for that column).
+                    fill_map
+                        .as_ref()
+                        .and_then(|m| m.get(name.as_str()))
+                        .unwrap_or(&null_lit)
+                        .clone()
+                        .alias(name.clone())
+                        .cast(&field.dtype)
                 }
             })
             .collect();
@@ -436,7 +497,9 @@ impl Table {
         res.push_str("<thead><tr>");
 
         for (name, field) in &self.schema.fields {
-            res.push_str("<th style=\"text-wrap: nowrap; max-width:192px; overflow:auto\">");
+            res.push_str(
+                "<th style=\"text-wrap: nowrap; max-width:192px; overflow:auto; text-align:left\">",
+            );
             res.push_str(&html_escape::encode_text(name));
             res.push_str("<br />");
             res.push_str(&html_escape::encode_text(&format!("{}", field.dtype)));
@@ -551,12 +614,12 @@ mod test {
             b.field().clone().rename("b"),
         ])?;
         let table = Table::new(schema, vec![a, b])?;
-        let e1 = col("a") + col("b");
+        let e1 = col("a").add(col("b"));
         let result = table.eval_expression(&e1)?;
         assert_eq!(*result.data_type(), DataType::Float64);
         assert_eq!(result.len(), 3);
 
-        let e2 = (col("a") + col("b")).cast(&DataType::Int64);
+        let e2 = col("a").add(col("b")).cast(&DataType::Int64);
         let result = table.eval_expression(&e2)?;
         assert_eq!(*result.data_type(), DataType::Int64);
         assert_eq!(result.len(), 3);
