@@ -1,3 +1,4 @@
+use daft_core::JoinType;
 use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
 use pyo3::types::PyDict;
@@ -33,7 +34,7 @@ impl PyTable {
     }
 
     pub fn eval_expression_list(&self, py: Python, exprs: Vec<PyExpr>) -> PyResult<Self> {
-        let converted_exprs: Vec<daft_dsl::Expr> = exprs.into_iter().map(|e| e.into()).collect();
+        let converted_exprs: Vec<daft_dsl::ExprRef> = exprs.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
                 .table
@@ -47,7 +48,7 @@ impl PyTable {
     }
 
     pub fn filter(&self, py: Python, exprs: Vec<PyExpr>) -> PyResult<Self> {
-        let converted_exprs: Vec<daft_dsl::Expr> = exprs.into_iter().map(|e| e.into()).collect();
+        let converted_exprs: Vec<daft_dsl::ExprRef> = exprs.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| Ok(self.table.filter(converted_exprs.as_slice())?.into()))
     }
 
@@ -57,7 +58,7 @@ impl PyTable {
         sort_keys: Vec<PyExpr>,
         descending: Vec<bool>,
     ) -> PyResult<Self> {
-        let converted_exprs: Vec<daft_dsl::Expr> =
+        let converted_exprs: Vec<daft_dsl::ExprRef> =
             sort_keys.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
@@ -73,7 +74,7 @@ impl PyTable {
         sort_keys: Vec<PyExpr>,
         descending: Vec<bool>,
     ) -> PyResult<PySeries> {
-        let converted_exprs: Vec<daft_dsl::Expr> =
+        let converted_exprs: Vec<daft_dsl::ExprRef> =
             sort_keys.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
@@ -84,8 +85,9 @@ impl PyTable {
     }
 
     pub fn agg(&self, py: Python, to_agg: Vec<PyExpr>, group_by: Vec<PyExpr>) -> PyResult<Self> {
-        let converted_to_agg: Vec<daft_dsl::Expr> = to_agg.into_iter().map(|e| e.into()).collect();
-        let converted_group_by: Vec<daft_dsl::Expr> =
+        let converted_to_agg: Vec<daft_dsl::ExprRef> =
+            to_agg.into_iter().map(|e| e.into()).collect();
+        let converted_group_by: Vec<daft_dsl::ExprRef> =
             group_by.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
@@ -95,25 +97,79 @@ impl PyTable {
         })
     }
 
-    pub fn join(
+    pub fn pivot(
+        &self,
+        py: Python,
+        group_by: Vec<PyExpr>,
+        pivot_col: PyExpr,
+        values_col: PyExpr,
+        names: Vec<String>,
+    ) -> PyResult<Self> {
+        let converted_group_by: Vec<daft_dsl::ExprRef> =
+            group_by.into_iter().map(|e| e.into()).collect();
+        let converted_pivot_col: daft_dsl::ExprRef = pivot_col.into();
+        let converted_values_col: daft_dsl::ExprRef = values_col.into();
+        py.allow_threads(|| {
+            Ok(self
+                .table
+                .pivot(
+                    converted_group_by.as_slice(),
+                    converted_pivot_col,
+                    converted_values_col,
+                    names,
+                )?
+                .into())
+        })
+    }
+
+    pub fn hash_join(
         &self,
         py: Python,
         right: &Self,
         left_on: Vec<PyExpr>,
         right_on: Vec<PyExpr>,
+        how: JoinType,
     ) -> PyResult<Self> {
-        let left_exprs: Vec<daft_dsl::Expr> = left_on.into_iter().map(|e| e.into()).collect();
-        let right_exprs: Vec<daft_dsl::Expr> = right_on.into_iter().map(|e| e.into()).collect();
+        let left_exprs: Vec<daft_dsl::ExprRef> = left_on.into_iter().map(|e| e.into()).collect();
+        let right_exprs: Vec<daft_dsl::ExprRef> = right_on.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
                 .table
-                .join(&right.table, left_exprs.as_slice(), right_exprs.as_slice())?
+                .hash_join(
+                    &right.table,
+                    left_exprs.as_slice(),
+                    right_exprs.as_slice(),
+                    how,
+                )?
+                .into())
+        })
+    }
+
+    pub fn sort_merge_join(
+        &self,
+        py: Python,
+        right: &Self,
+        left_on: Vec<PyExpr>,
+        right_on: Vec<PyExpr>,
+        is_sorted: bool,
+    ) -> PyResult<Self> {
+        let left_exprs: Vec<daft_dsl::ExprRef> = left_on.into_iter().map(|e| e.into()).collect();
+        let right_exprs: Vec<daft_dsl::ExprRef> = right_on.into_iter().map(|e| e.into()).collect();
+        py.allow_threads(|| {
+            Ok(self
+                .table
+                .sort_merge_join(
+                    &right.table,
+                    left_exprs.as_slice(),
+                    right_exprs.as_slice(),
+                    is_sorted,
+                )?
                 .into())
         })
     }
 
     pub fn explode(&self, py: Python, to_explode: Vec<PyExpr>) -> PyResult<Self> {
-        let converted_to_explode: Vec<daft_dsl::Expr> =
+        let converted_to_explode: Vec<daft_dsl::ExprRef> =
             to_explode.into_iter().map(|e| e.expr).collect();
 
         py.allow_threads(|| Ok(self.table.explode(converted_to_explode.as_slice())?.into()))
@@ -203,7 +259,7 @@ impl PyTable {
                 "Can not partition into negative number of partitions: {num_partitions}"
             )));
         }
-        let exprs: Vec<daft_dsl::Expr> = exprs.into_iter().map(|e| e.into()).collect();
+        let exprs: Vec<daft_dsl::ExprRef> = exprs.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
                 .table
@@ -248,7 +304,7 @@ impl PyTable {
         boundaries: &Self,
         descending: Vec<bool>,
     ) -> PyResult<Vec<Self>> {
-        let exprs: Vec<daft_dsl::Expr> = partition_keys.into_iter().map(|e| e.into()).collect();
+        let exprs: Vec<daft_dsl::ExprRef> = partition_keys.into_iter().map(|e| e.into()).collect();
         py.allow_threads(|| {
             Ok(self
                 .table
@@ -259,6 +315,37 @@ impl PyTable {
         })
     }
 
+    pub fn partition_by_value(
+        &self,
+        py: Python,
+        partition_keys: Vec<PyExpr>,
+    ) -> PyResult<(Vec<Self>, Self)> {
+        let exprs: Vec<daft_dsl::ExprRef> = partition_keys.into_iter().map(|e| e.into()).collect();
+        py.allow_threads(|| {
+            let (tables, values) = self.table.partition_by_value(exprs.as_slice())?;
+            let pytables = tables
+                .into_iter()
+                .map(|t| t.into())
+                .collect::<Vec<PyTable>>();
+            let values = values.into();
+            Ok((pytables, values))
+        })
+    }
+
+    pub fn add_monotonically_increasing_id(
+        &self,
+        py: Python,
+        partition_num: u64,
+        column_name: &str,
+    ) -> PyResult<Self> {
+        py.allow_threads(|| {
+            Ok(self
+                .table
+                .add_monotonically_increasing_id(partition_num, 0, column_name)?
+                .into())
+        })
+    }
+
     pub fn __len__(&self) -> PyResult<usize> {
         Ok(self.table.len())
     }
