@@ -20,7 +20,7 @@ from daft.api_annotations import APITypeError
 from daft.dataframe import DataFrame
 from daft.datatype import DataType
 from daft.utils import pyarrow_supports_fixed_shape_tensor
-from tests.conftest import *
+from tests.conftest import UuidType
 
 ARROW_VERSION = tuple(int(s) for s in pa.__version__.split(".") if s.isnumeric())
 
@@ -84,6 +84,16 @@ def test_create_dataframe_empty_list() -> None:
 
 def test_create_dataframe_list(valid_data: list[dict[str, float]]) -> None:
     df = daft.from_pylist(valid_data)
+    assert len(df) == len(valid_data)
+    assert len(df._preview.preview_partition) == len(valid_data)
+    assert set(df.column_names) == set(COL_NAMES)
+
+
+def test_create_dataframe_list_data_longer_than_preview(valid_data: list[dict[str, float]]) -> None:
+    valid_data = valid_data * 3
+    df = daft.from_pylist(valid_data)
+    assert len(df) == len(valid_data)
+    assert len(df._preview.preview_partition) == 8
     assert set(df.column_names) == set(COL_NAMES)
 
 
@@ -126,6 +136,17 @@ def test_create_dataframe_list_non_dicts() -> None:
 def test_create_dataframe_pydict(valid_data: list[dict[str, float]]) -> None:
     pydict = {k: [item[k] for item in valid_data] for k in valid_data[0].keys()}
     df = daft.from_pydict(pydict)
+    assert len(df) == len(valid_data)
+    assert len(df._preview.preview_partition) == len(valid_data)
+    assert set(df.column_names) == set(COL_NAMES)
+
+
+def test_create_dataframe_pydict_data_longer_than_preview(valid_data: list[dict[str, float]]) -> None:
+    valid_data = valid_data * 3
+    pydict = {k: [item[k] for item in valid_data] for k in valid_data[0].keys()}
+    df = daft.from_pydict(pydict)
+    assert len(df) == len(valid_data)
+    assert len(df._preview.preview_partition) == 8
     assert set(df.column_names) == set(COL_NAMES)
 
 
@@ -159,6 +180,8 @@ def test_create_dataframe_arrow(valid_data: list[dict[str, float]], multiple) ->
     df = daft.from_arrow(t)
     if multiple:
         t = pa.concat_tables(t)
+    assert len(df) == len(t)
+    assert len(df._preview.preview_partition) == (8 if multiple else len(t))
     assert set(df.column_names) == set(t.column_names)
     casted_field = t.schema.field("variety").with_type(pa.large_string())
     expected = t.cast(t.schema.set(t.schema.get_field_index("variety"), casted_field))
@@ -270,6 +293,8 @@ def test_create_dataframe_pandas(valid_data: list[dict[str, float]], multiple) -
     df = daft.from_pandas(pd_df)
     if multiple:
         pd_df = pd.concat(pd_df).reset_index(drop=True)
+    assert len(df) == len(pd_df)
+    assert len(df._preview.preview_partition) == (8 if multiple else len(pd_df))
     assert set(df.column_names) == set(pd_df.columns)
     # Check roundtrip.
     pd.testing.assert_frame_equal(df.to_pandas(), pd_df)
@@ -325,6 +350,27 @@ def test_create_dataframe_pandas_tensor(valid_data: list[dict[str, float]]) -> N
             id="numpy_1d_arrays",
         ),
         pytest.param(pa.array([[1, 2, 3], [1, 2], [1]]), DataType.list(DataType.int64()), id="pa_nested"),
+        pytest.param(
+            pa.array([[("a", 1), ("b", 2)], [("c", 3), ("d", 4)]], type=pa.map_(pa.string(), pa.int64())),
+            DataType.map(DataType.string(), DataType.int64()),
+            id="pa_map",
+        ),
+        # TODO(Colin): Enable this test once cast_array_for_daft_if_needed in src/daft-core/src/utils/arrow.rs supports nested maps
+        # pytest.param(
+        #     pa.array(
+        #         [{"a": {"b": 1}, "c": {"d": 2}}, {"e": {"f": 3}, "g": {"h": 4}}],
+        #         type=pa.map_(pa.string(), pa.map_(pa.string(), pa.int64())),
+        #     ),
+        #     DataType.map(
+        #         DataType.struct(
+        #             {
+        #                 "key": DataType.string(),
+        #                 "value": DataType.map(DataType.struct({"key": DataType.string(), "value": DataType.int64()})),
+        #             }
+        #         )
+        #     ),
+        #     id="pa_nested_map",
+        # ),
         pytest.param(
             pa.chunked_array([pa.array([[1, 2, 3], [1, 2], [1]])]),
             DataType.list(DataType.int64()),
@@ -399,11 +445,7 @@ def test_create_dataframe_csv_generate_headers(valid_data: list[dict[str, float]
             writer.writerows([[item[col] for col in header] for item in valid_data])
             f.flush()
 
-        cnames = (
-            [f"column_{i}" for i in range(1, 6)]
-            if use_native_downloader or os.environ.get("DAFT_MICROPARTITIONS", "1") == "1"
-            else [f"f{i}" for i in range(5)]
-        )
+        cnames = [f"column_{i}" for i in range(1, 6)]
         df = daft.read_csv(fname, has_headers=False, use_native_downloader=use_native_downloader)
         assert df.column_names == cnames
 
@@ -499,15 +541,6 @@ def test_create_dataframe_csv_specify_schema_no_headers(
             "column_5": DataType.string(),
         }
 
-        if use_native_downloader == False and os.environ.get("DAFT_MICROPARTITIONS") == "0":
-            schema_hints_for_csv_without_headers = {
-                "f0": DataType.float64(),
-                "f1": DataType.float64(),
-                "f2": DataType.float64(),
-                "f3": DataType.float64(),
-                "f4": DataType.string(),
-            }
-
         df = daft.read_csv(
             fname,
             delimiter="\t",
@@ -772,6 +805,76 @@ def test_create_dataframe_json_schema_hints_ignore_random_hint(valid_data: list[
         assert len(pd_df) == len(valid_data)
 
 
+def test_create_dataframe_json_schema_hints_two_files() -> None:
+    with create_temp_filename() as fname, create_temp_filename() as fname2:
+        with open(fname, "w") as f:
+            f.write(json.dumps({"foo": {"bar": "baz"}}))
+            f.write("\n")
+            f.flush()
+
+        with open(fname2, "w") as f:
+            f.write(json.dumps({"foo": {"bar2": "baz2"}}))
+            f.write("\n")
+            f.flush()
+
+        # Without schema hints, schema inference should not pick up bar2
+        assert daft.read_json([fname, fname2]).schema()["foo"].dtype == DataType.struct({"bar": DataType.string()})
+
+        # With schema hints, bar2 should be included
+        df = daft.read_json(
+            [fname, fname2],
+            schema_hints={"foo": DataType.struct({"bar": DataType.string(), "bar2": DataType.string()})},
+        )
+        assert df.schema()["foo"].dtype == DataType.struct({"bar": DataType.string(), "bar2": DataType.string()})
+
+        # When dataframe is materialized, the schema hints should be enforced and bar2 should be included
+        df = df.select(df["foo"].struct.get("bar2"))
+        df = df.where(df["bar2"].not_null()).collect()
+
+        assert len(df) == 1
+        assert df.to_pydict()["bar2"][0] == "baz2"
+
+
+@pytest.mark.parametrize(
+    "input,expected",
+    [
+        pytest.param(['{"foo": {}}', '{"foo": {}}'], {"foo": [{}, {}]}, id="AllEmptyObjects"),
+        pytest.param(
+            ['{"foo": {}}', '{"foo": {"bar":"baz"}}'],
+            {"foo": [{"bar": None}, {"bar": "baz"}]},
+            id="OneEmptyObject",
+        ),
+        pytest.param(
+            ['{"foo": {}}', '{"foo":null}'],
+            {"foo": [{}, None]},
+            id="EmptyObjectAndNulls",
+        ),
+        pytest.param(
+            ['{"foo": {"bar":{"baz":{}}}}', '{"foo": {"bar":{"baz":{}}}}'],
+            {"foo": [{"bar": {"baz": {}}}, {"bar": {"baz": {}}}]},
+            id="AllEmptyNestedObjects",
+        ),
+        pytest.param(
+            ['{"foo": {"bar":{"baz":{}}}}', '{"foo": {"bar":{"baz":{"foo2":"bar2"}}}}'],
+            {"foo": [{"bar": {"baz": {"foo2": None}}}, {"bar": {"baz": {"foo2": "bar2"}}}]},
+            id="OneEmptyNestedObject",
+        ),
+    ],
+)
+def test_create_dataframe_json_empty_objects(input, expected) -> None:
+    with create_temp_filename() as fname:
+        with open(fname, "w") as f:
+            for data in input:
+                f.write(data)
+                f.write("\n")
+            f.flush()
+
+        df = daft.read_json(fname)
+        pydict = df.to_pydict()
+
+        assert pydict == expected
+
+
 ###
 # Parquet tests
 ###
