@@ -4,6 +4,7 @@ import re
 
 import numpy as np
 import pandas as pd
+import pytest
 from PIL import Image
 
 import daft
@@ -14,6 +15,7 @@ SHOWING_N_ROWS_REGEX = re.compile(r".*\(Showing first (\d+) of (\d+) rows\).*")
 UNMATERIALIZED_REGEX = re.compile(r".*\(No data to display: Dataframe not materialized\).*")
 MATERIALIZED_NO_ROWS_REGEX = re.compile(r".*\(No data to display: Materialized dataframe has no rows\).*")
 TD_STYLE = 'style="text-align:left; max-width:192px; max-height:64px; overflow:auto"'
+TH_STYLE = 'style="text-wrap: nowrap; max-width:192px; overflow:auto; text-align:left"'
 
 
 def parse_str_table(
@@ -85,6 +87,16 @@ def test_empty_repr(make_df):
     assert df._repr_html_() == "<small>(No data to display: Dataframe has no columns)</small>"
 
 
+@pytest.mark.parametrize("num_preview_rows", [9, 10, None])
+def test_repr_with_non_default_preview_rows(make_df, num_preview_rows):
+    df = make_df({"A": [i for i in range(10)], "B": [i for i in range(10)]})
+    df.collect(num_preview_rows=num_preview_rows)
+    df.__repr__()
+
+    assert df._preview.dataframe_num_rows == 10
+    assert len(df._preview.preview_partition) == (num_preview_rows if num_preview_rows is not None else 10)
+
+
 def test_empty_df_repr(make_df):
     df = make_df({"A": [1, 2, 3], "B": ["a", "b", "c"]})
     df = df.where(df["A"] > 10)
@@ -93,9 +105,9 @@ def test_empty_df_repr(make_df):
     assert parse_str_table(df.__repr__(), expected_user_msg_regex=UNMATERIALIZED_REGEX) == expected_data
     assert (
         df._repr_html_()
-        == """<div>
+        == f"""<div>
 <table class="dataframe">
-<thead><tr><th style="text-wrap: nowrap; max-width:192px; overflow:auto">A<br />Int64</th><th style="text-wrap: nowrap; max-width:192px; overflow:auto">B<br />Utf8</th></tr></thead>
+<thead><tr><th {TH_STYLE}>A<br />Int64</th><th {TH_STYLE}>B<br />Utf8</th></tr></thead>
 </table>
 <small>(No data to display: Dataframe not materialized)</small>
 </div>"""
@@ -115,9 +127,9 @@ def test_empty_df_repr(make_df):
     assert parse_str_table(df.__repr__(), expected_user_msg_regex=MATERIALIZED_NO_ROWS_REGEX) == expected_data
     assert (
         df._repr_html_()
-        == """<div>
+        == f"""<div>
 <table class="dataframe">
-<thead><tr><th style="text-wrap: nowrap; max-width:192px; overflow:auto">A<br />Int64</th><th style="text-wrap: nowrap; max-width:192px; overflow:auto">B<br />Utf8</th></tr></thead>
+<thead><tr><th {TH_STYLE}>A<br />Int64</th><th {TH_STYLE}>B<br />Utf8</th></tr></thead>
 <tbody>
 </tbody>
 </table>
@@ -134,9 +146,9 @@ def test_alias_repr(make_df):
     assert parse_str_table(df.__repr__(), expected_user_msg_regex=UNMATERIALIZED_REGEX) == expected_data
     assert (
         df._repr_html_()
-        == """<div>
+        == f"""<div>
 <table class="dataframe">
-<thead><tr><th style="text-wrap: nowrap; max-width:192px; overflow:auto">A2<br />Int64</th><th style="text-wrap: nowrap; max-width:192px; overflow:auto">B<br />Utf8</th></tr></thead>
+<thead><tr><th {TH_STYLE}>A2<br />Int64</th><th {TH_STYLE}>B<br />Utf8</th></tr></thead>
 </table>
 <small>(No data to display: Dataframe not materialized)</small>
 </div>"""
@@ -154,15 +166,12 @@ def test_alias_repr(make_df):
             ["a", "b", "c"],
         ),
     }
-    expected_data_html = {
-        **expected_data,
-    }
     assert parse_str_table(df.__repr__()) == expected_data
     assert (
         df._repr_html_()
         == f"""<div>
 <table class="dataframe">
-<thead><tr><th style="text-wrap: nowrap; max-width:192px; overflow:auto">A2<br />Int64</th><th style="text-wrap: nowrap; max-width:192px; overflow:auto">B<br />Utf8</th></tr></thead>
+<thead><tr><th {TH_STYLE}>A2<br />Int64</th><th {TH_STYLE}>B<br />Utf8</th></tr></thead>
 <tbody>
 <tr><td><div {TD_STYLE}>1</div></td><td><div {TD_STYLE}>a</div></td></tr>
 <tr><td><div {TD_STYLE}>2</div></td><td><div {TD_STYLE}>b</div></td></tr>
@@ -174,24 +183,10 @@ def test_alias_repr(make_df):
     )
 
 
-def test_repr_with_unicode(make_df):
+def test_repr_with_unicode(make_df, data_source):
     df = make_df({"🔥": [1, 2, 3], "🦁": ["🔥a", "b🔥", "🦁🔥" * 60]})
-
-    expected_data = {"🔥": ("Int64", []), "🦁": ("Utf8", [])}
-    assert parse_str_table(df.__repr__(), expected_user_msg_regex=UNMATERIALIZED_REGEX) == expected_data
-    assert (
-        df._repr_html_()
-        == """<div>
-<table class="dataframe">
-<thead><tr><th style="text-wrap: nowrap; max-width:192px; overflow:auto">🔥<br />Int64</th><th style="text-wrap: nowrap; max-width:192px; overflow:auto">🦁<br />Utf8</th></tr></thead>
-</table>
-<small>(No data to display: Dataframe not materialized)</small>
-</div>"""
-    )
-
-    df.collect()
-
-    expected_data = {
+    expected_data_unmaterialized = {"🔥": ("Int64", []), "🦁": ("Utf8", [])}
+    expected_data_materialized = {
         "🔥": (
             "Int64",
             ["1", "2", "3"],
@@ -201,16 +196,17 @@ def test_repr_with_unicode(make_df):
             ["🔥a", "b🔥", "🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁…"],
         ),
     }
-    expected_data_html = {
-        **expected_data,
-    }
+
     string_array = ["🔥a", "b🔥", "🦁🔥" * 60]  # we dont truncate for html
-    assert parse_str_table(df.__repr__()) == expected_data
-    assert (
-        df._repr_html_()
-        == f"""<div>
+    expected_html_unmaterialized = f"""<div>
 <table class="dataframe">
-<thead><tr><th style="text-wrap: nowrap; max-width:192px; overflow:auto">🔥<br />Int64</th><th style="text-wrap: nowrap; max-width:192px; overflow:auto">🦁<br />Utf8</th></tr></thead>
+<thead><tr><th {TH_STYLE}>🔥<br />Int64</th><th {TH_STYLE}>🦁<br />Utf8</th></tr></thead>
+</table>
+<small>(No data to display: Dataframe not materialized)</small>
+</div>"""
+    expected_html_materialized = f"""<div>
+<table class="dataframe">
+<thead><tr><th {TH_STYLE}>🔥<br />Int64</th><th {TH_STYLE}>🦁<br />Utf8</th></tr></thead>
 <tbody>
 <tr><td><div {TD_STYLE}>1</div></td><td><div {TD_STYLE}>{string_array[0]}</div></td></tr>
 <tr><td><div {TD_STYLE}>2</div></td><td><div {TD_STYLE}>{string_array[1]}</div></td></tr>
@@ -219,12 +215,27 @@ def test_repr_with_unicode(make_df):
 </table>
 <small>(Showing first 3 of 3 rows)</small>
 </div>"""
-    )
+
+    variant = data_source
+    if variant == "parquet":
+        assert (
+            parse_str_table(df.__repr__(), expected_user_msg_regex=UNMATERIALIZED_REGEX) == expected_data_unmaterialized
+        )
+        assert df._repr_html_() == expected_html_unmaterialized
+    elif variant == "arrow":
+        assert (
+            parse_str_table(df.__repr__(), expected_user_msg_regex=SHOWING_N_ROWS_REGEX) == expected_data_materialized
+        )
+        assert df._repr_html_() == expected_html_materialized
+
+    df.collect()
+
+    assert parse_str_table(df.__repr__()) == expected_data_materialized
+    assert df._repr_html_() == expected_html_materialized
 
 
 def test_repr_with_html_string():
     df = daft.from_pydict({"A": [f"<div>body{i}</div>" for i in range(3)]})
-    df.collect()
 
     non_html_table = df.__repr__()
     html_table = df._repr_html_()
@@ -249,7 +260,6 @@ def test_repr_html_custom_hooks():
             "pil": daft.Series.from_pylist([img for _ in range(3)], pyobj="force"),
         }
     )
-    df.collect()
 
     assert (
         ANSI_ESCAPE.sub("", df.__repr__()).replace("\r", "")
@@ -285,3 +295,44 @@ def test_repr_html_custom_hooks():
 
     # Assert that numpy array viz hook correctly triggers in html repr
     assert f"<td><div {TD_STYLE}>&ltnp.ndarray<br>shape=(3, 3)<br>dtype=float64&gt</div></td><td>" in html_repr
+
+
+def test_repr_empty_struct():
+    data = {"empty_structs": [{}, {}], "nested_empty_structs": [{"a": {}}, {"b": {}}]}
+    df = daft.from_pydict(data)
+
+    expected_schema_truncated_repr = """╭───────────────┬──────────────────────────────────╮
+│ empty_structs ┆ nested_empty_structs             │
+│ ---           ┆ ---                              │
+│ Struct[]      ┆ Struct[a: Struct[], b: Struct[]] │
+╰───────────────┴──────────────────────────────────╯
+"""
+    assert df.schema()._truncated_table_string() == expected_schema_truncated_repr
+
+    expected_schema_repr = """╭──────────────────────┬──────────────────────────────────╮
+│ Column Name          ┆ Type                             │
+╞══════════════════════╪══════════════════════════════════╡
+│ empty_structs        ┆ Struct[]                         │
+├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+│ nested_empty_structs ┆ Struct[a: Struct[], b: Struct[]] │
+╰──────────────────────┴──────────────────────────────────╯
+"""
+    assert repr(df.schema()) == expected_schema_repr
+
+    expected_repr = """╭───────────────┬──────────────────────────────────╮
+│ empty_structs ┆ nested_empty_structs             │
+│ ---           ┆ ---                              │
+│ Struct[]      ┆ Struct[a: Struct[], b: Struct[]] │
+╞═══════════════╪══════════════════════════════════╡
+│ {}            ┆ {a: {},                          │
+│               ┆ b: None,                         │
+│               ┆ }                                │
+├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+│ {}            ┆ {a: None,                        │
+│               ┆ b: {},                           │
+│               ┆ }                                │
+╰───────────────┴──────────────────────────────────╯
+
+(Showing first 2 of 2 rows)"""
+
+    assert str(df) == expected_repr
