@@ -9,7 +9,7 @@ from daft.table import MicroPartition
 
 def test_expressions_projection_error_dup_name():
     with pytest.raises(ValueError):
-        ep = ExpressionsProjection(
+        ExpressionsProjection(
             [
                 col("x"),
                 col("y").alias("x"),
@@ -30,9 +30,6 @@ def test_expressions_projection_empty():
     assert ep == ep
     assert ep != ExpressionsProjection([col("x")])
 
-    # Test required_columns
-    assert ep.required_columns() == set()
-
     # Test to_name_set()
     assert ep.to_name_set() == set()
 
@@ -59,9 +56,6 @@ def test_expressions_projection():
     assert ep == ep
     assert ep != ExpressionsProjection([])
 
-    # Test required_columns
-    assert ep.required_columns() == {"x", "y", "z"}
-
     # Test to_name_set()
     assert ep.to_name_set() == {"x", "y", "a"}
 
