@@ -7,6 +7,8 @@ import pytest
 from pyarrow import parquet as pq
 
 import daft
+import daft.table
+from daft.exceptions import ConnectTimeoutError, ReadTimeoutError
 from daft.filesystem import get_filesystem, get_protocol_from_path
 from daft.table import MicroPartition, Table
 
@@ -171,9 +173,17 @@ DAFT_CAN_READ_FILES = [
         "s3a://daft-public-data/test_fixtures/parquet-dev/mvp.parquet",
     ),
     (
-        "azure/mvp",
+        "azure/mvp/az",
         "az://public-anonymous/mvp.parquet",
     ),
+    (
+        "azure/mvp/abfs",
+        "abfs://public-anonymous/mvp.parquet",
+    ),
+    (
+        "azure/mvp/abfss",
+        "abfss://public-anonymous/mvp.parquet",
+    ),
     (
         "gcs/mvp",
         "gs://daft-public-data-gs/mvp.parquet",
@@ -200,11 +210,28 @@ def parquet_file(request) -> tuple[str, str]:
     return request.param
 
 
+@pytest.fixture(params=[(True, True), (True, False), (False, False)], ids=["split", "merge", "ignore"])
+def set_split_config(request):
+    max_size = 0 if request.param[0] else 384 * 1024 * 1024
+    min_size = 0 if request.param[1] else 96 * 1024 * 1024
+
+    old_execution_config = daft.context.get_context().daft_execution_config
+
+    try:
+        daft.set_execution_config(
+            scan_tasks_max_size_bytes=max_size,
+            scan_tasks_min_size_bytes=min_size,
+        )
+        yield
+    finally:
+        daft.set_execution_config(old_execution_config)
+
+
 def read_parquet_with_pyarrow(path) -> pa.Table:
     kwargs = {}
     if get_protocol_from_path(path) == "s3" or get_protocol_from_path(path) == "s3a":
         kwargs["anon"] = True
-    if get_protocol_from_path(path) == "az":
+    if get_protocol_from_path(path) in ("az", "abfs", "abfss"):
         kwargs["account_name"] = "dafttestdata"
         kwargs["anon"] = True
 
@@ -287,7 +314,7 @@ def test_parquet_into_pyarrow_bulk(parquet_file, public_storage_io_config, multi
 
 
 @pytest.mark.integration()
-def test_parquet_read_df(parquet_file, public_storage_io_config):
+def test_parquet_read_df(parquet_file, public_storage_io_config, set_split_config):
     _, url = parquet_file
     daft_native_read = daft.read_parquet(url, io_config=public_storage_io_config)
     pa_read = MicroPartition.from_arrow(read_parquet_with_pyarrow(url))
@@ -377,3 +404,61 @@ def test_row_groups_selection_into_pyarrow_bulk(public_storage_io_config, multit
     for i, t in enumerate(rest):
         assert len(t) == 10
         assert first[i * 10 : (i + 1) * 10] == t
+
+
+@pytest.mark.integration()
+@pytest.mark.parametrize(
+    "multithreaded_io",
+    [False, True],
+)
+def test_connect_timeout(multithreaded_io):
+    url = "s3://daft-public-data/test_fixtures/parquet-dev/mvp.parquet"
+    connect_timeout_config = daft.io.IOConfig(
+        s3=daft.io.S3Config(
+            # NOTE: no keys or endpoints specified for an AWS public s3 bucket
+            region_name="us-west-2",
+            anonymous=True,
+            connect_timeout_ms=1,
+            num_tries=3,
+        )
+    )
+
+    with pytest.raises((ReadTimeoutError, ConnectTimeoutError), match=f"timed out when trying to connect to {url}"):
+        MicroPartition.read_parquet(url, io_config=connect_timeout_config, multithreaded_io=multithreaded_io).to_arrow()
+
+
+@pytest.mark.integration()
+@pytest.mark.parametrize(
+    "multithreaded_io",
+    [False, True],
+)
+def test_read_timeout(multithreaded_io):
+    url = "s3://daft-public-data/test_fixtures/parquet-dev/mvp.parquet"
+    read_timeout_config = daft.io.IOConfig(
+        s3=daft.io.S3Config(
+            # NOTE: no keys or endpoints specified for an AWS public s3 bucket
+            region_name="us-west-2",
+            anonymous=True,
+            read_timeout_ms=1,
+            num_tries=3,
+        )
+    )
+
+    with pytest.raises((ReadTimeoutError, ConnectTimeoutError), match=f"timed out when trying to connect to {url}"):
+        MicroPartition.read_parquet(url, io_config=read_timeout_config, multithreaded_io=multithreaded_io).to_arrow()
+
+
+@pytest.mark.integration()
+def test_read_file_level_timeout():
+    url = "s3://daft-public-data/test_fixtures/parquet-dev/mvp.parquet"
+    read_timeout_config = daft.io.IOConfig(
+        s3=daft.io.S3Config(
+            # NOTE: no keys or endpoints specified for an AWS public s3 bucket
+            region_name="us-west-2",
+            anonymous=True,
+            num_tries=3,
+        )
+    )
+
+    with pytest.raises((ReadTimeoutError), match=f"Parquet reader timed out while trying to read: {url}"):
+        daft.table.read_parquet_into_pyarrow(url, io_config=read_timeout_config, file_timeout_ms=2)
