@@ -588,10 +588,10 @@ def test_series_cast_timestamp(input_t, input, output_t, output) -> None:
 @pytest.mark.parametrize(
     ["timezone", "expected_dt", "tz_suffix"],
     [
-        (None, "1970-01-01T00:00", ""),
-        ("UTC", "1970-01-01T00:00", "+00:00"),
-        ("-04:00", "1969-12-31T20:00", "-04:00"),
-        ("America/Los_Angeles", "1969-12-31T16:00", "-08:00"),
+        (None, "1970-01-01 00:00", ""),
+        ("UTC", "1970-01-01 00:00", " +00:00"),
+        ("-04:00", "1969-12-31 20:00", " -04:00"),
+        ("America/Los_Angeles", "1969-12-31 16:00", " PST"),
     ],
 )
 def test_series_cast_timestamp_string(timeunit, sec_str, timezone, expected_dt, tz_suffix) -> None:
@@ -773,3 +773,21 @@ def test_cast_date_to_timestamp():
 
     back = casted.dt.date()
     assert (input == back).to_pylist() == [True]
+
+
+@pytest.mark.parametrize("timeunit", ["us", "ns"])
+def test_cast_timestamp_to_time(timeunit):
+    from datetime import datetime, time
+
+    input = Series.from_pylist([datetime(2022, 1, 6, 12, 34, 56, 78)])
+    casted = input.cast(DataType.time(timeunit))
+    assert casted.to_pylist() == [time(12, 34, 56, 78)]
+
+
+@pytest.mark.parametrize("timeunit", ["s", "ms"])
+def test_cast_timestamp_to_time_unsupported_timeunit(timeunit):
+    from datetime import datetime
+
+    input = Series.from_pylist([datetime(2022, 1, 6, 12, 34, 56, 78)])
+    with pytest.raises(ValueError):
+        input.cast(DataType.time(timeunit))
