@@ -10,7 +10,7 @@ from ray.data.extensions import ArrowTensorArray
 from daft import DataType, Series
 from daft.context import get_context
 from daft.utils import pyarrow_supports_fixed_shape_tensor
-from tests.conftest import *
+from tests.conftest import UuidType
 from tests.series import ARROW_FLOAT_TYPES, ARROW_INT_TYPES, ARROW_STRING_TYPES
 
 ARROW_VERSION = tuple(int(s) for s in pa.__version__.split(".") if s.isnumeric())
@@ -87,6 +87,31 @@ def test_series_concat_list_array(chunks, fixed) -> None:
             counter += 1
 
 
+@pytest.mark.parametrize("chunks", [1, 2, 3, 10])
+def test_series_concat_map_array(chunks) -> None:
+    series = []
+    for i in range(chunks):
+        series.append(
+            Series.from_arrow(
+                pa.array(
+                    [[("a", i + j), ("b", float(i * j))] for j in range(i)],
+                    type=pa.map_(pa.string(), pa.float64()),
+                )
+            )
+        )
+
+    concated = Series.concat(series)
+
+    assert concated.datatype() == DataType.map(DataType.string(), DataType.float64())
+    concated_list = concated.to_pylist()
+    counter = 0
+    for i in range(chunks):
+        for j in range(i):
+            assert concated_list[counter][0][1] == i + j
+            assert concated_list[counter][1][1] == float(i * j)
+            counter += 1
+
+
 @pytest.mark.parametrize("chunks", [1, 2, 3, 10])
 def test_series_concat_struct_array(chunks) -> None:
     series = []
