@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import decimal
-from datetime import date, datetime
+from datetime import date, datetime, time
 
 import numpy as np
 import pytest
@@ -36,10 +36,10 @@ def test_hash_int_array_with_reference(nbytes, dtype):
 
     hashed_again = arr.hash(hashed)
 
-    for v, hv, hav in zip(arr.to_pylist(), hashed.to_pylist(), hashed_again.to_pylist()):
+    for v, hashed_value, hashed_again_value in zip(arr.to_pylist(), hashed.to_pylist(), hashed_again.to_pylist()):
         vbytes = v.to_bytes(nbytes, "little")
-        ref_value = xxhash.xxh3_64_intdigest(vbytes, seed=hv)
-        assert hav == ref_value
+        ref_value = xxhash.xxh3_64_intdigest(vbytes, seed=hashed_value)
+        assert hashed_again_value == ref_value
 
 
 def test_hash_bool_array_with_reference():
@@ -161,7 +161,7 @@ def test_murmur3_32_hash_string():
     assert hashes.to_pylist() == [1210000089, None]
 
 
-def test_murmur3_32_hash_string():
+def test_murmur3_32_hash_bytes():
     arr = Series.from_pylist([b"\x00\x01\x02\x03", None])
     assert arr.datatype() == DataType.binary()
     hashes = arr.murmur3_32()
@@ -176,6 +176,21 @@ def test_murmur3_32_hash_date():
     assert hashes.to_pylist() == [-653330422, None]
 
 
+def test_murmur3_32_hash_time():
+    arr = Series.from_pylist([time(22, 31, 8, 0), None])
+    assert arr.datatype() == DataType.time("us")
+    hashes = arr.murmur3_32()
+    assert hashes.to_pylist() == [-662762989, None]
+
+
+def test_murmur3_32_hash_time_nanoseconds():
+    arr = Series.from_pylist([time(22, 31, 8, 0), None])
+    arr = arr.cast(DataType.time("ns"))
+    assert arr.datatype() == DataType.time("ns")
+    hashes = arr.murmur3_32()
+    assert hashes.to_pylist() == [-662762989, None]
+
+
 def test_murmur3_32_hash_timestamp():
     arr = Series.from_pylist([datetime(2017, 11, 16, 22, 31, 8), None])
     hashes = arr.murmur3_32()
