@@ -231,6 +231,70 @@ def test_series_if_else_fixed_size_list(if_true, if_false, expected) -> None:
     assert result.to_pylist() == expected
 
 
+@pytest.mark.parametrize(
+    ["if_true", "if_false", "expected"],
+    [
+        # Same length, same type
+        (
+            pa.array(
+                [[("a", 1), ("b", 2)], [("b", 3), ("c", 4)], None, [("a", 5), ("c", 7)]],
+                type=pa.map_(pa.string(), pa.int64()),
+            ),
+            pa.array(
+                [[("a", 8), ("b", 9)], [("c", 10)], None, [("a", 12), ("b", 13)]],
+                type=pa.map_(pa.string(), pa.int64()),
+            ),
+            [[("a", 1), ("b", 2)], [("c", 10)], None, [("a", 5), ("c", 7)]],
+        ),
+        # TODO(Colin): Uncomment this case when StructArrays are supported.
+        # Same length, different super-castable data type
+        # (
+        #     pa.array(
+        #         [[("a", 1), ("b", 2)], [("b", 3), ("c", 4)], None, [("a", 5), ("c", 7)]],
+        #         type=pa.map_(pa.string(), pa.int64()),
+        #     ),
+        #     pa.array(
+        #         [[("a", 8), ("b", 9)], [("c", 10)], None, [("a", 12), ("b", 13)]],
+        #         type=pa.map_(pa.string(), pa.int64()),
+        #     ),
+        #     [[("a", 1), ("b", 2)], [("c", 10)], None, [("a", 5), ("c", 7)]],
+        # ),
+        # ),
+        # Broadcast left
+        (
+            pa.array([[("a", 1), ("b", 2)]], type=pa.map_(pa.string(), pa.int64())),
+            pa.array(
+                [[("a", 8), ("b", 9)], [("c", 10)], None, [("a", 12), ("b", 13)]],
+                type=pa.map_(pa.string(), pa.int64()),
+            ),
+            [[("a", 1), ("b", 2)], [("c", 10)], None, [("a", 1), ("b", 2)]],
+        ),
+        # Broadcast right
+        (
+            pa.array(
+                [[("a", 1), ("b", 2)], [("b", 3), ("c", 4)], None, [("a", 5), ("c", 7)]],
+                type=pa.map_(pa.string(), pa.int64()),
+            ),
+            pa.array([[("a", 8), ("b", 9)]], type=pa.map_(pa.string(), pa.int64())),
+            [[("a", 1), ("b", 2)], [("a", 8), ("b", 9)], None, [("a", 5), ("c", 7)]],
+        ),
+        # Broadcast both
+        (
+            pa.array([[("a", 1), ("b", 2)]], type=pa.map_(pa.string(), pa.int64())),
+            pa.array([[("a", 8), ("b", 9)]], type=pa.map_(pa.string(), pa.int64())),
+            [[("a", 1), ("b", 2)], [("a", 8), ("b", 9)], None, [("a", 1), ("b", 2)]],
+        ),
+    ],
+)
+def test_series_if_else_map(if_true, if_false, expected) -> None:
+    if_true_series = Series.from_arrow(if_true)
+    if_false_series = Series.from_arrow(if_false)
+    predicate_series = Series.from_arrow(pa.array([True, False, None, True]))
+    result = predicate_series.if_else(if_true_series, if_false_series)
+    assert result.datatype() == DataType.map(DataType.string(), DataType.int64())
+    assert result.to_pylist() == expected
+
+
 @pytest.mark.parametrize(
     ["if_true", "if_false", "expected"],
     [
