@@ -24,9 +24,11 @@ PYTHON_TYPE_ARRAYS = {
     "str": ["foo", "bar"],
     "binary": [b"foo", b"bar"],
     "date": [datetime.date.today(), datetime.date.today()],
+    "time": [datetime.time(1, 2, 3, 4), datetime.time(5, 6, 7, 8)],
     "list": [[1, 2], [3]],
     "struct": [{"a": 1, "b": 2.0}, {"b": 3.0}],
     "empty_struct": [{}, {}],
+    "nested_struct": [{"a": {"b": 1}, "c": {}}, {"a": {"b": 3}, "c": {}}],
     "null": [None, None],
     "tensor": [np.ones((2, 2), np.int64), np.ones((3, 3), np.int64)],
     # The following types are not natively supported and will be cast to Python object types.
@@ -40,9 +42,13 @@ PYTHON_INFERRED_TYPES = {
     "str": DataType.string(),
     "binary": DataType.binary(),
     "date": DataType.date(),
+    "time": DataType.time(TimeUnit.us()),
     "list": DataType.list(DataType.int64()),
     "struct": DataType.struct({"a": DataType.int64(), "b": DataType.float64()}),
     "empty_struct": DataType.struct({"": DataType.null()}),
+    "nested_struct": DataType.struct(
+        {"a": DataType.struct({"b": DataType.int64()}), "c": DataType.struct({"": DataType.null()})}
+    ),
     "null": DataType.null(),
     "tensor": DataType.tensor(DataType.int64()),
     # The following types are not natively supported and will be cast to Python object types.
@@ -61,9 +67,11 @@ ROUNDTRIP_TYPES = {
     "str": pa.large_string(),
     "binary": pa.large_binary(),
     "date": pa.date32(),
+    "time": pa.time64("us"),
     "list": pa.large_list(pa.int64()),
     "struct": pa.struct({"a": pa.int64(), "b": pa.float64()}),
-    "empty_struct": pa.struct({"": pa.null()}),
+    "empty_struct": pa.struct({}),
+    "nested_struct": pa.struct({"a": pa.struct({"b": pa.int64()}), "c": pa.struct({})}),
     "null": pa.null(),
     "tensor": PYTHON_INFERRED_TYPES["tensor"].to_arrow_dtype(),
     # The following types are not natively supported and will be cast to Python object types.
@@ -86,10 +94,26 @@ ARROW_TYPE_ARRAYS = {
     "binary": pa.array(PYTHON_TYPE_ARRAYS["binary"], pa.binary()),
     "boolean": pa.array(PYTHON_TYPE_ARRAYS["bool"], pa.bool_()),
     "date32": pa.array(PYTHON_TYPE_ARRAYS["date"], pa.date32()),
+    "date64": pa.array(PYTHON_TYPE_ARRAYS["date"], pa.date64()),
+    "time64_microseconds": pa.array(PYTHON_TYPE_ARRAYS["time"], pa.time64("us")),
+    "time64_nanoseconds": pa.array(PYTHON_TYPE_ARRAYS["time"], pa.time64("ns")),
     "list": pa.array(PYTHON_TYPE_ARRAYS["list"], pa.list_(pa.int64())),
     "fixed_size_list": pa.array([[1, 2], [3, 4]], pa.list_(pa.int64(), 2)),
+    "map": pa.array(
+        [[(1.0, 1), (2.0, 2)], [(3.0, 3), (4.0, 4)]],
+        pa.map_(pa.float32(), pa.int32()),
+    ),
     "struct": pa.array(PYTHON_TYPE_ARRAYS["struct"], pa.struct([("a", pa.int64()), ("b", pa.float64())])),
-    "empty_struct": pa.array(PYTHON_TYPE_ARRAYS["empty_struct"], pa.struct({"": pa.null()})),
+    "empty_struct": pa.array(PYTHON_TYPE_ARRAYS["empty_struct"], pa.struct([])),
+    "nested_struct": pa.array(
+        PYTHON_TYPE_ARRAYS["nested_struct"],
+        pa.struct(
+            {
+                "a": pa.struct([("b", pa.int64())]),
+                "c": pa.struct([]),
+            }
+        ),
+    ),
     "null": pa.array(PYTHON_TYPE_ARRAYS["null"], pa.null()),
     "tensor": pa.ExtensionArray.from_storage(
         ROUNDTRIP_TYPES["tensor"],
@@ -126,10 +150,15 @@ ARROW_ROUNDTRIP_TYPES = {
     "binary": pa.large_binary(),
     "boolean": pa.bool_(),
     "date32": pa.date32(),
+    "date64": pa.timestamp("ms"),
+    "time64_microseconds": pa.time64("us"),
+    "time64_nanoseconds": pa.time64("ns"),
     "list": pa.large_list(pa.int64()),
     "fixed_size_list": pa.list_(pa.int64(), 2),
+    "map": pa.map_(pa.float32(), pa.int32()),
     "struct": pa.struct([("a", pa.int64()), ("b", pa.float64())]),
-    "empty_struct": pa.struct({"": pa.null()}),
+    "empty_struct": pa.struct([]),
+    "nested_struct": pa.struct([("a", pa.struct([("b", pa.int64())])), ("c", pa.struct([]))]),
     "null": pa.null(),
     "tensor": PYTHON_INFERRED_TYPES["tensor"].to_arrow_dtype(),
     # The following types are not natively supported and will be cast to Python object types.
@@ -182,7 +211,11 @@ def test_from_pydict_arrow_roundtrip(uuid_ext_type) -> None:
     assert len(table) == 2
     assert set(table.column_names()) == set(arrow_type_arrays.keys())
     for field in table.schema():
-        assert field.dtype == DataType.from_arrow_type(arrow_type_arrays[field.name].type)
+        assert field.dtype == (
+            DataType.from_arrow_type(arrow_type_arrays[field.name].type)
+            if (field.name != "empty_struct" and field.name != "nested_struct")
+            else PYTHON_INFERRED_TYPES[field.name]  # empty structs are internally represented as {"": None}
+        )
     expected_table = pa.table(arrow_type_arrays).cast(pa.schema(arrow_roundtrip_types))
     assert table.to_arrow() == expected_table
 
@@ -194,7 +227,11 @@ def test_from_arrow_roundtrip(uuid_ext_type) -> None:
     assert len(table) == 2
     assert set(table.column_names()) == set(arrow_type_arrays.keys())
     for field in table.schema():
-        assert field.dtype == DataType.from_arrow_type(arrow_type_arrays[field.name].type)
+        assert field.dtype == (
+            DataType.from_arrow_type(arrow_type_arrays[field.name].type)
+            if (field.name != "empty_struct" and field.name != "nested_struct")
+            else PYTHON_INFERRED_TYPES[field.name]  # empty structs are internally represented as {"": None}
+        )
     expected_table = pa.table(arrow_type_arrays).cast(pa.schema(arrow_roundtrip_types))
     assert table.to_arrow() == expected_table
 
@@ -208,8 +245,10 @@ def test_from_pandas_roundtrip() -> None:
         assert field.dtype == PANDAS_INFERRED_TYPES[field.name]
     # pyarrow --> pandas will insert explicit Nones within the struct fields.
     df["struct"][1]["a"] = None
-    df["empty_struct"][0][""] = None
-    df["empty_struct"][1][""] = None
+    df["empty_struct"][0] = {}
+    df["empty_struct"][1] = {}
+    df["nested_struct"][0]["c"] = {}
+    df["nested_struct"][1]["c"] = {}
     pd.testing.assert_frame_equal(table.to_pandas(), df)
 
 
@@ -253,6 +292,17 @@ def test_from_pydict_arrow_fixed_size_list_array() -> None:
     assert daft_table.to_arrow()["a"].combine_chunks() == expected
 
 
+def test_from_pydict_arrow_map_array() -> None:
+    data = [[(1, 2.0), (3, 4.0)], None, [(5, 6.0), (7, 8.0)]]
+    arrow_arr = pa.array(data, pa.map_(pa.int64(), pa.float64()))
+    daft_table = MicroPartition.from_pydict({"a": arrow_arr})
+    assert "a" in daft_table.column_names()
+    # Perform expected Daft cast, where the inner string and int arrays are cast to large string and int arrays.
+    expected = arrow_arr.cast(pa.map_(pa.int64(), pa.float64()))
+    assert daft_table.to_arrow()["a"].combine_chunks() == expected
+    assert daft_table.to_pydict()["a"] == data
+
+
 def test_from_pydict_arrow_struct_array() -> None:
     data = [{"a": "foo", "b": "bar"}, {"b": "baz", "c": "quux"}]
     arrow_arr = pa.array(data)
@@ -310,6 +360,10 @@ def test_from_pydict_arrow_deeply_nested() -> None:
             pa.array([{"a": 1, "b": 2}, {"b": 3, "c": 4}, None, {"a": 5, "c": 6}]),
             pa.struct([("a", pa.int64()), ("b", pa.int64()), ("c", pa.int64())]),
         ),
+        (
+            pa.array([[(1, 2), (3, 4)], None, [(5, 6), (7, 8)]], pa.map_(pa.int64(), pa.int64())),
+            pa.map_(pa.int64(), pa.int64()),
+        ),
     ],
 )
 @pytest.mark.parametrize("chunked", [False, True])
@@ -336,6 +390,10 @@ def test_from_pydict_arrow_with_nulls_roundtrip(data, out_dtype, chunked) -> Non
             pa.array([{"a": 1, "b": 2}, {"b": 3, "c": 4}, {"a": 5}, {"a": 6, "c": 7}]),
             pa.struct([("a", pa.int64()), ("b", pa.int64()), ("c", pa.int64())]),
         ),
+        (
+            pa.array([[(1, 2), (3, 4)], [(5, 6)], [(7, 8)]], pa.map_(pa.int64(), pa.int64())),
+            pa.map_(pa.int64(), pa.int64()),
+        ),
         # Contains nulls.
         (pa.array([1, 2, None, 4], type=pa.int64()), pa.int64()),
         (pa.array(["a", "b", None, "d"], type=pa.string()), pa.large_string()),
@@ -346,6 +404,10 @@ def test_from_pydict_arrow_with_nulls_roundtrip(data, out_dtype, chunked) -> Non
             pa.array([{"a": 1, "b": 2}, {"b": 3, "c": 4}, None, {"a": 5, "c": 6}]),
             pa.struct([("a", pa.int64()), ("b", pa.int64()), ("c", pa.int64())]),
         ),
+        (
+            pa.array([[(1, 2), (3, 4)], None, [(7, 8)]], pa.map_(pa.int64(), pa.int64())),
+            pa.map_(pa.int64(), pa.int64()),
+        ),
     ],
 )
 @pytest.mark.parametrize("chunked", [False, True])
@@ -382,6 +444,10 @@ def test_from_pydict_series() -> None:
             pa.array([{"a": 1, "b": 2}, {"b": 3, "c": 4}, {"a": 5}, {"a": 6, "c": 7}]),
             pa.struct([("a", pa.int64()), ("b", pa.int64()), ("c", pa.int64())]),
         ),
+        (
+            pa.array([[(1, 2), (3, 4)], [(5, 6)], [(7, 8)]], pa.map_(pa.int64(), pa.int64())),
+            pa.map_(pa.int64(), pa.int64()),
+        ),
         # Contains nulls.
         (pa.array([1, 2, None, 4], type=pa.int64()), pa.int64()),
         (pa.array(["a", "b", None, "d"], type=pa.string()), pa.large_string()),
@@ -392,6 +458,10 @@ def test_from_pydict_series() -> None:
             pa.array([{"a": 1, "b": 2}, {"b": 3, "c": 4}, None, {"a": 5, "c": 6}]),
             pa.struct([("a", pa.int64()), ("b", pa.int64()), ("c", pa.int64())]),
         ),
+        (
+            pa.array([[(1, 2), (3, 4)], None, [(7, 8)]], pa.map_(pa.int64(), pa.int64())),
+            pa.map_(pa.int64(), pa.int64()),
+        ),
     ],
 )
 @pytest.mark.parametrize("slice_", list(itertools.combinations(range(4), 2)))
@@ -438,6 +508,17 @@ def test_from_arrow_struct_array() -> None:
     assert daft_table.to_arrow()["a"].combine_chunks() == expected
 
 
+def test_from_arrow_map_array() -> None:
+    data = [[(1.0, 1), (2.0, 2)], [(3.0, 3), (4.0, 4)]]
+    arrow_arr = pa.array(data, pa.map_(pa.float32(), pa.int32()))
+    daft_table = MicroPartition.from_arrow(pa.table({"a": arrow_arr}))
+    assert "a" in daft_table.column_names()
+    # Perform expected Daft cast, where the inner string and int arrays are cast to large string and int arrays.
+    expected = arrow_arr.cast(pa.map_(pa.float32(), pa.int32()))
+    assert daft_table.to_arrow()["a"].combine_chunks() == expected
+    assert daft_table.to_pydict()["a"] == data
+
+
 @pytest.mark.skipif(
     get_context().runner_config.name == "ray",
     reason="pyarrow extension types aren't supported on Ray clusters.",
