@@ -40,13 +40,16 @@ daft_string_types = [DataType.string()]
         ],
     ),
 )
-def test_table_join_single_column(dtype, data) -> None:
-    l, r, expected_pairs = data
-    left_table = MicroPartition.from_pydict({"x": l, "x_ind": list(range(len(l)))}).eval_expression_list(
+@pytest.mark.parametrize("join_impl", ["hash_join", "sort_merge_join"])
+def test_table_join_single_column(join_impl, dtype, data) -> None:
+    left, right, expected_pairs = data
+    left_table = MicroPartition.from_pydict({"x": left, "x_ind": list(range(len(left)))}).eval_expression_list(
         [col("x").cast(dtype), col("x_ind")]
     )
-    right_table = MicroPartition.from_pydict({"y": r, "y_ind": list(range(len(r)))})
-    result_table = left_table.join(right_table, left_on=[col("x")], right_on=[col("y")], how=JoinType.Inner)
+    right_table = MicroPartition.from_pydict({"y": right, "y_ind": list(range(len(right)))})
+    result_table = getattr(left_table, join_impl)(
+        right_table, left_on=[col("x")], right_on=[col("y")], how=JoinType.Inner
+    )
 
     assert result_table.column_names() == ["x", "x_ind", "y", "y_ind"]
 
@@ -61,7 +64,9 @@ def test_table_join_single_column(dtype, data) -> None:
     assert result_table.get_column("y").to_pylist() == result_r
 
     # make sure the result is the same with right table on left
-    result_table = right_table.join(left_table, right_on=[col("x")], left_on=[col("y")], how=JoinType.Inner)
+    result_table = getattr(right_table, join_impl)(
+        left_table, right_on=[col("x")], left_on=[col("y")], how=JoinType.Inner
+    )
 
     assert result_table.column_names() == ["y", "y_ind", "x", "x_ind"]
 
@@ -76,12 +81,13 @@ def test_table_join_single_column(dtype, data) -> None:
     assert result_table.get_column("y").to_pylist() == result_r
 
 
-def test_table_join_mismatch_column() -> None:
+@pytest.mark.parametrize("join_impl", ["hash_join", "sort_merge_join"])
+def test_table_join_mismatch_column(join_impl) -> None:
     left_table = MicroPartition.from_pydict({"x": [1, 2, 3, 4], "y": [2, 3, 4, 5]})
     right_table = MicroPartition.from_pydict({"a": [1, 2, 3, 4], "b": [2, 3, 4, 5]})
 
     with pytest.raises(ValueError, match="Mismatch of number of join keys"):
-        left_table.join(right_table, left_on=[col("x"), col("y")], right_on=[col("a")])
+        getattr(left_table, join_impl)(right_table, left_on=[col("x"), col("y")], right_on=[col("a")])
 
 
 @pytest.mark.parametrize(
@@ -98,7 +104,8 @@ def test_table_join_mismatch_column() -> None:
         {"x": ["banana", "apple"], "y": [3, 4]},
     ],
 )
-def test_table_join_multicolumn_empty_result(left, right) -> None:
+@pytest.mark.parametrize("join_impl", ["hash_join", "sort_merge_join"])
+def test_table_join_multicolumn_empty_result(join_impl, left, right) -> None:
     """Various multicol joins that should all produce an empty result."""
     left_table = MicroPartition.from_pydict(left).eval_expression_list(
         [col("a").cast(DataType.string()), col("b").cast(DataType.int32())]
@@ -107,11 +114,12 @@ def test_table_join_multicolumn_empty_result(left, right) -> None:
         [col("x").cast(DataType.string()), col("y").cast(DataType.int32())]
     )
 
-    result = left_table.join(right_table, left_on=[col("a"), col("b")], right_on=[col("x"), col("y")])
+    result = getattr(left_table, join_impl)(right_table, left_on=[col("a"), col("b")], right_on=[col("x"), col("y")])
     assert result.to_pydict() == {"a": [], "b": [], "x": [], "y": []}
 
 
-def test_table_join_multicolumn_nocross() -> None:
+@pytest.mark.parametrize("join_impl", ["hash_join", "sort_merge_join"])
+def test_table_join_multicolumn_nocross(join_impl) -> None:
     """A multicol join that should produce two rows and no cross product results.
 
     Input has duplicate join values and overlapping single-column values,
@@ -132,7 +140,7 @@ def test_table_join_multicolumn_nocross() -> None:
         }
     )
 
-    result = left_table.join(right_table, left_on=[col("a"), col("b")], right_on=[col("x"), col("y")])
+    result = getattr(left_table, join_impl)(right_table, left_on=[col("a"), col("b")], right_on=[col("x"), col("y")])
     assert set(utils.freeze(utils.pydict_to_rows(result.to_pydict()))) == set(
         utils.freeze(
             [
@@ -143,7 +151,8 @@ def test_table_join_multicolumn_nocross() -> None:
     )
 
 
-def test_table_join_multicolumn_cross() -> None:
+@pytest.mark.parametrize("join_impl", ["hash_join", "sort_merge_join"])
+def test_table_join_multicolumn_cross(join_impl) -> None:
     """A multicol join that should produce a cross product and a non-cross product."""
 
     left_table = MicroPartition.from_pydict(
@@ -161,7 +170,7 @@ def test_table_join_multicolumn_cross() -> None:
         }
     )
 
-    result = left_table.join(right_table, left_on=[col("a"), col("b")], right_on=[col("x"), col("y")])
+    result = getattr(left_table, join_impl)(right_table, left_on=[col("a"), col("b")], right_on=[col("x"), col("y")])
     assert set(utils.freeze(utils.pydict_to_rows(result.to_pydict()))) == set(
         utils.freeze(
             [
@@ -178,7 +187,8 @@ def test_table_join_multicolumn_cross() -> None:
     )
 
 
-def test_table_join_multicolumn_all_nulls() -> None:
+@pytest.mark.parametrize("join_impl", ["hash_join", "sort_merge_join"])
+def test_table_join_multicolumn_all_nulls(join_impl) -> None:
     left_table = MicroPartition.from_pydict(
         {
             "a": Series.from_pylist([None, None, None]).cast(DataType.int64()),
@@ -194,23 +204,25 @@ def test_table_join_multicolumn_all_nulls() -> None:
         }
     )
 
-    result = left_table.join(right_table, left_on=[col("a"), col("b")], right_on=[col("x"), col("y")])
+    result = getattr(left_table, join_impl)(right_table, left_on=[col("a"), col("b")], right_on=[col("x"), col("y")])
     assert set(utils.freeze(utils.pydict_to_rows(result.to_pydict()))) == set(utils.freeze([]))
 
 
-def test_table_join_no_columns() -> None:
+@pytest.mark.parametrize("join_impl", ["hash_join", "sort_merge_join"])
+def test_table_join_no_columns(join_impl) -> None:
     left_table = MicroPartition.from_pydict({"x": [1, 2, 3, 4], "y": [2, 3, 4, 5]})
     right_table = MicroPartition.from_pydict({"a": [1, 2, 3, 4], "b": [2, 3, 4, 5]})
 
     with pytest.raises(ValueError, match="No columns were passed in to join on"):
-        left_table.join(right_table, left_on=[], right_on=[])
+        getattr(left_table, join_impl)(right_table, left_on=[], right_on=[])
 
 
-def test_table_join_single_column_name_conflicts() -> None:
+@pytest.mark.parametrize("join_impl", ["hash_join", "sort_merge_join"])
+def test_table_join_single_column_name_conflicts(join_impl) -> None:
     left_table = MicroPartition.from_pydict({"x": [0, 1, 2, 3], "y": [2, 3, 4, 5]})
     right_table = MicroPartition.from_pydict({"x": [3, 2, 1, 0], "y": [6, 7, 8, 9]})
 
-    result_table = left_table.join(right_table, left_on=[col("x")], right_on=[col("x")])
+    result_table = getattr(left_table, join_impl)(right_table, left_on=[col("x")], right_on=[col("x")])
     assert result_table.column_names() == ["x", "y", "right.y"]
     result_sorted = result_table.sort([col("x")])
     assert result_sorted.get_column("y").to_pylist() == [2, 3, 4, 5]
@@ -218,11 +230,12 @@ def test_table_join_single_column_name_conflicts() -> None:
     assert result_sorted.get_column("right.y").to_pylist() == [9, 8, 7, 6]
 
 
-def test_table_join_single_column_name_conflicts_different_named_join() -> None:
+@pytest.mark.parametrize("join_impl", ["hash_join", "sort_merge_join"])
+def test_table_join_single_column_name_conflicts_different_named_join(join_impl) -> None:
     left_table = MicroPartition.from_pydict({"x": [0, 1, 2, 3], "y": [2, 3, 4, 5]})
     right_table = MicroPartition.from_pydict({"y": [3, 2, 1, 0], "x": [6, 7, 8, 9]})
 
-    result_table = left_table.join(right_table, left_on=[col("x")], right_on=[col("y")])
+    result_table = getattr(left_table, join_impl)(right_table, left_on=[col("x")], right_on=[col("y")])
 
     # NOTE: right.y is not dropped because it has a different name from the corresponding left
     # column it is joined on, left_table["x"]
@@ -233,11 +246,12 @@ def test_table_join_single_column_name_conflicts_different_named_join() -> None:
     assert result_sorted.get_column("right.x").to_pylist() == [9, 8, 7, 6]
 
 
-def test_table_join_single_column_name_multiple_conflicts() -> None:
+@pytest.mark.parametrize("join_impl", ["hash_join", "sort_merge_join"])
+def test_table_join_single_column_name_multiple_conflicts(join_impl) -> None:
     left_table = MicroPartition.from_pydict({"x": [0, 1, 2, 3], "y": [2, 3, 4, 5], "right.y": [6, 7, 8, 9]})
     right_table = MicroPartition.from_pydict({"x": [3, 2, 1, 0], "y": [10, 11, 12, 13]})
 
-    result_table = left_table.join(right_table, left_on=[col("x")], right_on=[col("x")])
+    result_table = getattr(left_table, join_impl)(right_table, left_on=[col("x")], right_on=[col("x")])
     assert result_table.column_names() == ["x", "y", "right.y", "right.right.y"]
     result_sorted = result_table.sort([col("x")])
     assert result_sorted.get_column("y").to_pylist() == [2, 3, 4, 5]
@@ -246,22 +260,24 @@ def test_table_join_single_column_name_multiple_conflicts() -> None:
     assert result_sorted.get_column("right.right.y").to_pylist() == [13, 12, 11, 10]
 
 
-def test_table_join_single_column_name_boolean() -> None:
+@pytest.mark.parametrize("join_impl", ["hash_join", "sort_merge_join"])
+def test_table_join_single_column_name_boolean(join_impl) -> None:
     left_table = MicroPartition.from_pydict({"x": [False, True, None], "y": [0, 1, 2]})
     right_table = MicroPartition.from_pydict({"x": [None, True, False, None], "y": [0, 1, 2, 3]})
 
-    result_table = left_table.join(right_table, left_on=[col("x")], right_on=[col("x")])
+    result_table = getattr(left_table, join_impl)(right_table, left_on=[col("x")], right_on=[col("x")])
     assert result_table.column_names() == ["x", "y", "right.y"]
     result_sorted = result_table.sort([col("x")])
     assert result_sorted.get_column("y").to_pylist() == [0, 1]
     assert result_sorted.get_column("right.y").to_pylist() == [2, 1]
 
 
-def test_table_join_single_column_name_null() -> None:
+@pytest.mark.parametrize("join_impl", ["hash_join", "sort_merge_join"])
+def test_table_join_single_column_name_null(join_impl) -> None:
     left_table = MicroPartition.from_pydict({"x": [None, None, None], "y": [0, 1, 2]})
     right_table = MicroPartition.from_pydict({"x": [None, None, None, None], "y": [0, 1, 2, 3]})
 
-    result_table = left_table.join(right_table, left_on=[col("x")], right_on=[col("x")])
+    result_table = getattr(left_table, join_impl)(right_table, left_on=[col("x")], right_on=[col("x")])
     assert result_table.column_names() == ["x", "y", "right.y"]
     result_sorted = result_table.sort([col("x")])
     assert result_sorted.get_column("y").to_pylist() == []
