@@ -48,8 +48,9 @@ def test_schema_iter():
 
 
 def test_schema_eq():
-    t1, t2 = MicroPartition.from_pydict({k: data for k, (data, _) in DATA.items()}), MicroPartition.from_pydict(
-        {k: data for k, (data, _) in DATA.items()}
+    t1, t2 = (
+        MicroPartition.from_pydict({k: data for k, (data, _) in DATA.items()}),
+        MicroPartition.from_pydict({k: data for k, (data, _) in DATA.items()}),
     )
     s1, s2 = t1.schema(), t2.schema()
     assert s1 == s2
@@ -63,9 +64,9 @@ def test_schema_to_name_set():
     assert schema.to_name_set() == set(DATA.keys())
 
 
-def test_repr():
+def test_truncated_repr():
     schema = TABLE.schema()
-    out_repr = repr(schema)
+    out_repr = schema._truncated_table_string()
     without_escape = ANSI_ESCAPE.sub("", out_repr)
     assert (
         without_escape.replace("\r", "")
@@ -78,6 +79,27 @@ def test_repr():
     )
 
 
+def test_repr():
+    schema = TABLE.schema()
+    out_repr = repr(schema)
+    without_escape = ANSI_ESCAPE.sub("", out_repr)
+    assert (
+        without_escape.replace("\r", "")
+        == """╭─────────────┬─────────╮
+│ Column Name ┆ Type    │
+╞═════════════╪═════════╡
+│ int         ┆ Int64   │
+├╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌┤
+│ float       ┆ Float64 │
+├╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌┤
+│ string      ┆ Utf8    │
+├╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌┤
+│ bool        ┆ Boolean │
+╰─────────────┴─────────╯
+"""
+    )
+
+
 def test_to_col_expr():
     schema = TABLE.schema()
     schema_col_exprs = ExpressionsProjection.from_schema(schema)
@@ -122,8 +144,9 @@ def test_field_pickling():
 
 
 def test_schema_pickling():
-    t1, t2 = MicroPartition.from_pydict({k: data for k, (data, _) in DATA.items()}), MicroPartition.from_pydict(
-        {k: data for k, (data, _) in DATA.items()}
+    t1, t2 = (
+        MicroPartition.from_pydict({k: data for k, (data, _) in DATA.items()}),
+        MicroPartition.from_pydict({k: data for k, (data, _) in DATA.items()}),
     )
 
     s1, s2 = t1.schema(), t2.schema()
@@ -144,6 +167,7 @@ def test_schema_from_pyarrow():
             "int": pa.int64(),
             "str": pa.string(),
             "list": pa.list_(pa.int64()),
+            "map": pa.map_(pa.string(), pa.int64()),
         }
     )
 
@@ -152,6 +176,7 @@ def test_schema_from_pyarrow():
             ("int", DataType.int64()),
             ("str", DataType.string()),
             ("list", DataType.list(DataType.int64())),
+            ("map", DataType.map(DataType.string(), DataType.int64())),
         ]
     )
 
